#!/usr/bin/python
# -*- coding: utf-8 -*-

#                         LACIE CONFIDENTIAL
#
# The information contained in this file is the property of LACIE.
# The user of this file shall keep information contained herein confidential
# and shall protect it in whole or in part from disclosure and dissemination
# to third parties.
# Without prior written consent of LACIE, you may not reproduce, represent,
# or download through any means, the information contained herein in any way
# or in any form for a purpose other that the use for internal evaluation,
# operation or maintenance within LACIE.
#
#          (c) LACIE, 2009 - 2012, All Rights Reserved.

import re
import sys
from utils import extract_type
from utils import get_access, RW, RO, WO, set_prefix

from utils import Object, Service, Enum


class Version(object):
    def __init__(self, string):
        self.version = int(string[1:].split('.')[0])
        self.sub_version = int(string[1:].split('.')[1])

    def is_lower(self, t_version):
        if self.version < t_version.version:
            return True
        elif self.version == t_version.version:
            if self.sub_version < t_version.sub_version:
                return True
        return False

    def equals(self, t_version):
        if self.version == t_version.version and\
            self.sub_version == t_version.sub_version:
                return True
        else:
            return False

current_version = Version("v0.0")


def find_object(config, lines, i, namespaces):
    if " extends " in lines[i]:
        first_line, parent_name = lines[i].split(" extends ")
        parent_name = parent_name.replace("{", "").strip()
    else:
        first_line  = lines[i]
        parent_name = None
    typedef, name = first_line.strip().replace("{", "").split()
    import_path = None
    if "(" in name:
        assert (")" in name), "name doesn't end with )"
        inherit = None
        if "inherits" in name:
            name, inherit = name.split("inherits")

        in_index = name.rfind("(")
        import_path = name.strip()[in_index + 1:-1]
        # Check that the import path includes a version
        match = re.match("v\d+.\d+", import_path)
        assert (match is not None), "Missing version in Object definition %s" % name
        version = Version(match.group())
        name = name[:in_index]
        identification = import_path[len(match.group()) + 1:]

    current_object = Object(name, namespaces, "%s.v%s.sv%s.%s" % (
        config.backend_path,
        version.version,
        version.sub_version,
        identification),
        parent_name
    )
    current_object.version = version
    global current_version

    if current_version.is_lower(version):
        current_version = version

    current_object.inherit = inherit

    j = i + 1
    count = 1
    comments = []
    param_comments = {}
    while j < len(lines) and count != 0:
        line = lines[j]
        line = line.strip().replace('\n', '')
        if line.startswith("#"):
            res = re.match("#(\w+): (.*)", line)
            if res is not None:
                param_comments.setdefault(res.group(1), [])
                param_comments[res.group(1)].append(res.group(2))
            else:
                comments.append("%s" % line[1:])
        elif 0 == len(line):
            pass
        else:
            count += line.count("{")
            count -= line.count("}")
            validator_value = None
            assert (line.endswith(',') or 0 == count), "no , at end of line\
 %d:%s for object %s" % (j, line, current_object.name)

            if line.endswith(','):
                line = line.strip()[:-1]
            if ":" not in line:
                assert (0 == count), "missing } for object %s" % current_object.name

            else:
                hidden = False
                if line.startswith("hidden"):
                    hidden = True
                    line = line.replace("hidden", "")
                name, type_valid = line.replace("}", "", 1).replace("{", "", 1).split(":")
                name = name.strip()
                type_valid = type_valid.strip()
                # Check for access
                if 0 != type_valid.count("["):
                    assert (type_valid.count("]") == type_valid.count("[")), \
                        "missing ] at line %s: %s\n" % (j, line)
                    end_index = type_valid.index("]")
                    access_index = type_valid.index("[")
                    access = get_access(type_valid[access_index + 1:end_index])
                    type_valid = type_valid[:access_index] + type_valid[end_index + 1:]
                else:
                    access = RW

                if "(" in type_valid:
                    assert (type_valid.count(")") == type_valid.count("(")),\
                    "Missing ) or ( at line %d" % j
                    # This is the regexp for the value. Only valid for Int and
                    # Unicode
                    start = type_valid.index("(")
                    end = type_valid.rfind(")")
                    validator_value = type_valid[start + 1:end].strip()
                    type_valid = type_valid[:start].strip()

                p_type = extract_type(type_valid)
                current_object.add_attribute(name, p_type, access, hidden, validator_value)
            #except ValueError:
            #    import traceback
            #    traceback.print_exc()

        j += 1

    for key, comment in param_comments.iteritems():
        current_object.attributes[key].comments = comment
    current_object.comments = comments
    return j, current_object


def find_enum(lines, i, namespaces):
    enum, name = lines[i].strip().replace("{", "").split()
    import_path = None
    assert (")" in name), "%s doesn't end with )" % name
    in_index = name.rfind("(")
    out_index = name.rfind(")")
    sub_type = name.strip()[in_index + 1:out_index]
    assert ("[" in name and "]" in name), "version should be definied for the enum %s" % name

    name, version_def = name.split("[")
    match = re.match("v\d+.\d+", version_def[:-1])
    assert (match is not None), "Missing version in service implementation %s" % name
    version = Version(match.group())

    global current_version
    if current_version.is_lower(version):
        current_version = version

    name = name[:in_index]

    current_enum = Enum(name, sub_type, namespaces)
    current_enum.version = version
    j = i + 1
    count = 1
    comments = ""
    while j < len(lines) and count != 0:
        line = lines[j]
        line = line.strip().replace('\n', '')
        if line.startswith("#"):
            comments += "%s\n" % line[1:]
        elif 0 == len(line):
            pass
        else:
            count += line.count("{")
            count -= line.count("}")
            assert (line.endswith(',') or 0 == count), "no , at end of line %d:%s" % (j, line)
            try:
                name, value = line.replace("}", "", 1).replace("{", "", 1).split("=")
                value = value.replace(",", "").strip()
                current_enum.add_value(name.strip(), value)
            except ValueError:
                assert (0 == count), "missing }"
        j += 1
    current_enum.comments = comments

    assert("UNKNOWN" in current_enum.values), "missing UNKNOWN enum value for type %s" % current_enum.name

    return j, current_enum


class CurrentFunction(object):
    def __init__(self, func_name, display_name, private):
        self.func_name = func_name
        self.params = ([], [], [], [], [])
        self.set_parameters = 0
        self.authentication = None
        if display_name is None:
            display_name = func_name
        self.display_name   = display_name
        self.errors         = []
        self.private        = private
        self.timeout        = None
        self.async          = False

    def add_parameter(self, element, param_type, param_name):
        move = 0
        if element == "in":
            move = 0
        elif element == "out":
            move = 1
        elif element == "errors":
            move = 2
        elif element == "search_criteria":
            move = 3
        elif element == "order_criteria":
            move = 4
        self.set_parameters = self.set_parameters | (1 << move)
        self.params[move].append((param_name, param_type))

    def add_error(self, error):
        error = error[1:].strip()
        self.errors.append(error)

    def check(self):
        found = False
        if self.params[3]:
            for (param_name, param_type) in self.params[0]:
                if param_type.name == "ListInfo":
                    found = True
                    break
        else:
            found = True

        assert found


def get_parameter(line, i):
    line = line.strip()

    try:
        if line.startswith("-"):
            type_name, param_name = line[1:].strip().split()
        else:
            assert("void" == line), "expecting - as a start indentifier: found _%s_\n line %s" % (line, i)
            return None, None
    except:
        print "Error line %s:\n %s\n" % (i, line)
        raise

    return extract_type(type_name), param_name


def find_functions(lines, start, stop):
    '''


    Expecting

    function FunctionName:
                timeout: value
                async: true
                in: - ParamType param1
                    - ParamType param2
                out: - ParamType return1
                error: - ErrorType
                       - ErrorType
                search_criteria:
                    - Integer no_access_share_id
                    - String[exclude] login
                order_criteria:
                    - Integer param1

    if void parameter for in/out/error then no special treatment to do


    @return types: types that will be imported for this service
            functions: functions to be created

    '''
    i = start
    function_list = []
    current_function = None
    current_element = None
    types = []

    comments = []
    try:
        while i < stop:
            line = lines[i].strip()
            if line.startswith("#"):
                i += 1
                assert(comments is not None), "line %d: lines %s" %\
                    (i, "".join(lines[i-1:i+2]))
                comments.append(line[1:])
                continue
            elif line.startswith("function") or line.startswith("local"):
                assert(line.endswith(":"))
                func = line.split()
                private = False
                if 3 == len(func):
                    private = True

                func_name = func[-1]
                display_name = None
                if "(" in func_name:
                    assert(func_name.endswith("):")), "Missing ) at line %d" % i
                    index = func_name.index("(")
                    display_name = func_name[index+1:-2]
                    func_name = func_name[:index]
                else:
                    func_name = func_name[:-1]
                if current_function is not None:
                    current_function.comments = comments
                    current_function.check()
                current_function = CurrentFunction(func_name, display_name,
                        private)
                function_list.append(current_function)
                comments = []

            elif line.startswith("timeout"):
                current_function.timeout = int(line.split(":")[1].strip())
            elif line.startswith("async"):
                value = line.split(":")[1].strip()
                current_function.async = value=="true" and True or False
            elif line.startswith("search_criteria"):
                current_element = "search_criteria"
                p_type, p_name = get_parameter(line.split(":")[1], i)
                if p_type is not None:
                    if p_type.name not in types:
                        types.append(p_type.name)
                    current_function.add_parameter(current_element, p_type, p_name)
            elif line.startswith("order_by_criteria"):
                current_element = "order_criteria"
                p_type, p_name = get_parameter(line.split(":")[1], i)
                if p_type is not None:
                    if p_type.name not in types:
                        types.append(p_type.name)
                    current_function.add_parameter(current_element, p_type, p_name)

            elif line.startswith("in"):
                current_element = "in"
                p_type, p_name = get_parameter(line.split(":")[1], i)
                if p_type is not None:
                    if p_type.name not in types:
                        types.append(p_type.name)

                    current_function.add_parameter(current_element, p_type, p_name)
            elif line.startswith("out"):
                current_element = "out"
                p_type, p_name = get_parameter(line.split(":")[1], i)
                if p_type is not None:

                    if p_type.name not in types:
                        types.append(p_type.name)
                    current_function.add_parameter(current_element, p_type, p_name)

            elif line.startswith("errors"):
                current_element = "errors"
                error = line.split(":")[1].strip()
                if error.startswith("-"):
                    current_function.add_error(error)

            elif current_element == "in":
                if ":" in line:
                    line = line.split(":")[1]
                p_type, p_name = get_parameter(line, i)
                if p_type.name not in types:
                    types.append(p_type.name)

                current_function.add_parameter(current_element, p_type, p_name)

            elif current_element == "out":
                try:
                    p_type, p_name = get_parameter(line.split(":")[1], i)
                    if p_type.name not in types:
                        types.append(p_type.name)

                    current_function.add_parameter(current_element, p_type, p_name)
                except:
                    print "Error line %s:\n%s" % (i, line)
                    raise
            elif current_element == "search_criteria":
                try:
                    if not line.strip():
                        i += 1
                        continue
                    if ":" in line:
                        line = line.split(":")[1]

                    p_type, p_name = get_parameter(line, i)
                    if p_type.name not in types:
                        types.append(p_type.name)

                    current_function.add_parameter(current_element, p_type, p_name)

                except Exception, exc:
                    print "Error search_criteria line %s:\n%s" % (i, line)
                    raise
            elif current_element == "order_criteria":
                try:
                    if not line.strip():
                        i += 1
                        continue
                    if ":" in line:
                        line = line.split(":")[1]

                    p_type, p_name = get_parameter(line, i)
                    if p_type.name not in types:
                        types.append(p_type.name)

                    current_function.add_parameter(current_element, p_type, p_name)

                except Exception, exc:
                    import traceback
                    print traceback.format_exc()
                    print str(exc)
                    print "Error order_criteria line %s:\n%s" % (i, line)
                    raise
            elif current_element == "errors":
                if line.startswith("-"):
                    current_function.add_error(line)

            i += 1

        current_function.comments = comments
        current_function.check()
        return types, function_list
    except Exception, exc:
        print exc
        print "failed line %i" % i


class Error(object):

    def __init__(self, name, category, id, text, comments=None, generate=True):
        self.name       = name.strip()
        self.category   = category.strip()
        self.id         = int(id.strip())
        self.text       = text.strip()
        self.comments   = comments
        self.generate   = generate
        self.real_id    = int(errors_offsets[self.category]) + int(self.id)


def find_error(lines, i):
    '''
    error ErrorName{
        @DO NOT GENERATE@
        #Comment
        category: OFFSET,
        id: ID,
        text: Text {0}

    }
    '''
    line = lines[i].strip()
    assert(line.endswith("{")), "line %d doesn't end with a {" % i
    discard, ErrorName = line[:-1].split(" ")
    i += 1
    line = lines[i].strip()
    comments = ""
    category = None
    id = None
    text = None
    generate = True
    while not line.endswith("}"):
        line = line.strip()
        if line.startswith("@DO NOT GENERATE@"):
            generate = False
            i += 1
        elif line.startswith("#"):
            comments += line[1:]
            i += 1
        elif line.startswith("category"):
            assert(line.endswith(",")), "line %d doesn't end with a ," % i
            discard, category = line[:-1].split(":")
            i += 1
        elif line.startswith("id"):
            assert(line.endswith(",")), "line %d doesn't end with a ," % i
            discard, id = line[:-1].split(":")
            i += 1

        elif line.startswith("text:"):
            assert(line.endswith(",")), "line %d doesn't end with a ," % i
            discard, text = line[:-1].split(":", 1)
            i += 1

        line = lines[i].strip()
    for v in (comments, text, category, id):
        assert(v is not None), "Error missing information line %d error %s" %\
        (i, ErrorName)

    error = Error(ErrorName, category, id, text, comments, generate)

    return i+1, error


def find_service(config, lines, i, namespaces):
    line = lines[i]
    typedef, name = line.strip().replace("{", "").split()
    assert (")" in name), "name doesn't end with )"
    in_index = name.rfind("(")
    service_implementation = name.strip()[in_index+1:-1]
    match = re.match("v\d+.\d+", service_implementation)
    assert (match is not None), "Missing version in service implementation %s" % name
    version = Version(match.group())

    implementation=service_implementation[len(match.group())+1:]


    name = name[:in_index]

    identification = "%s.%s" % (namespaces["python"], name)

    service = Service(name, namespaces, identification, "%s.v%s.sv%s.%s" % (config.backend_path,
        version.version, version.sub_version, implementation))
    service.version = version

    global current_version
    if current_version.is_lower(version):
        current_version = version

    return parse_functions_in_service(lines, i, service)


def parse_functions_in_service(lines, i, service):
    j = i + 1
    count = 1
    start_line = -1
    stop_line = -1
    while j < len(lines) and count != 0:
        line = lines[j]
        line = line.strip().replace('\n', '')
        if line.startswith("#"):
            if start_line == -1:
                assert(stop_line == -1)
                service.comments += "%s\n" % line[1:]
        elif 0 == len(line):
            pass
        else:
            count += line.count("{")
            count -= line.count("}")

            if line.startswith("extend"):
                m = re.match("extend (.*)", line)
                service.extends.append(m.groups()[0])

            elif 2 == count and line.endswith("{"):
                auth_list = [a.strip() for a in line.split(":")[0].split(',')]
                j += 1
                start_line = j
                continue
            elif 1 == count:
                stop_line = j

                functions = find_functions(lines, start_line, stop_line)
                for function in functions[1]:
                    service.add_import_types(function)
                    service.add_function(auth_list, function)
                j += 1
                continue
            elif 0 == count:
                if "=>" in line:
                    identification = line[line.index("=>") + 2:].strip()
                    service.identification = identification
                j += 1
                break
            # Check the authorization

            #assert (line.endswith(',') or 1 == count), "line %d:%s doesn't end with ," % (j,line)
            #try:
            #    function_res = find_function(lines, j)
            #    return_type, function = line[:len(line)-1].replace("}", "", 1).replace("{","", 1).split(" ", 1)
            #    return_name = u"void"
            #    if return_type != "void":
            #        return_name, return_type = return_type.split(":")
            #    return_type = extract_type(return_type)

            #    # Now extract the name
            #    function_name, parameters = function.split("(", 1)
            #    # now remove the last )
            #    parameters = parameters.strip()
            #    parameters = parameters[:len(parameters) - 1]

            #    # now we have a list of parameters separated with ,

            #    param_list = {}

            #    for param in parameters.split(","):
            #        param = param.strip()
            #        if 0 != len(param):
            #            param_type, param_name = param.strip().split()
            #            p_type = extract_type(param_type.strip())
            #            param_list[param_name.strip()] = p_type
            #            if p_type.name not in service.objects_type:
            #                service.objects_type.append(p_type.name)
            #    service.add_function(auth_list, {return_name: return_type}, function_name, param_list)
            #except ValueError, exc:
            #    print exc
            #    assert (0 == count), "missing ending } for line %s" % line
        j += 1
    return j, service


def find_interface(lines, i, namespaces):
    line = lines[i].strip()
    assert(line.endswith("{")), "missing { line %s" % i

    m = re.match("interface (.*)\((v\d.\d)\){", line)
    name = m.groups()[0]
    version = Version(m.groups()[1])

    service = Service(name, namespaces, "", "", True)
    service.version = version

    i, s = parse_functions_in_service(lines, i, service)
    return s, i


class Objects(dict):
    unique_id = None


def process(config, file_name, unique_id):
    with open(file_name) as f:

        objects = Objects()
        objects.unique_id = unique_id
        services = {}
        namespaces = {}
        errors = {}
        current_service = False
        current_object = False
        lines = f.readlines()

        i = 0
        while i < len(lines):
            line = lines[i].strip()
            if line.startswith('typedef'):

                i, current_object = find_object(config, lines, i, namespaces)

                objects[current_object.name] = current_object
                if current_object.parent_name is not None:
                    current_object.set_parent(objects[current_object.parent_name])
                continue
            elif line.startswith('service'):
                i, service = find_service(config, lines, i, namespaces)
                services[service.name] = service
                continue
            elif line.startswith("require"):
                req_word, name = line.strip().split()
                file_dict = file_name.split("/")
                t_objects, t_services, t_errors = process(
                    config,
                    "%s/%s.lacie" % ("/".join(file_dict[:-1]), name),
                    unique_id
                )

                #t_objects, t_services = process("%s.lacie"% (name))
                objects.update(t_objects)
                services.update(t_services)
                for key, value in t_errors.iteritems():
                    if key in errors:
                        if errors[key].name != value.name:
                            raise Exception("%s id already set for %s. Couldn't use it for %s" % (key, errors[key].name, value.name))
                    errors[key] = value
            elif line.startswith("enum"):
                i, enum = find_enum(lines, i, namespaces)
                objects[enum.name] = enum
                continue
            elif line.startswith("error"):
                # We define an error
                i, error = find_error(lines, i)
                if error.real_id in errors:
                    if errors[error.real_id].name != error.name:
                        raise Exception("%s id already set for %s. Couldn't use it for %s" % (error.real_id, errors[error.real_id].name, error.name))
                errors[error.real_id] = error
            elif line.startswith("package"):
                [namespace_text, language, namespace] = line.split(" ")
                try:
                    assert(len(objects) == 0 and len(services) == 0)
                    namespaces[language] = namespace
                except AssertionError:
                    print("Namespace should be defined before first Object/Service declaration")
                    raise
            elif line.strip() == "":
                pass
            #elif "}" == line.strip():
            #    pass
            elif line.startswith("#"):
                pass
            elif line.startswith("interface"):
                interface, i = find_interface(lines, i, namespaces)
                services[interface.name] = interface
            else:
                raise Exception("Unexpected keyword line %s: \"%s\"" % (i, line))

            i += 1
        return objects, services, errors


def get_offsets(lines, i):
    line = lines[i].strip()
    assert(line.endswith("{")), "Line %d should end with {" % i
    i += 1
    line = lines[i].strip()
    offsets = {}
    while not line.endswith("}"):
        assert(line.endswith(",")), "Line %d should end with a ," % i
        offset, value = line[:-1].split("=")
        offsets[offset.strip()] = value.strip()
        i += 1
        line = lines[i].strip()

    return i+1, offsets


def get_errors(file_name):
    with open(file_name) as f:
        lines = f.readlines()
        i = 0
        errors = {}
        OFFSETS = None
        while i < len(lines):
            line = lines[i].strip()
            if line.startswith("#"):
                i += 1
                pass
            elif line.startswith("error"):
                i, error = find_error(lines, i)
                errors[error.real_id] = error
                pass
            elif line.startswith("features_offset"):
                assert(OFFSETS is None), "New definition for features_offsets"
                i, OFFSETS = get_offsets(lines, i)
                errors_offsets.update(OFFSETS)
            else:
                i += 1

    return errors, OFFSETS


class Config(object):

    def __init__(self):
        self.backend_name       = ""
        self.min_version        = Version("v0.0")
        self.current_version    = Version("v0.0")
        self.files              = []

def parse_option_file(file_path):
    """
    Parse the options given in the file_path

    extract the files to generate, the version to start generating from
    and the backend name.

    returns those 3 informations

    args, version, backend_path
    """
    with open(file_path) as f:
        import json
        jdict = json.load(f)
        import os.path
        prefix_path = os.path.dirname(file_path)
        config = Config()
        config.files        = [os.path.join(prefix_path, f) for f in jdict['files']]
        config.min_version  = Version("v%s" % jdict['min_version'])
        config.backend_path = jdict['backend']
        config.error_file   = os.path.join( prefix_path, jdict['error_file'])
        config.gen_path     = jdict['gen_path']
        config.lib_path     = jdict['lib_path']
        return config


g_objects = Objects()
g_services = {}
g_errors = {}
errors_offsets = {}


def main():
    from optparse import OptionParser
    parser = OptionParser()
    parser.generation = []

    import os
    import os.path
    path = os.path.dirname(os.path.abspath(__file__))
    plugins_path = os.path.join(path, "plugins")
    for plugin in os.listdir(plugins_path):
        if not os.path.isdir(os.path.join(plugins_path, plugin)):
            continue
        if plugin in ['.svn', ]:
            continue
        name = "optimus_prime.plugins.%s" % plugin
        __import__(name)
        module = sys.modules[name]
        result = module.get_options(parser)
        if result is not None:
            parser.add_option_group(result[0])
            setattr(parser, plugin, result[1])

    parser.add_option("-q", "--quiet",
                      action="store_false", dest="verbose", default=True,
                      help="don't print status messages to stdout")
    parser.add_option("-c", "--client", dest="client", default=None,
            help="python, java")

    parser.add_option("-s", "--server", dest="server", default=None,
            help="python, java")

    parser.add_option("-l", "--lib_path", dest="lib_path", default=None,
            help="the prefix for the lib")

    parser.add_option("-b", "--backend", dest="backend_name", default=None,
            help="The backend_name")

    parser.add_option("-g", "--gen_path", dest="gen_path", default=None,
            help="the folder where the generated folders will be stored (server only)")

    parser.add_option("-v", "--min_version", dest="min_version", default=None,
            help="the version to generate")

    parser.add_option("-e", "--error_file", dest="error_file", default=None,
            help="The error file for offset definition")

    parser.add_option("-f", "--file", dest="file_desc", default=None,
            help="Use a file description instead")

    parser.add_option("-p", "--prefix", dest="prefix", default=None,
            help="Set prefix before filename")

    parser.add_option("-o", "--old", dest="legacy", default=False,
            action="store_true",
            help="Build legacy style serialize, no version and no sub_version")

    parser.add_option("-O", "--out", dest="out", default="gen",
            type="string", action="store",
            help="specify build folder")

    (options, args) = parser.parse_args()
    if not args and options.file_desc is None:
        print "Missing generation file"
        parser.print_help()
        exit(-1)

    if args and options.file_desc is not None:
        print "Choose either by file or by parameters"
        parser.print_help()
        exit(-1)


    if options.file_desc is not None:
        options.file_desc = os.path.realpath(options.file_desc)
        config = parse_option_file(options.file_desc)
        if options.lib_path is not None:

            config.lib_path     = options.lib_path
    else:
        config = Config()

    if options.error_file is not None:
        config.error_file   = options.error_file
    if options.min_version is not None:
        config.min_version  = Version(options.min_version)
    if options.backend_name is not None:
        config.backend_name = options.backend_name
    if options.gen_path is not None:
        config.gen_path     = options.gen_path

    if args is None:
        config.files        = args

    if options.error_file is not None:
        config.error_file   = options.error_file

    if options.lib_path is not None:
        config.lib_path     = options.lib_path

    config.legacy = options.legacy
    config.output = options.out

    import time

    set_prefix(options.prefix)
    if config.error_file is None and len(g_errors) != 0:
        print "Error definition must include a error file"
        exit(-1)
    elif config.error_file:
        #Now start by loading the error file to get the OFFSET values
        #and check for all the errors
        default_errors, errors_offsets = get_errors(config.error_file)
        g_errors.update(default_errors)
        for error_name, error in g_errors.iteritems():
            assert(error.category in errors_offsets.keys()), \
                "Error categorie %s not definied for %s" % (error.category, error_name)

    #for error_id in g_errors:
    #    print "%s %s" % (error_id, g_errors[error_id].name)
    for file_name in config.files:
        print "Parsing %s" % file_name
        objects, services, errors = process(config, file_name, time.time())
        g_objects.update(objects)
        g_services.update(services)
        for key, value in errors.iteritems():
            if key in g_errors:
                if g_errors[key].name != value.name:
                    raise Exception("%s id already set for %s. Couldn't use it for %s" % (key, g_errors[key].name, value.name))
        g_errors.update(errors)

    #for error_id in g_errors:
    #    print "%s %s" % (error_id, g_errors[error_id].name)


    g_objects.unique_id = time.strftime("%b %d %Y %H:%M:%S", time.localtime())
    g_objects.config    = config

    config.current_version = current_version
    for gen_type in parser.generation:
        #Now check for the generation plugin
        name = "optimus_prime.plugins.%s" % gen_type
        module = sys.modules[name]

        config.options = getattr(parser, gen_type)
        args = module.pre_generate_action(config)

        module.GenerateServices(config, g_services, g_objects, *args)
        module.GenerateObjects(config, g_objects, *args)
        module.GenerateErrors(config, g_errors, errors_offsets, *args)


        module.post_generate_action(config, *args)

if "__main__" == __name__:
    main()
