from optimus_prime.utils import Enum, default_types, result_set_param
from optimus_prime.utils import list_param, dict_param, gen_obj, DefaultType

JAVA = "java"
config = None
g_services = None

prefix_comm_layer = "com.lacie.communicationlayer2"

def check_keyword(keyword):
    if keyword in ["public", "interface"]:
        return "_%s" % keyword
    return keyword

def check_and_create_gen_file(object_type, Object):
    import os
    import os.path

    folder = "gen.%s.client.%s" % (prefix_comm_layer,
            Object.get_import_path(config.min_version, JAVA))
    if os.path.exists(os.path.join("gen/android",  "/".join(folder.split(".")))):
        pass
    else:
        os.makedirs(os.path.join("gen/android", "/".join(folder.split('.'))))


    file_path = "%s/%s.java" % ("/".join(folder.split('.')), object_type)
    file = open(os.path.join("gen/android", file_path), "w")
    return file

base_type = {
                "Unicode":  "String",
                "Integer":  "Integer",
                "Boolean":  "Boolean",
                "Long":     "Long",
                "Double":    "Double",
                "void":     "void"
        }


def get_java_instantiator(param_type, objects):
    if isinstance(param_type, list_param):
        return "ListInstantiator.getInstance()"
    elif isinstance(param_type, dict_param):
        return "DictInstantiator.getInstance()"
    elif isinstance(param_type, result_set_param):
        return "ResultSetInstantiator.getInstance()"
    elif isinstance(param_type, gen_obj):
        if param_type.name in objects and isinstance(objects[param_type.name], Enum):
            return "null"
        val = get_java_instantiator(param_type.name, objects)
        if val == "null":
            return val
        if val == DefaultType.Date:
            return "null";
        return "%sInstantiator.getInstance()" % val
    elif param_type in base_type:
        return u"null"
    else:
        if param_type in objects and isinstance(objects[param_type], Enum):
            return u"null"
        return param_type

    return u"instantiator"

def get_imports(o_object, objects):
    string = u""
    for obj_type in o_object.imports:
        if not(obj_type in base_type) and not(obj_type in default_types):
            ob = objects[obj_type]
            string += "import %s.client.%s.%s;\n" %\
            (prefix_comm_layer, ob.get_import_path(config.min_version, JAVA), obj_type)
            if not isinstance(ob, Enum):
                string += "import %s.client.%s.%sInstantiator;\n" %\
            (prefix_comm_layer, ob.get_import_path(config.min_version, JAVA), obj_type)

        elif obj_type in default_types:
            value = obj_type
            if value in [DefaultType.Date,]:
                string += "import java.util.Date;\n"
                continue
            if value in [DefaultType.Long, DefaultType.Double]:
                continue

            string +="import %s.client.lib.%sInstantiator;\n" %\
                (prefix_comm_layer, value)
            if value in [DefaultType.Dict, DefaultType.List,
                    DefaultType.ResultSet]:
                string +="import %s.client.lib.%sInstance;\n" %\
                    (prefix_comm_layer, value)
                if value == DefaultType.List:
                    string += "import %s.client.lib.Types;\n" % prefix_comm_layer
                value = "Nice%s" % value
            string += "import %s.client.lib.%s;\n" % (prefix_comm_layer, value)

    return string+"\n"


def get_java_type(param_type):

    if isinstance(param_type, list_param):
        return "NiceList<%s>"%get_java_type(param_type.sub_type)
    elif isinstance(param_type, dict_param):
        print param_type.sub_type
        return "NiceDict<%s>"%get_java_type(param_type.sub_type)
    elif isinstance(param_type, result_set_param):
        return "NiceResultSet<%s>"%get_java_type(param_type.sub_type)
    elif isinstance(param_type, gen_obj):
        return get_java_type(param_type.name)
    elif param_type in base_type:
        return base_type[param_type]
    else:
        return param_type

def to_java_style_param(string):
    return check_keyword(string.title().replace('_',''))

def to_java_style_var(string):
    tmp = string.title().replace('_','')
    return check_keyword(tmp[0].lower() + tmp[1:])

def GenerateEnum(Object, objects):
    f = check_and_create_gen_file("%s" % Object.name, Object)
    f.write("package %s.client.%s;\n" %\
            (prefix_comm_layer, Object.get_import_path(config.min_version,
                JAVA)))
    f.write("import java.util.HashMap;\n")
    f.write("import java.util.Map;\n\n")
    f.write("import com.lacie.communicationlayer2.client.lib.NiceEnum;\n")
    f.write("\npublic enum %s implements NiceEnum<%s> {\n\n" % (Object.name, get_java_type(Object.sub_type)))

    values = []
    for key, value in Object.values.iteritems():
        if Object.sub_type == DefaultType.Unicode:
            special_char = '"'
        else:
            special_char = ''
        values.append("%s(%s%s%s)" % (key.upper(), special_char, value,\
            special_char))
    f.write("\t")
    f.write(", ".join(values))
    f.write(";\n")

    f.write("\n\tprivate final %s value;\n\n" % get_java_type(Object.sub_type) )

    f.write("\tprivate %s(%s value) {\n" % (Object.name, get_java_type(Object.sub_type)))
    f.write("\t\tthis.value = value;\n")
    f.write("\t}\n")
    f.write("\n")
    f.write("\t@Override\n")
    f.write("\tpublic %s getValue() {\n" % get_java_type(Object.sub_type))
    f.write("\t\treturn this.value;\n")
    f.write("\t}\n")
    f.write("\n")
    f.write("\tprivate static Map<%s, %s> map%s = new HashMap<%s, %s>();\n"\
            % (get_java_type(Object.sub_type), Object.name,  Object.name,\
                get_java_type(Object.sub_type), Object.name))
    f.write("\tstatic {\n")
    f.write("\t\tfor (%s %s : values()) {\n" % (Object.name, to_java_style_var(Object.name)))
    f.write("\t\t\tmap%s.put(%s.getValue(), %s);\n" % (Object.name, to_java_style_var(Object.name), to_java_style_var(Object.name)))
    f.write("\t\t}\n")
    f.write("\t}\n")
    f.write("\n")
    f.write("\tpublic static %s fromValue(%s value) {\n" % (Object.name, get_java_type(Object.sub_type)))
    f.write("\t\tfinal %s %s = map%s.get(value);\n" % (Object.name, to_java_style_var(Object.name), Object.name))
    f.write("\t\tif (null != %s) {\n" % to_java_style_var(Object.name))
    f.write("\t\t\treturn %s;\n" % to_java_style_var(Object.name))
    f.write("\t\t}\n")
    f.write("\t\telse {\n")
    f.write("\t\t\treturn UNKNOWN;\n")
    f.write("\t\t}\n")
    f.write("\t}\n")
    f.write("}")

def GenerateObject(Object, objects):
    '''
    Generate the corresponding Python File
    '''
    if isinstance(Object, Enum):
        GenerateEnum(Object, objects)
        return
    f = check_and_create_gen_file("%sInstantiator" % Object.name, Object)
    f.write("package %s.client.%s;\n" %\
            (prefix_comm_layer, Object.get_import_path(config.min_version,
                JAVA)))

    f.write("import %s.client.lib.Instantiable;\n" % prefix_comm_layer)
    f.write("import %s.client.lib.KeyWords;\n" % prefix_comm_layer)

    f.write("import %s.client.lib.Types;\n" % prefix_comm_layer)
    f.write("import java.util.HashMap;\n")
    f.write("import java.util.Map;\n")
    f.write("import java.util.List;\n")
    f.write("import java.util.ArrayList;\n")
    f.write("import %s.client.lib.Instantiator;\n" % prefix_comm_layer)

    f.write(get_imports(Object, objects))

    f.write("@SuppressWarnings(\"unused\")\n")
    f.write("public class %sInstantiator extends Instantiator<%s> {\n" % (Object.name, Object.name))
    f.write("\n\tprivate static %sInstantiator instance;\n" % Object.name)
    f.write("\n\tprivate final Map<String, Instantiator<?>> attributes = new HashMap<String, Instantiator<?>>();\n")
    f.write("\tprivate final List<String> optionalList = new ArrayList<String>();\n")


    f.write("\n\tprivate %sInstantiator() {\n" % Object.name)
    f.write("\t\tsuper();\n")
    optional_list = []
    for param, attribute in Object.attributes.iteritems():
        if not attribute.write():
            continue
        if attribute.type.is_optional:
            optional_list.append(param)
        f.write("\t\tattributes.put(%s.param%s, %s);\n" % (Object.name,
            to_java_style_param(param), get_java_instantiator(attribute.type, objects)))

    for param in optional_list:
        f.write("\t\toptionalList.add(%s.param%s);\n" % (Object.name,
            to_java_style_param(param)))
    f.write("\t}\n")


    f.write("\n\t@Override\n")
    f.write("\tpublic List<String> getOptionalList() {\n")
    f.write("\t\treturn optionalList;\n")
    f.write("\t}\n")

    f.write("\n\tpublic static %sInstantiator getInstance() {\n" % Object.name)
    f.write("\t\tif(null == instance){\n")
    f.write("\t\t\tinstance = new %sInstantiator();\n"%Object.name)
    f.write("\t\t}\n")
    f.write("\t\treturn instance;\n")
    f.write("\t}\n")

    f.write("\n\t@Override\n")
    f.write("\tpublic String getName() {\n")
    f.write("\t\treturn Types.%s.getValue();\n" % Object.name.upper())
    f.write("\t}\n")

    f.write("\n\t@Override\n")
    f.write("\tpublic %s getNewObject() {\n" % Object.name)
    f.write("\t\treturn new %s();\n" % Object.name)
    f.write("\t}\n")

    f.write("\n\t@Override\n")
    f.write("\tpublic Map<String, Instantiator<?>> getAttributesSerializers() {\n")
    f.write("\t\treturn attributes;\n")
    f.write("\t}\n")


    f.write("\n\t@Override\n")
    f.write("\tpublic %s deserialize(Instantiable<?> object, Object value) {\n" \
            % Object.name)
    f.write("\t\tMap<String, Object> temp = (Map<String, Object>) value;\n")
    f.write("\t\tMap<String, Object> __properties__ = (Map<String, Object>)temp.get(KeyWords.Properties);\n")
    for param, attribute in Object.attributes.iteritems():
        if not attribute.read():
            continue
        f.write("\n\t\t// %s\n" % to_java_style_param(param))
        f.write("\t\tObject json%s = __properties__.get(%s.param%s);\n" %\
                (to_java_style_param(param), Object.name, to_java_style_param(param)))

        f.write("\t\tObject temp%s;\n" % to_java_style_param(param))
        if attribute.hidden:
            f.write("\t\tif (null != json%s)\n" % to_java_style_param(param))
        f.write("\t\t\ttemp%s = Instantiator.deserialize(json%s);\n" % (to_java_style_param(param), to_java_style_param(param)))
        if attribute.hidden:
            f.write("\t\telse\n")
            f.write("\t\t\ttemp%s = null;\n" % to_java_style_param(param))
        if isinstance(attribute.type, list_param):
            f.write("\t\tobject.setAttr(%s.param%s, new\
 %s(Types.%s, (ListInstance)temp%s));\n" % (Object.name, to_java_style_param(param),\
     get_java_type(attribute.type), attribute.type.sub_type.upper(), to_java_style_param(param)))
        elif isinstance(attribute.type, dict_param):
            f.write("\t\tobject.setAttr(%s.param%s, new\
 %s(Types.%s, (DictInstance)temp%s));\n" % (Object.name, to_java_style_param(param),\
     get_java_type(attribute.type), attribute.type.sub_type.upper(), to_java_style_param(param)))
        else:
            if attribute.type.name in objects and isinstance(objects[attribute.type.name], Enum):
                f.write("\t\tobject.setAttr(%s.param%s, %s.fromValue(" % (Object.name, to_java_style_param(param), attribute.type.name))
                sub_type_enum = objects[attribute.type.name].sub_type
                if sub_type_enum == DefaultType.Unicode:
                    f.write("(String) temp%s));\n" % to_java_style_param(param))
                else:
                    f.write("(Integer) temp%s));\n" % to_java_style_param(param))
            else:
                f.write("\t\tobject.setAttr(%s.param%s, temp%s);\n" % (Object.name,\
                    to_java_style_param(param), to_java_style_param(param)))

    f.write("\n\t\treturn (%s) object;\n" % Object.name)
    f.write("\t}\n")
    f.write("}\n")
    f.close()

    #### Generate Object file ###

    f = check_and_create_gen_file(Object.name, Object)
    f.write("package %s.client.%s;\n\n" %\
            (prefix_comm_layer, Object.get_import_path(config.min_version,
                JAVA)))
    f.write("import java.util.HashMap;\n")
    f.write("import java.util.Map;\n")
    f.write("import java.util.List;\n\n")

    f.write("import %s.client.lib.Instantiable;\n" % prefix_comm_layer)
    f.write(get_imports(Object, objects))

    f.write("@SuppressWarnings(\"unused\")\n")
    f.write("public class %s extends Instantiable<%s> {\n\n" % (Object.name, Object.name))

    for param in Object.attributes:
        f.write("\tpublic static final String param%s = \"%s\";\n" % (to_java_style_param(param), param))
    f.write("\n\t@Override")
    f.write("\n\tpublic %sInstantiator getInstantiator() {\n" % Object.name)
    f.write("\t\treturn %sInstantiator.getInstance();\n" % Object.name)
    f.write("\t}\n\n")



    for param, attribute in Object.attributes.iteritems():

        public = ""
        if attribute.write():
            public = "public "

        # If NiceList: NiceList => List!
        visibleType = ""
        if attribute.write() and isinstance(attribute.type, list_param):
            visibleType = "List<%s>" % get_java_type(attribute.type.sub_type)
        else:
            visibleType = get_java_type(attribute.type)

        # Javadoc
        f.write("\t/**\n")
        if attribute.comments:
            f.write("\t * %s\n" % attribute.comments)
            f.write("\t * \n")
        f.write("\t * @param %s\n" % to_java_style_var(param))
        f.write("\t *            %s\n" % visibleType)
        f.write("\t */\n")

        f.write("\t%svoid set%s(%s %s) {\n" % (public, to_java_style_param(param), visibleType, to_java_style_var(param)))
        # If NiceList: NiceList => List!
        if  attribute.write() and isinstance(attribute.type, list_param):
            f.write("\t\tif (%s instanceof NiceList) {\n" % to_java_style_var(param))
            f.write("\t\t\tsetAttr(param%s, %s);\n" % (to_java_style_param(param), to_java_style_var(param)))
            f.write("\t\t} else {\n")
            f.write("\t\t\t%s niceList = new %s(Types.fromValue(\"%s\"));\n" % (get_java_type(attribute.type), get_java_type(attribute.type),  \
                     get_java_type(attribute.type.sub_type)))
            f.write("\t\t\tniceList.addAll(%s);\n" % to_java_style_var(param))
            f.write("\t\t\tsetAttr(param%s, niceList);\n" % to_java_style_param(param))
            f.write("\t\t}\n")
        else:
            f.write("\t\tsetAttr(param%s, %s);\n" % (to_java_style_param(param), to_java_style_var(param)))

        f.write("\t}\n\n")


        if attribute.read():

            # Javadoc
            f.write("\t/**\n")
            if attribute.comments:
                f.write("\t * %s\n" % attribute.comments)
                f.write("\t * \n")
            f.write("\t * @return %s\n" % to_java_style_var(param))
            f.write("\t */\n")

            if isinstance(attribute.type, list_param):
                f.write("\t@SuppressWarnings(\"unchecked\")\n")
            f.write("\tpublic %s get%s() {\n" % (get_java_type(attribute.type), to_java_style_param(param) ))

            f.write("\t\treturn (%s) getAttr(param%s);\n" %\
                        (get_java_type(attribute.type), to_java_style_param(param)))

            f.write("\t}\n\n")
    f.write("}")
    f.close()


def GenerateInterface(Object, objects):
    f = check_and_create_gen_file(Object.name, Object)
    f.write("package %s.client.%s;\n" %\
            (prefix_comm_layer, Object.get_import_path(config.min_version,
                JAVA)))
    f.write("import java.util.HashMap;\n")
    f.write("import java.util.Map;\n")
    f.write("import java.util.List;\n")
    f.write("import %s.client.lib.RPCRequestCallback;\n" % prefix_comm_layer)
    f.write("import %s.client.lib.VoidRequestCallback;\n" % prefix_comm_layer)

    f.write(get_imports(Object, objects))
    f.write("@SuppressWarnings(\"unused\")\n")
    f.write("public interface %s {\n\n" % Object.name)
    for func_name, function in Object.functions.iteritems():
        in_params = function.params[0]
        out_params = [p for p,t in function.params[1]]

        if 1 == len(function.params[1]):
            r_type = get_java_type(function.params[1][0][1])

        elif 0 == len(function.params[1]):
            r_type = "void"
        else:
            r_type = "Map<String, Object>"
        f.write("\tpublic void %s(" % func_name)
        list_string = []
        for param, param_type in in_params:
            list_string.append("%s %s" % (get_java_type(param_type), check_keyword(param)))

        f.write(",".join(list_string))
        if list_string:
            f.write(", ")


        if 'void' == r_type:
            f.write("VoidRequestCallback callback);\n")
        else:
            f.write("final RPCRequestCallback<%s> callback);\n" % r_type)



    f.write("\n}")
    f.close()





def GenerateService(Object, objects):
    '''
    Generate the corresponding Python File
    '''
    f = check_and_create_gen_file(Object.name, Object)
    f.write("package %s.client.%s;\n" %\
           (prefix_comm_layer,  Object.get_import_path(config.min_version,
               JAVA)))
    f.write("import java.util.HashMap;\n")
    f.write("import java.util.Map;\n")
    f.write("import java.util.List;\n")

    f.write("import %s.client.lib.Instantiator;\n" % prefix_comm_layer)
    f.write("import %s.client.lib.RequestSender;\n" % prefix_comm_layer)
    f.write("import %s.client.lib.RPCRequestCallback;\n" % prefix_comm_layer)
    f.write("import %s.client.lib.VoidRequestCallback;\n" % prefix_comm_layer)
    f.write("import %s.client.lib.SerializeCallBack;\n" % prefix_comm_layer)
    f.write("import %s.client.lib.TransformerSerializer;\n" % prefix_comm_layer)
    f.write("import %s.client.lib.Types;\n" % prefix_comm_layer)
    f.write(get_imports(Object, objects))

    functions = Object.functions

    if Object.extends:
        extend_string = " implements "
        count = len(Object.extends)
        for extend in Object.extends:
            count -= 1
            interface = g_services[extend]
            f.write("import %s.client.%s.%s;\n" %\
                (prefix_comm_layer, interface.get_import_path(config.min_version, JAVA), interface.name))
            f.write(get_imports(interface, objects))

            extend_string += interface.name
            if not count == 0:
                extend_string += ","

            functions.update(interface.functions)
    else:
        extend_string = ""


    f.write("@SuppressWarnings(\"unused\")\n")
    f.write("public class %s%s {\n\n" % ( Object.name, extend_string))

    f.write("\tpublic static final String CAPABILITY = \"%s\";\n\n" % Object.identification)
    f.write("\tprivate static %s instance = null;\n\n" % Object.name)
    f.write("\tpublic static %s getInstance() {\n" % Object.name)
    f.write("\t\tif(null == instance){\n")
    f.write("\t\t\tinstance = new %s();\n" % Object.name)
    f.write("\t\t}\n")
    f.write("\t\treturn instance;\n")
    f.write("\t}\n")


    func_name_list = functions.keys()
    func_name_list.sort()
    # Now generate the checkers for the types
    for func_name in func_name_list:
        function = functions[func_name]
        if function.private:
            continue
        in_params = function.params[0]
        out_params = [p for p,t in function.params[1]]

        if 1 == len(function.params[1]):
            r_type = get_java_type(function.params[1][0][1])

        elif 0 == len(function.params[1]):
            r_type = "void"
        else:
            r_type = "Map<String, Object>"

        # Javadoc
        f.write("\n\t/**\n")
        if function.comments:
            for comment in function.comments:
                f.write("\t * %s\n" % comment)
            f.write("\t *\n")
        for param, param_type in function.params[0]:
            f.write("\t * @param %s\n" % to_java_style_var(param))
            f.write("\t *            %s\n" % get_java_type(param_type))
        f.write("\t * @param callback\n")
        if 'void' == r_type:
            f.write("\t *            VoidRequestCallback\n")
        else:
            f.write("\t *            RPCRequestCallback<%s>\n" % r_type)
        f.write("\t */\n")

        f.write("\t@SuppressWarnings({ \"unchecked\", \"rawtypes\" })\n")
        f.write("\tpublic void %s(" % func_name)
        list_string = []
        for param, param_type in function.params[0]:
            list_string.append("%s %s" % (get_java_type(param_type), to_java_style_var(param)))
        f.write(", ".join(list_string))
        if list_string:
            f.write(", ")


        if 'void' == r_type:
            f.write("VoidRequestCallback callback) {\n")
        else:
            f.write("final RPCRequestCallback<%s> callback) {\n" % r_type)
        f.write("\t\tMap<String, Object> params = new HashMap<String, Object>();\n")
        for param, p_type in in_params:
            param_value = to_java_style_var(param)
            if p_type.name in objects and isinstance(objects[p_type.name], Enum):
                param_value = "%s.getValue()" % param_value
            f.write("\t\tparams.put(\"%s\", %s);\n" % (param, param_value))
        obj_version = Object.version
        if obj_version.is_lower(config.min_version):
            obj_version = config.min_version
        f.write("\t\tString url = \"/api/external/%s.%s/%s.%s\";\n" %\
               (obj_version.version, obj_version.sub_version, Object.identification, func_name))
        f.write("\t\tRequestSender sender = new RequestSender(TransformerSerializer.toString(params), url);\n")
        if function.timeout:
            f.write("\t\tsender.setTimeoutSeconds(%s);\n" % function.timeout)
        f.write("\t\tsender.send(new SerializeCallBack(callback) {\n");


        f.write("\t\t\t@Override\n")
        f.write("\t\t\tpublic void onError(String response){\n")
        f.write("\t\t\t}\n")

        f.write("\t\t\t@Override\n")
        f.write("\t\t\tpublic void onSuccess(String response) {\n")
        if 0 == len(function.params[1]):
            f.write("\t\t\t\tcallback1.onSuccess(null);\n")
        else:
            f.write("Map<String, Object> temp = TransformerSerializer.fromString(response);\n")
            f.write("\t\t\t\tMap<String, Object> obj = (Map<String, Object>) Instantiator.deserialize(temp);\n")
            if 1 == len(function.params[1]):
                key = out_params[0]
                param_type = function.params[1][0][1]
                if isinstance(param_type, list_param):
                    f.write("\t\t\t\t%s return_obj = new %s(Types.%s, (ListInstance)obj.get(\"%s\"));\n"\
                            % (r_type, r_type, param_type.name.upper(), key))
                elif isinstance(param_type, dict_param):
                    f.write("\t\t\t\t%s return_obj = new %s(Types.%s, (DictInstance)obj.get(\"%s\") );\n"\
                            % (r_type, r_type, param_type.name.upper(), key))
                elif isinstance(param_type, result_set_param):
                    f.write("\t\t\t\t%s return_obj = new %s(Types.%s, (ResultSetInstance)obj.get(\"%s\"));\n"\
                            % (r_type, r_type, param_type.name.upper(), key))
                else:
                    f.write("\t\t\t\t%s return_obj = (%s) obj.get(\"%s\");\n" %\
                            (r_type, r_type, key))

            else:
                f.write("%s return_obj = obj;\n"%r_type)
            f.write("\t\t\t\tcallback1.onSuccess(return_obj);\n")
        f.write("\t\t\t}\n")
        f.write("\t\t});\n")
        f.write("\t}\n")

    f.write("}\n")
    f.close()



types_string = "\
/*************************************************************************\n\
                     LACIE CONFIDENTIAL\n\
\n\
The information contained in this file is the property of LACIE.\n\
The user of this file shall keep information contained herein confidential\n\
and shall protect it in whole or in part from disclosure and dissemination\n\
to third parties. \n\
Without prior written consent of LACIE, you may not reproduce, represent,\n\
or download through any means, the information contained herein in any way\n\
or in any form for a purpose other that the use for internal evaluation,\n\
operation or maintenance within LACIE.\n\
\n\
          (c) LACIE, 2009 - 2012, All Rights Reserved.\n\
 *************************************************************************/\n\
package %s.client.lib;\n\
\n\
import java.util.HashMap;\n\
import java.util.Map;\n\
\n\
public enum Types {\n\
\tUNKNOWN(\"Unknown\"), UNICODE(\"Unicode\"), INTEGER(\"Integer\"), BOOLEAN(\"Boolean\"), \n\
\tDOUBLE(\"Double\"), LONG(\"Long\"), ULONG(\"ULong\"), \n\
\tLIST(\"List\"), DICT(\"Dict\"), RESULTSET(\"ResultSet\"), \n\
\tDATE(\"Date\"), TIME(\"Time\"), IP(\"IP\")" %\
    prefix_comm_layer


types_string2 = ";\n\n\tprivate String name;\n\
\n\
\tprivate Types(String name) {\n\
\t\tthis.name = name;\n\
\t}\n\
\n\
\tpublic String getValue() {\n\
\t\treturn this.name;\n\
\t}\n\
\n\
\tprivate static Map<String, Types> map = new HashMap<String, Types>();\n\
\tstatic {\n\
\t\tfor (Types type : values()) {\n\
\t\t\tmap.put(type.getValue(), type);\n\
\t\t}\n\
\t}\n\
\n\
\tpublic static Types fromValue(String value) {\n\
\t\tfinal Types type = map.get(value);\n\
\t\tif (null != type) {\n\
\t\t\treturn type;\n\
\t\t} else {\n\
\t\t\treturn UNKNOWN;\n\
\t\t}\n\
\t}\n\
}"

register_inst = "\
/*************************************************************************\n\
                 LACIE CONFIDENTIAL\n\
                 \n\
The information contained in this file is the property of LACIE.\n\
The user of this file shall keep information contained herein confidential\n\
and shall protect it in whole or in part from disclosure and dissemination\n\
to third parties. \n\
Without prior written consent of LACIE, you may not reproduce, represent,\n\
or download through any means, the information contained herein in any way\n\
or in any form for a purpose other that the use for internal evaluation,\n\
operation or maintenance within LACIE.\n\
\n\
          (c) LACIE, 2009 - 2010, All Rights Reserved.\n\
 *************************************************************************/\n\
package %s.client.lib;\n\
\n\
import java.util.ArrayList;\n\
import java.util.HashMap;\n\
import java.util.Map;\n" % prefix_comm_layer

register_inst2 = "\
\n\
public class RegisteredInstantiators {\n\n\
\tpublic static final java.util.List<String> base_types = new ArrayList<String>();\n\
\tstatic {\n\
\t\tbase_types.add(Types.UNICODE.getValue());\n\
\t\tbase_types.add(Types.INTEGER.getValue());\n\
\t\tbase_types.add(Types.BOOLEAN.getValue());\n\
\t\tbase_types.add(Types.DOUBLE.getValue());\n\
\t\tbase_types.add(Types.LONG.getValue());\n\
\t\tbase_types.add(Types.DATE.getValue());\n\
\t}\n\n\
\tprivate static Map<String, Instantiator<?>> instantiators;\n\
\tstatic {\n\
\t\tinstantiators = new HashMap<String, Instantiator<?>>();\n\
\t\tinstantiators.put(Types.LIST.getValue(), ListInstantiator.getInstance());\n\
\t\tinstantiators.put(Types.DICT.getValue(), DictInstantiator.getInstance());\n\
\t\tinstantiators.put(Types.IP.getValue(), IPInstantiator.getInstance());\n\
\t\tinstantiators.put(Types.TIME.getValue(), TimeInstantiator.getInstance());\n\
\t\tinstantiators.put(Types.ULONG.getValue(), ULongInstantiator.getInstance());\n\
\t\tinstantiators.put(Types.RESULTSET.getValue(), ResultSetInstantiator.getInstance());\n"

register_inst3 = "\n\t}\n\
\n\
\tpublic static Instantiator<?> getInstantiator(String key) {\n\
\t\treturn instantiators.get(key);\n\
\t}\n\
\n\
}\n"


def GenerateObjects(config, g_objects):
    with open("gen/android/RegisteredInstantiators.java", "w") as f:
        with open("gen/android/Types.java", "w") as f2:
            f.write(register_inst)
            f2.write(types_string)
            list_instantiators = []
            for obj_name, obj in g_objects.iteritems():
                GenerateObject(obj, g_objects)
                if not isinstance(obj, Enum):
                    list_instantiators.append("\t\tinstantiators.put(Types.%s.getValue(), %sInstantiator.getInstance());" % (obj_name.upper(), obj_name))
                    f.write("import %s.client.%s.%sInstantiator;\n"\
                            % (prefix_comm_layer, obj.get_import_path(config.min_version, JAVA), obj_name))
                f2.write(", %s(\"%s\")" % (obj_name.upper(), obj_name))
            f.write(register_inst2)
            f.write("\n".join(list_instantiators))
            f2.write(types_string2)
        f.write(register_inst3)



errors_string = "\
/*************************************************************************\n\
                     LACIE CONFIDENTIAL\n\
\n\
The information contained in this file is the property of LACIE.\n\
The user of this file shall keep information contained herein confidential\n\
and shall protect it in whole or in part from disclosure and dissemination\n\
to third parties. \n\
Without prior written consent of LACIE, you may not reproduce, represent,\n\
or download through any means, the information contained herein in any way\n\
or in any form for a purpose other that the use for internal evaluation,\n\
operation or maintenance within LACIE.\n\
\n\
          (c) LACIE, 2009 - 2010, All Rights Reserved.\n\
 *************************************************************************/\n\
\n\
package %s.client.lib;\n\
\n\
\n\
import java.util.HashMap;\n\
import java.util.Map;\n\
\n\
public enum Errors {\n\
\tTIMEOUT(-2), UNKNOWN(-1),  " % prefix_comm_layer

errors_string2 = ";\n\tInteger name;\n\
\n\
\tErrors(Integer name) {\n\
\t\tthis.name = name;\n\
\t}\n\
\n\
\tpublic Integer getValue() {\n\
\t\treturn this.name;\n\
\t}\n\
\n\
\tprivate static Map<Integer, Errors> mapStatus = new HashMap<Integer, Errors>();\n\
\tstatic {\n\
\t\tfor (Errors type : values()) {\n\
\t\t\tmapStatus.put(type.getValue(), type);\n\
\t\t}\n\
\t}\n\
\n\
\tpublic static Errors fromValue(Integer string2) {\n\
\t\tfinal Errors status = mapStatus.get(string2);\n\
\t\tif (null != status) {\n\
\t\t\treturn status;\n\
\t\t} else {\n\
\t\t\treturn UNKNOWN;\n\
\t\t}\n\
\t}\n\
}"


def GenerateServices(config, global_services, g_objects):
    global g_services
    g_services = global_services
    for ser_name, ser in g_services.iteritems():
        if ser.interface:
            GenerateInterface(ser, g_objects)
        else:
            GenerateService(ser, g_objects)


def GenerateErrors(config, g_errors, errors_offsets):
    with open("gen/android/Errors.java", "w") as f:
        f.write(errors_string)

        errors = []
        for error_name, error in g_errors.iteritems():
            errors.append("%s(%s)" % (error.name.upper(), int(error.id) +\
                int(errors_offsets[error.category])))
        f.write(",".join(errors))
        f.write(errors_string2)


def pre_generate_action(in_config):
    global config
    config = in_config
    import os
    import os.path
    if not os.path.exists("gen/android"):
        os.makedirs("gen/android")
    return set()

def post_generate_action(discard):
    return

from optparse import OptionGroup

def options_check(option, opt_str, value, parser):
    parser.generation.append("android")

def get_options(parser):
    group = OptionGroup(parser, "Android Client Options Options",
                    "No parameters to provide")
    group.add_option("--android", action="callback", help="Generate code for Android\
    JAVA client side", callback=options_check)
    return (group, None)

