#!/usr/bin/python
# -*- coding: utf-8 -*-

#                         LACIE CONFIDENTIAL
#
# The information contained in this file is the property of LACIE.
# The user of this file shall keep information contained herein confidential
# and shall protect it in whole or in part from disclosure and dissemination
# to third parties.
# Without prior written consent of LACIE, you may not reproduce, represent,
# or download through any means, the information contained herein in any way
# or in any form for a purpose other that the use for internal evaluation,
# operation or maintenance within LACIE.
#
#          (c) LACIE, 2009 - 2012, All Rights Reserved.

from optimus_prime.utils import Enum, default_types, DefaultType, Object, get_prefix
from optimus_prime.utils import list_param, dict_param, gen_obj, result_set_param

import os.path

LIB_NAME = u"transformer_lib"
#Use the python namespaces
NAMESPACE_NAME  = u"python"
SEPARATOR       = u"_"

global_objects = None
g_services = None


def lib_path():

    if not global_objects.config.lib_path is None and len(global_objects.config.lib_path) != 0:
        return "%s/" % global_objects.config.lib_path
    return u""


def GenerateEnum(Object):
    with open("gen/inc/%s_%s.hpp" % (global_objects.config.lib_path, Object.real_name(NAMESPACE_NAME, SEPARATOR)), "w") as f:
        f.write("typedef enum{\n")
        for key, value in Object.values.iteritems():
            f.write("\t%s = %s,\n" % (key, value))
        f.write("} %s_t;" % Object.name)

base_serializers = {DefaultType.Unicode: "char *",
                    DefaultType.Integer: "int",
                    DefaultType.Boolean: "bool_t",
                    }


def get_type(param_type, objects):

    if isinstance(param_type, list_param):
        return "list_t *"
    elif isinstance(param_type, dict_param):
        return "dict_t *"
    elif isinstance(param_type, result_set_param):
        return "result_set_t *"
    elif isinstance(param_type, gen_obj):
        if hasattr(param_type, "name"):
            if param_type.name in objects:
                o = objects[param_type.name]
                if isinstance(o, Enum):
                    return get_type(o.sub_type, objects)

        return get_type(param_type.name, objects)
    elif param_type in base_serializers:
        return base_serializers[param_type]
    elif param_type in default_types:
        return "%s_t *" % param_type
    else:
        ob = objects[param_type]
	ob_version = ob.version
        if ob_version.is_lower(objects.config.min_version):
            ob_version = objects.config.min_version
        return u"%s_%s_%s_t *" % (param_type, ob_version.version, ob_version.sub_version)


def generateUnicode(f, param_name, affectation, current_type, optional):
    f.write("\t%s = strdup(v.asString().c_str());\n"\
            % (affectation))


def generateBoolean(f, param_name, affectation, current_type, optional):
    f.write("\t%s = bool_to_bool_t(v.asBool());\n"\
            % (affectation))


def generateInteger(f, param_name, affectation, current_type, optional):
    f.write("\t%s = v.asInt();\n"\
            % (affectation))

def object_name(obj_name):
    if obj_name in global_objects:
        obj = global_objects[obj_name]
        ob_version = obj.version
        if ob_version.is_lower(global_objects.config.min_version):
            ob_version = global_objects.config.min_version
        return "%s_%s_%s" % (obj.name, ob_version.version, ob_version.sub_version)
    return obj_name


def generateDefault(f, param_name, affectation, current_type, optional):
    if current_type in global_objects:
        o = global_objects[current_type]
        if isinstance(o, Enum):
            generateFromJson.get(o.sub_type, generateDefault)(f, param_name,\
                    affectation, o.sub_type, optional)
            return
        
        current_type = object_name(o.name) 

    f.write("%s = (%s_t *)%s_deserialize(&v);\n" %\
            (affectation, current_type, current_type))

generateFromJson = {
        DefaultType.Unicode: generateUnicode,
        DefaultType.Integer: generateInteger,
        DefaultType.Boolean: generateBoolean,
}


def deleteUnicode(f, param_name, affectation, current_type, optional):
    f.write("\tsafe_free((void**)&%s);\n" % affectation)


def deleteInteger(f, param_name, affectation, current_type, optional):
    pass

deleteBoolean = deleteInteger


def DefaultDeleter(f, param_name, affectation, current_type, optional):
    if current_type in global_objects:
        o = global_objects[current_type]
        if isinstance(o, Enum):
            deleters.get(o.sub_type, DefaultDeleter)(f, param_name,\
                    affectation, o.sub_type, optional)
            return
        current_type = object_name(o.name)

    if current_type in (DefaultType.List, DefaultType.Dict,
            DefaultType.ResultSet):
        current_type = current_type.lower()
    f.write("\t%s_deleter(%s);\n" %\
            (current_type, affectation))


deleters = {
        DefaultType.Unicode: deleteUnicode,
        DefaultType.Integer: deleteInteger,
        DefaultType.Boolean: deleteBoolean,
    }


def DefaultCopier(f, param_name, affectation, current_type, optional):
    if current_type in global_objects:
        o = global_objects[current_type]
        if isinstance(o, Enum):
            copiers.get(o.sub_type, DefaultCopier)(f, param_name,\
                    affectation, o.sub_type, optional)
            return

        current_type = object_name(o.name)
    if current_type in (DefaultType.List, DefaultType.Dict,
            DefaultType.ResultSet):
        current_type = current_type.lower()
    elif current_type == DefaultType.Unicode:
        current_type = "string"

    f.write("\t%s_copier((void**)&%s, %s);\n" %\
            (current_type, affectation, param_name))


def copyUnicode(f, param_name, affectation, current_type, optional):
    f.write("\t%s = strdup(%s);\n" % (affectation, param_name))


def noCopy(f, param_name, affectation, current_type, optional):
    f.write("\t%s = %s;\n" % (affectation, param_name))

copiers = {
        DefaultType.Integer: noCopy,
        DefaultType.Boolean: noCopy,
        }


def serializeUnicode(f, obj_block, param_name, affectation, current_type, optional):
    f.write("\t%s[\"%s\"] = %s;\n" % (obj_block, param_name, affectation))


def serializeInteger(f, obj_block, param_name, affectation, current_type, optional):
    f.write("\t%s[\"%s\"] = %s;\n" % (obj_block, param_name, affectation))


def serializeBoolean(f, obj_block, param_name, affectation, current_type, optional):
    f.write("\t%s[\"%s\"] = True==%s?true:false;\n" % (obj_block, param_name, affectation))


def serializeDefault(f, obj_block, param_name, affectation, current_type, optional):
    if current_type in global_objects:
        o = global_objects[current_type]
        if isinstance(o, Enum):
            generateToJson.get(o.sub_type, generateDefault)(f, obj_block, param_name,\
                    affectation, o.sub_type, optional)
            return
        current_type = object_name(o.name)
    if current_type == DefaultType.List:
        current_type = "list"
    elif current_type == DefaultType.Dict:
        current_type = "dict"
    elif current_type == DefaultType.ResultSet:
        current_type = "result_set"
    f.write("%s_serialize(%s, &%s[\"%s\"]);\n" %\
            (current_type, affectation, obj_block, param_name))

generateToJson = {
        DefaultType.Unicode: serializeUnicode,
        DefaultType.Integer: serializeInteger,
        DefaultType.Boolean: serializeBoolean,
    }
base_type = [
                DefaultType.Unicode,
                DefaultType.Integer,
                DefaultType.Boolean,
                "void"
                ]


def get_imports(o_object, objects):
    string = u""
    for obj_type in o_object.get_imports():
        if not(obj_type in base_type) and not(obj_type in default_types):
            ob = objects[obj_type]
            if isinstance(ob, Enum):
                #string += "#include <%slib/%s.hpp>\n" % obj_type
                pass
            else:
                ob_version = ob.version
                if ob_version.is_lower(objects.config.min_version):
                    ob_version = objects.config.min_version
                obj_name = "%s.%s/%s" % (ob_version.version, ob_version.sub_version, ob.real_name(NAMESPACE_NAME, SEPARATOR))
                string += "#include \"%s%s.hpp\"\n" % (lib_path(), obj_name)
             #string += "from %s import %s\n" % (ob.get_import_path(PYTHON, False), obj_type)
        elif obj_type in default_types:
            string += "#include <%s/s%s.hpp>\n" % (LIB_NAME, obj_type.lower())

        else:
            if not (obj_type in base_type):
                raise Exception("WHAT!!!! %s is a strange type" % obj_type)
    string += "\n"
    return string


def print_bit_index(index):
    return hex(1 << index)[2:]


def check_keyword(keyword):
    if keyword in ["public", "static", "auto"]:
        return "_%s" % keyword
    return keyword


def GenerateObject(Object, objects):
    '''
    Generate the corresponding
    '''
    if isinstance(Object, Enum):
        #GenerateEnum(Object)
        return

    obj_version = Object.version
    if obj_version.is_lower(objects.config.min_version):
        obj_version = objects.config.min_version

    path_obj = "%s%s.%s/%s" % (lib_path(), obj_version.version, obj_version.sub_version, Object.real_name(NAMESPACE_NAME, SEPARATOR))
    print "Generating gen/inc/%s" % path_obj 

    attributes = Object.get_attributes()
    if_name = "%s_%s_%s_%s" % (get_prefix().upper(), obj_version.version, obj_version.sub_version, Object.name.upper())

    obj_name = "%s_%s_%s" % (Object.name, obj_version.version, obj_version.sub_version)

    for prefix_path in ("gen/inc", "gen/src"):
        if not os.path.exists(os.path.join(prefix_path, os.path.dirname(path_obj))):
            os.makedirs(os.path.join(prefix_path, os.path.dirname(path_obj)))

    with open("gen/inc/%s.hpp" % path_obj, "w") as f:
        f.write("#ifndef %s\n" % if_name) 
        f.write("#define %s\n" % if_name)
        f.write("#include <%s/tools.hpp>\n" % LIB_NAME)
        f.write(get_imports(Object, objects))

        f.write("typedef struct %s_s %s_t;\n" % (obj_name, obj_name))
        i = 0
        for param, attribute in attributes.iteritems():
            f.write("#define %s_%s_BIT  0x%s\n" % (obj_name.upper(), param.upper(), print_bit_index(i)))
            f.write("#define SET_%s_%s(value) (value |= %s_%s_BIT)\n" %
                    (obj_name.upper(), param.upper(),
                obj_name.upper(), param.upper()))
            f.write("#define GET_%s_%s(value) (value & %s_%s_BIT)\n" %
                    (obj_name.upper(), param.upper(),
                obj_name.upper(), param.upper()))
            i += 1

        f.write("void * %s_deserialize(void * jValue);\n" %\
                obj_name)
        f.write("void *%s_serialize(const void *, void* jValue);\n" % obj_name)
        f.write("void %s_copier(void **out, const void* in);\n" % obj_name)
        f.write("void %s_deleter(void *obj);\n" % obj_name)
        f.write("%s_t *%s_new();\n" % (obj_name, obj_name))
        f.write("void %s_init(%s_t *value);\n" % (obj_name, obj_name))
        f.write("void %s_destroy(%s_t *value);\n" % (obj_name, obj_name))

        #Now we have to generate the function headers that can be used by the
        #client.
        for param, attribute in attributes.iteritems():
            if attribute.read():
                f.write("const %s %s_get%s(const %s_t *%s);\n" % (\
                    get_type(attribute.type,\
                    objects), obj_name.lower(),
                    param.capitalize(),
                    obj_name,
                    obj_name.lower()))
            if attribute.write():
                f.write("void %s_set%s(%s_t *%s, const %s value);\n" %\
                        (obj_name.lower(), param.capitalize(),
                            obj_name,
                            obj_name.lower(),
                            get_type(attribute.type, objects)))
        f.write("\n#endif//%s\n" % if_name)

    with open("gen/src/%s.cpp" % (path_obj), "w") as f:
        f.write("#include \"%s.hpp\"\n" % (path_obj))
        f.write("#include <%s/tools.hpp>\n" % LIB_NAME)
        f.write("#include <json/json.h>\n")

        f.write("\ntypedef struct %s_s{\n" % obj_name)
        for param, attribute in attributes.iteritems():
            f.write("\n\t%s %s;" % (get_type(attribute.type, objects),
                check_keyword(param)))
            if attribute.type.is_optional:
                f.write("\n\tbool_t %s_is_set;" % param)

        f.write("\n\tlong properties_bitfield;")
        f.write("\n} %s_t;\n" % obj_name)

        for param, attribute in attributes.iteritems():
            if attribute.read():
                f.write("const %s %s_get%s(const %s_t *%s){\n" % (\
                    get_type(attribute.type,\
                    objects), obj_name.lower(),
                    param.capitalize(),
                    obj_name,
                    obj_name.lower()))
                f.write("\treturn %s->%s;\n}\n" % (obj_name.lower(),
                    check_keyword(param)))

                f.write("void %s_setJson%s(void *obj, const Json::Value& value){\n" % \
                        (obj_name.lower(), param.capitalize()))
                f.write("\t%s_t *%s = (%s_t*)obj;\n" % (obj_name,\
                    obj_name.lower(), obj_name))
                    #Now depending on the type we have many things to do :)
                f.write("\tJson::Value v = value.get(\"%s\",\
 Json::nullValue);\n" % param)
                if attribute.hidden:
                    f.write("\tif (!v.isNull()){\n\t")

                v_type = attribute.type
                sub_type = None
                if hasattr(v_type, "sub_type"):
                    sub_type = v_type.sub_type
                    if sub_type == DefaultType.Unicode:
                        sub_type = "string"

                if isinstance(v_type, list_param):
                    f.write("\t %s->%s= list_new(\"%s\", %s_deleter,\
                         %s_deserialize, %s_serialize, %s_copier);\n" %
                            (obj_name.lower(), param, v_type.sub_type, object_name(sub_type),\
                                object_name(sub_type), object_name(sub_type), object_name(sub_type)))

                    f.write("\t%s->%s = (list_t*)list_deserialize(%s->%s, &v);\n" %
                            (obj_name.lower(), param, obj_name.lower(), param))
                elif isinstance(v_type, result_set_param):
                    f.write("\t %s->%s= result_set_new(\"%s\", %s_deleter,\
                         %s_deserialize, %s_serialize, %s_copier);\n" %
                            (obj_name.lower(), param, v_type.sub_type, object_name(sub_type),\
                                object_name(sub_type), object_name(sub_type), object_name(sub_type)))

                    f.write("\t%s->%s = (result_set_t*)result_set_deserialize(%s->%s, &v);\n" %
                            (obj_name.lower(), param, obj_name.lower(), param))
                elif isinstance(v_type, dict_param):
                    f.write("\t %s->%s= dict_new(\"%s\", %s_deleter,\
                         %s_deserialize, %s_serialize, %s_copier);\n" %
                            (obj_name.lower(), param, v_type.sub_type, object_name(sub_type),\
                                object_name(sub_type), object_name(sub_type), object_name(sub_type)))

                    f.write("\t%s->%s = (dict_t*)dict_deserialize(%s->%s, &v);\n" %
                            (obj_name.lower(), param, obj_name.lower(), param))
                else:
                    #f.write("\t")
                    if v_type.name in objects:
                        o = objects[v_type.name]
                        if isinstance(o, Enum):
                            name = o.sub_type
                        else:
                            name = v_type.name
                    elif sub_type:
                        name = sub_type
                    else:
                        name = v_type.name
                    generateFromJson.get(name,\
                            generateDefault)(f, param, "%s->%s" %\
                            (obj_name.lower(), check_keyword(param)), attribute.type.name, attribute.type.is_optional)

                if attribute.hidden:
                    f.write("\t}\n")
                f.write("\tSET_%s_%s(%s->properties_bitfield);\n" %
                        (obj_name.upper(), param.upper(),
                    obj_name.lower()))

                f.write("}\n")
            if attribute.write():
                f.write("void %s_set%s(%s_t *%s, const %s value){\n" %\
                        (obj_name.lower(), param.capitalize(),
                            obj_name,
                            obj_name.lower(),
                            get_type(attribute.type, objects)))
                if attribute.type.is_optional:
                    f.write("\t%s->%s_is_set = True;\n" % (obj_name.lower(),
                        param))
                f.write("\tif (0 != GET_%s_%s(%s->properties_bitfield)){\n" %
                        (obj_name.upper(), param.upper(), obj_name.lower()))
                f.write("\t")
                # Now copy the incoming structure
                if isinstance(attribute.type, list_param):
                    v_type_name = DefaultType.List
                elif isinstance(attribute.type, result_set_param):
                    v_type_name = DefaultType.ResultSet
                elif isinstance(attribute.type, dict_param):
                    v_type_name = DefaultType.Dict
                else:
                    v_type_name = attribute.type.name

                deleters.get(v_type_name, DefaultDeleter)(f, param, "%s->%s" %\
                    (obj_name.lower(), check_keyword(param)), v_type_name,
                    attribute.type.is_optional)
                f.write("\t}\n")
                f.write("\telse{\n")
                f.write("\t\tSET_%s_%s(%s->properties_bitfield);\n" %
                        (obj_name.upper(), param.upper(),
                    obj_name.lower()))
                f.write("\t}\n")

                copiers.get(v_type_name, DefaultCopier)(f, "value", "%s->%s" %\
                    (obj_name.lower(), check_keyword(param)), v_type_name,
                    attribute.type.is_optional)

                f.write("}\n")

        f.write("void * %s_deserialize(void *vValue){\n" %
                obj_name)
        f.write('\tJson::Value &jValue = *(Json::Value*)vValue;\n')
        f.write('\tJson::Value properties = jValue.get("__properties__", Json::nullValue);\n')
        f.write('\t%s_t *%s = %s_new();\n' % (obj_name, obj_name.lower(),\
            obj_name))
        for param, attribute in attributes.iteritems():
            if attribute.read():
                f.write("\t%s_setJson%s(%s, properties);\n" % (obj_name.lower(),\
                    param.capitalize(),\
                    obj_name.lower()))
        f.write("\treturn %s;\n" % obj_name.lower())
        f.write("}\n")

        f.write("void *%s_serialize(const void *obj, void*jValue){\n" % obj_name)
        f.write("\t%s_t *%s = (%s_t*)obj;\n" % (obj_name,\
            obj_name.lower(), obj_name))
        f.write("\tJson::Value *jObject = (Json::Value*)jValue;\n")
        f.write("\t(*jObject)[\"__type__\"] = std::string(\"%s\");\n" % obj_name)
        f.write("\tJson::Value *value = &(*jObject)[\"__properties__\"];\n")
        for param, attribute in attributes.iteritems():
            if attribute.write():
                if attribute.type.is_optional:
                    f.write("\tif (True == %s->%s_is_set){\n\t" %
                            (obj_name.lower(), param))

                if isinstance(attribute.type, list_param):
                    v_type_name = DefaultType.List
                elif isinstance(attribute.type, result_set_param):
                    v_type_name = DefaultType.ResultSet
                elif isinstance(attribute.type, dict_param):
                    v_type_name = DefaultType.Dict
                else:
                    v_type_name = attribute.type.name

                generateToJson.get(v_type_name, serializeDefault)(f,
                "\t(*value)", param, "%s->%s" %\
                (obj_name.lower(), check_keyword(param)), v_type_name,
                attribute.type.is_optional)
                if attribute.type.is_optional:
                    f.write("\t}\n")

        f.write("\treturn jObject;\n")
        f.write("}\n")

        f.write("void %s_deleter(void *obj){\n" % obj_name)
        f.write("\t%s_t* %s = (%s_t*)obj;\n" % (obj_name,\
            obj_name.lower(), obj_name))
        f.write("\t%s_destroy(%s);\n" % (obj_name, obj_name.lower()))
        f.write("\tfree(obj);\n")
        f.write("}\n")

        f.write("void %s_copier(void **out, const void* in)\n" % obj_name)
        f.write("{\n")
        f.write("\t%s_t *current = (%s_t*)in;\n" % (obj_name, obj_name))
        f.write("\t%s_t *new_obj = %s_new();\n" % (obj_name, obj_name))
        for param, attribute in attributes.iteritems():
            f.write("\tif(0 != GET_%s_%s(current->properties_bitfield)){\n" %
                    (obj_name.upper(), param.upper()))
            f.write("\t")
            if isinstance(attribute.type, list_param):
                v_type_name = DefaultType.List
            elif isinstance(attribute.type, result_set_param):
                v_type_name = DefaultType.ResultSet
            elif isinstance(attribute.type, dict_param):
                v_type_name = DefaultType.Dict
            else:
                v_type_name = attribute.type.name

            copiers.get(v_type_name, DefaultCopier)(f, "current->%s" %
                    check_keyword(param), "new_obj->%s" %\
            (check_keyword(param)), v_type_name, attribute.type.is_optional)
            f.write("\tnew_obj->properties_bitfield = current->properties_bitfield;\n")
            f.write("\t}\n")

        f.write("\t*out = new_obj;\n")

        f.write("}\n")

        f.write("%s_t *%s_new(){\n" % (obj_name, obj_name))
        f.write("\t%s_t *obj_p = (%s_t*)malloc(sizeof(%s_t));\n" %\
                (obj_name, obj_name, obj_name))
        f.write("\t%s_init(obj_p);\n" % obj_name)
        f.write("\treturn obj_p;\n")
        f.write("}\n")
        f.write("void %s_init(%s_t *obj_p)\n{\n" % (obj_name, obj_name))
        f.write("\tmemset(obj_p, 0, sizeof(%s_t));\n" % obj_name)
        f.write("}\n")

        f.write("void %s_destroy(%s_t *%s)\n{\n" % (obj_name,
            obj_name, obj_name.lower()))
        for param, attribute in attributes.iteritems():
            f.write("\tif(0 != GET_%s_%s(%s->properties_bitfield)){\n" %
                    (obj_name.upper(), param.upper(),
                obj_name.lower()))
            f.write("\t")
            if isinstance(attribute.type, list_param):
                v_type_name = DefaultType.List
            elif isinstance(attribute.type, result_set_param):
                v_type_name = DefaultType.ResultSet
            elif isinstance(attribute.type, dict_param):
                v_type_name = DefaultType.Dict
            else:
                v_type_name = attribute.type.name

            deleters.get(v_type_name, DefaultDeleter)(f, param, "%s->%s" %\
            (obj_name.lower(), check_keyword(param)), v_type_name,
            attribute.type.is_optional)
            f.write("\t}\n")

        f.write("}\n")


def GenerateService(Object, objects):
    print "generating services"

    functions = [t for t in Object.functions.iteritems()]
    if Object.extends:
        for extend in Object.extends:
            interface = g_services[extend]
            functions.extend(interface.functions.iteritems())
    obj_version = Object.version
    if obj_version.is_lower(objects.config.min_version):
        obj_version = objects.config.min_version

        
    service_name = "%s_%s_%s" % (get_prefix(), "_".join(Object.namespaces[NAMESPACE_NAME].split(".")), Object.name)

    service_path = "%s%s.%s/%s_service" % (lib_path(), obj_version.version, obj_version.sub_version, service_name)

    for prefix_path in ("gen/inc", "gen/src"):
        if not os.path.exists(os.path.join(prefix_path, os.path.dirname(service_path))):
            os.makedirs(os.path.join(prefix_path, os.path.dirname(service_path)))

    with open("gen/inc/%s.hpp" % service_path, "w") as f:
        f.write("#ifndef %s_SERVICE\n" % (service_name.upper()))
        f.write("#define %s_SERVICE\n" % (service_name.upper()))
        f.write("#include <%s/tools.hpp>\n" % LIB_NAME)
        f.write("#include <%s/client_http.hpp>\n" % LIB_NAME)
        f.write(get_imports(Object, objects))
        if Object.extends:
            for extend in Object.extends:
                interface = g_services[extend]
                f.write(get_imports(interface, objects))

        for func_name, function in functions:
            if function.async:
                #Async functions are not correctly handled yet. So just skip them...
                pass 
            f_name = function.display_name
            params = []
            for key, v_type in function.params[0]:
                params.append("const %s in_%s" % (get_type(v_type, objects), key))

            for key, v_type in function.params[1]:
                params.append("%s *out_%s" % (get_type(v_type, objects), key))
            if len(params) == 0:
                f.write("\nint %s_%s_%s_%s(client_context_t *context_p);" %
                        (service_name, obj_version.version, obj_version.sub_version, f_name))
            else:
                f.write("\nint %s_%s_%s_%s(client_context_t *context_p, %s);" % (service_name, obj_version.version, obj_version.sub_version, f_name, ", ".join(params)))
        f.write("\n#endif//%s" % service_name.upper())

    with open("gen/src/%s.cpp" % (service_path), "w") as f:
        f.write("#include <string>\n")
        f.write("#include \"%s.hpp\"\n" % (service_path))
        f.write("#include <%s/client_http.hpp>\n" % LIB_NAME)
        f.write("#include <json/json.h>\n")
        for func_name, function in functions:
            f_name = function.display_name
            params = []
            for key, v_type in function.params[0]:
                params.append("const %s in_%s" % (get_type(v_type, objects), key))

            for key, v_type in function.params[1]:
                params.append("%s *out_%s" % (get_type(v_type, objects), key))

            if len(params) == 0:
                f.write("\nint %s_%s_%s_%s(client_context_t *context_p){\n" %
                        (service_name, obj_version.version, obj_version.sub_version, f_name))
            else:
                f.write("\nint %s_%s_%s_%s(client_context_t *context_p, %s){\n" % (service_name, obj_version.version, obj_version.sub_version, f_name, ", ".join(params)))

            f.write("\tJson::Value value(Json::objectValue);\n")
            for key, v_type in function.params[0]:
                f.write("\t")
                if isinstance(v_type, list_param):
                    v_type_name = DefaultType.List
                elif isinstance(v_type, result_set_param):
                    v_type_name = DefaultType.ResultSet
                elif isinstance(v_type, dict_param):
                    v_type_name = DefaultType.Dict
                else:
                    v_type_name = v_type.name

                generateToJson.get(v_type_name, serializeDefault)(f, 'value', "%s" % key, "in_%s" %\
                (key), v_type_name, False)

            url_prefix = u"api/external"
            if function.private:
                url_prefix = u"api/local"
            f.write("\tstd::string url(\"/%s/%s.%s/%s.%s\");\n" %\
                    (url_prefix, obj_version.version, obj_version.sub_version, Object.identification, func_name))
            f.write("\tstd::string resp;\n")
            f.write("\tint res = send_request(context_p, url, value.toStyledString(), &resp);\n")
            f.write("\tbool parsingSuccessful;\n")
            f.write("\tJson::Value jResp;\n")
            f.write("\tJson::Reader reader;\n")
            f.write("\tJson::Value v;\n")

            f.write("\tif ( 0 != res)\n")
            f.write("\t{\n")
            #Now we have to extract the JSON
            f.write("\t\tparsingSuccessful = reader.parse(resp, jResp);\n")
            f.write("\t\tif ( !parsingSuccessful )\n\t\t{\n\t\treturn res;\n\t\t}\n")
            f.write("\t\tv = jResp.get(\"real_id\", Json::nullValue);\n")
            f.write("\t\tres = v.asInt();\n")
            f.write("\t\treturn res;\n")
            f.write("\t}\n")
            f.write("\tparsingSuccessful = reader.parse(resp, jResp);\n")
            f.write("\tif ( !parsingSuccessful )\n\t{\n\t\treturn -1;\n\t}\n")
            for key, v_type in function.params[1]:
                f.write("\tv =  jResp.get(\"%s\", Json::nullValue);\n" % key)
                output = ""
                sub_type = None
                if hasattr(v_type, "sub_type"):
                    sub_type = v_type.sub_type
                    if sub_type == DefaultType.Unicode:
                        sub_type = "string"

                if isinstance(v_type, list_param):
                    output += "\tlist_t *%s = list_new(\"%s\", %s_deleter,\
 %s_deserialize, %s_serialize, %s_copier);\n" % (key, v_type.sub_type, object_name(sub_type),\
         object_name(sub_type), object_name(sub_type), object_name(sub_type))
                    output += "\t%s = (list_t*)list_deserialize(%s, &v);\n" % (key, key)
                elif isinstance(v_type, result_set_param):
                    output += "\tresult_set_t *%s = result_set_new(\"%s\", %s_deleter,\
 %s_deserialize, %s_serialize, %s_copier);\n" % (key, v_type.sub_type, object_name(sub_type),\
         object_name(sub_type), object_name(sub_type), object_name(sub_type))
                    output += "\t%s = (result_set_t*)result_set_deserialize(%s, &v);\n" % (key, key)
                elif isinstance(v_type, dict_param):
                    output += "\tdict_t *%s = dict_new(\"%s\", %s_deleter,\
 %s_deserialize, %s_serialize, %s_copier);\n" % (key, v_type.sub_type, object_name(sub_type),
         object_name(sub_type), object_name(sub_type), object_name(sub_type))
                    output += "\t%s = (dict_t*)dict_deserialize(%s, &v);\n" % (key, key)
                else:
                    #f.write("\t")
                    if v_type.name in objects:
                        o = objects[v_type.name]
                        if isinstance(o, Enum):
                            name = o.sub_type
                        else:
                            name = v_type.name

                    elif sub_type:
                        name = sub_type
                    else:
                        name = v_type.name
                    generateFromJson.get(name,\
                            generateDefault)(f, key, "%s %s" %\
                            (get_type(name, objects), key), name,\
                            False)

                f.write(output)
                f.write("\t*out_%s = %s;\n" % (key, key))

            f.write("\treturn 0;\n")
            f.write("\n}\n")

    #f.write("%s" % Object.identification)
    #f.write("\n")
    #f.write("-" * len(Object.identification))
    #f.write("\n\n")

    #f.write("%s\n" % Object.comments)

    #for func_name, function in Object.functions.iteritems():
    #    f_name = function.display_name
    #    if server:
    #        f_name += "(%s)" % func_name
    #    f.write("\n%s\n" % f_name)
    #    f.write("`" * len(f_name))
    #    f.write("\n")
    #    f.write("\n%s\n" % function.comments)
    #    f.write("\ninput\n")
    #    f.write("~~~~~\n\n")
    #    if len(function.params[0]) == 0:
    #        f.write("None\n")
    #    else:
    #        for key, v_type in function.params[0].iteritems():
    #            f.write("- %s: %s\n" % (key, get_type(v_type)))
    #    f.write("\noutput\n")
    #    f.write("~~~~~~\n\n")
    #    if len(function.params[1]) == 0:
    #        f.write("None\n")
    #    else:

    #        for key, v_type in function.params[1].iteritems():
    #            f.write("- %s: %s\n" % (key, get_type(v_type)))
    #    f.write("\n")


def GenerateError(error, offsets, f):
    f.write("\t%s%s=%s,\n" % (get_prefix(), error.name, error.real_id))


def pre_generate_action(discard):
    if not os.path.exists("gen/inc"):
        os.makedirs("gen/inc")
    if not os.path.exists("gen/src"):
        os.makedirs("gen/src")
    return set()


def post_generate_action(discard):
    pass


def GenerateObjects(config, g_objects):
    global global_objects
    global_objects = g_objects
    print "Objectssss"
    for obj_name, obj in g_objects.iteritems():
        print obj_name
        GenerateObject(obj, g_objects)


def GenerateServices(config, global_services, g_objects):
    global global_objects, g_services
    g_services = global_services
    global_objects = g_objects

    for ser_name, ser in g_services.iteritems():
        if ser.interface:
            continue
        GenerateService(ser, g_objects)


def GenerateErrors(config, g_errors, offsets):
    with open("gen/inc/%serrors.hpp" % get_prefix(), "w") as f:
        f.write("typedef enum{\n")
        for error_name, error in g_errors.iteritems():
            GenerateError(error, offsets, f)
            print error_name, error.name
        f.write("} %serrors;\n" % get_prefix())

from optparse import OptionGroup


def options_check(option, opt_str, value, parser):
    parser.generation.append("c")


def get_options(parser):
    group = OptionGroup(parser, "C Client Options",
                    "No parameters to provide")
    group.add_option("--gen_c", action="callback", help="Generate code for C client side", callback=options_check)
    return (group, None)

