#!/usr/bin/python
# -*- coding: utf-8 -*-

#                         LACIE CONFIDENTIAL
#
# The information contained in this file is the property of LACIE.
# The user of this file shall keep information contained herein confidential
# and shall protect it in whole or in part from disclosure and dissemination
# to third parties.
# Without prior written consent of LACIE, you may not reproduce, represent,
# or download through any means, the information contained herein in any way
# or in any form for a purpose other that the use for internal evaluation,
# operation or maintenance within LACIE.
#
#          (c) LACIE, 2009 - 2012, All Rights Reserved.

from optimus_prime.utils import Enum, default_types, DefaultType, Object
from optimus_prime.utils import list_param, dict_param, gen_obj, result_set_param

from tools import (Tree, Leef, base_types, set_tree, get_type, FakeFile, genFakeJSONObject, genFakeJSONService)


import json


TABS = u" "

def GenerateEnum(Object, f):
    f.write("\n%s\n" % Object.name)
    f.write("-" * (len(Object.name)))
    f.write("\n")
    f.write("This is an Enum\n")
    f.write("\n%s\n\n" % Object.comments)
    for key, value in Object.values.iteritems():
        f.write("- %s: %s\n" % (key, value))



def GenerateObject(Object, objects, files):
    '''
    Generate the corresponding Python File
    '''
    f = files["doc"][0]
    if isinstance(Object, Enum):
        GenerateEnum(Object, f)
        return

    f.write("\n%s" % Object.name)
    f.write("\n")
    f.write("-" * (len(Object.name)))
    f.write("\n")
    if Object.comments:
        f.write("\n%s\n" % Object.comments)

    name_size = len("Name")
    type_size = len("type")
    for param, attribute in Object.get_attributes().iteritems():
        true = "true "
        false = "false"
        #if attribute.type.name in base_types:
        #    f.write("\n:%s: %s\n\n" % (param, get_type(attribute.type)))
        #else:
        #    if isinstance(attribute.type, optional_param):
        #        f.write("\n:%s: Optional<`%s`_>\n\n" % (param,
        #            get_type(attribute.type.sub_type)))
        #    else:
        #        f.write("\n:%s: `%s`_\n\n" % (param, get_type(attribute.type)))

        #if attribute.comments:
        #    f.write("%s\n\n" % attribute.comments)

        #if attribute.validator is not None:
        #    f.write("Allowed values %s\n\n" % attribute.validator)
        #f.write("+-----------+-----------+------------+\n")
        #f.write("| **Read**  | **Write** | **Hidden** |\n")
        #f.write("+===========+===========+============+\n")
        #f.write("|   %s   |   %s   |   %s    |\n" % ((attribute.read() and true) or false,\
        #    (attribute.write() and true) or false, (attribute.hidden and true) or\
        #    false))
        #f.write("+-----------+-----------+------------+\n")

        name_size = max(name_size, len(param))
        type_size = max(type_size, len(get_type(attribute.type)))
    type_size += 3
    f.write("\n%s %s ===== ===== ====== ========\n" % ("="*name_size, "="*type_size))
    f.write("%s %s Read  Write Hidden Optional\n" % ("Name" + " "*(name_size -
    len("Name")), "type" + " "*(type_size - len("type"))))
    f.write("%s %s ===== ===== ====== ========\n" % ("="*name_size, "="*type_size))

    for param, attribute in Object.attributes.iteritems():
        if attribute.type.name in base_types:
            attribute_type = get_type(attribute.type)
        else:
            if isinstance(attribute.type, list_param):
                attribute_type = "List<`%s`_>" %\
                    get_type(attribute.type.sub_type)
            elif isinstance(attribute.type, dict_param):
                attribute_type = "Dict<`%s`_>" %\
                    get_type(attribute.type.sub_type)
            else:
                attribute_type = "`%s`_" % get_type(attribute.type)

        f.write("%s %s %s %s %s %s\n" % (write_stuff(param, name_size),
            write_stuff(attribute_type, type_size),
            write_stuff((attribute.read() and true) or false, 5),
            write_stuff((attribute.write() and true) or false, 5),
            write_stuff((attribute.hidden and true) or false, 6),
            write_stuff(hasattr(attribute.type, "is_optional") and attribute.type.is_optional, 8)))
    f.write("%s %s ===== ===== ====== ========\n" % ("="*name_size, "="*type_size))

    for param, attribute in Object.attributes.iteritems():
        if attribute.comments:
            f.write("\n*%s*: %s\n" % (param, attribute.comments))
    f.write("\n")

    # Now let's generate the corresponding JSON
    f = files["annexe"][0]
    f.write("%s\n" % Object.name)
    f.write("%s" % "=" * len(Object.name))
    f.write("\n")
    f.write(".. code-block:: json\n\n")
    temp = FakeFile()
    genFakeJSONObject(temp, 1, Object, objects, 2)
    jDict = json.loads(temp.datas)
    f.write(json.dumps(jDict, indent=2))
    f.write("\n\n")


def genPdfFakeJSONService(f, function, objects, inputJson):

    f.write("\n\n")
    f.write(".. code-block:: json\n\n")
    genFakeJSONService(f, function, objects, inputJson)


def GenerateService(Object, objects, files):
    fannexe = files["annexe"][0]
    fdoc = files["doc"][0]
    for f in [fannexe, fdoc]:
        f.write("%s" % Object.identification)
        f.write("\n")
        f.write("-" * len(Object.identification))
        f.write("\n\n")


    fdoc.write("%s\n" % Object.comments)

    for func_name, function in Object.functions.iteritems():
        f_name = function.display_name
        fdoc.write("\n%s\n" % f_name)
        fdoc.write("`" * len(f_name))
        fdoc.write("\n")

        fdoc.write("Access is granted to %s\n" % function.authentication)
        fdoc.write("\n%s\n" % function.comments)
        fdoc.write("\ninput\n")
        fdoc.write("~~~~~\n\n")
        if len(function.params[0]) == 0:
            fdoc.write("None\n")
        else:
            for key, v_type in function.params[0]:
                fdoc.write("- %s: %s\n" % (key, get_type(v_type)))
        fdoc.write("\noutput\n")
        fdoc.write("~~~~~~\n\n")
        if len(function.params[1]) == 0:
            fdoc.write("None\n")
        else:

            for key, v_type in function.params[1]:
                fdoc.write("- %s: %s\n" % (key, get_type(v_type)))
        fdoc.write("\n")

        fdoc.write("\nerrors\n")
        fdoc.write("~~~~~~\n\n")
        if len(function.errors) == 0:
            fdoc.write("None\n")
        else:
            for error in function.errors:
                fdoc.write("- %s_\n" % error)
        fdoc.write("\n")
        fdoc.write("\n")
        fdoc.write("\n\n")

        fannexe.write("%s\n" % function.display_name)
        fannexe.write("~"* len(function.display_name))
        fannexe.write("\n")
        fannexe.write("input\n")
        genPdfFakeJSONService(fannexe, function, objects, True)
        fannexe.write("\nouput\n")
        genPdfFakeJSONService(fannexe, function, objects, False)
        fannexe.write("\n")


def GenerateError(error, offsets, files):
    f = files["errors"][0]
    f.write("\n")
    f.write("%s\n" % error.name)
    f.write("-" * len(error.name))
    f.write("\n\n")
    f.write("\n%s\n\n" % error.comments)
    f.write(":Category: %s\n" % error.category)
    f.write(":id: %s\n" % error.id)
    f.write(":text: %s\n" % error.text)
    f.write(":real_id: %s\n\n" % (int(error.id) + int(offsets[error.category])))


import os
try:
    import pwd
    user_name = (pwd.getpwuid(os.getuid()).pw_gecos.split(',')[0])
except:
    user_name = "Unknown User Name"
#Document HEADER
doc_header = "=========================================\n\
{DOC_TITLE}\n\
=========================================\n\
\n\
.. |date| date::\n\
\n\
:Authors:    %s\n\
:Copyright:  Copyright |copy| 2010 LaCie\n\
:Version:    1.0\n\
:Revision:   Draft-0.1\n\
:Date:       |date|\n\
\n\
.. |copy| unicode:: 0xA9\n\
.. |SAS| replace:: Software Architecture Specification\n\
.. |SRS| replace:: Software Requirements Specification\n\
.. |TND| replace:: Technical Note\n\
.. |HOW| replace:: HOWTO\n\
.. |logo| image:: images/logo_lacie.jpg\n\
            :height: 100px\n\
            :width: 220px\n\
\n\
.. raw:: pdf\n\
\n\
    PageBreak oneColumn\n\
\n\
.. sectnum::\n\
\n\
.. contents::\n\
\n\
.. header::\n\
\n\
    .. class:: headertable\n\
\n\
        +------------------------+------------------------+\n\
        | |logo|                 | |SAS| for              |\n\
        |                        |                        |\n\
        |                        | ###Title###            |\n\
        +------------------------+------------------------+\n\
\n\
.. footer::\n\
\n\
    .. class:: footertable\n\
\n\
        +------------------------+------------------------+\n\
        | SAS                    |                        |\n\
        | ###Title###            | ###Page###/###Total### |\n\
        +------------------------+------------------------+\n\
\n\
.. raw:: pdf\n\
\n\
    PageBreak\n\
\n\
" % user_name


def pre_generate_action(config):
    if config.options.pdf:
        files = {}
        files['doc'] = (open("doc/doc.rst", "w"), "Documentation API")
        files['annexe'] = (open("doc/annexe.rst", "w"), "API Annexe JSON generation")
        files['errors'] = (open("doc/errors.rst", "w"), "API Errors")
        import os
        path = os.path.dirname(os.path.abspath( __file__ ))
        for f, title in files.values():
            f.write(doc_header.replace("{DOC_TITLE}", title))
        return (files, )
    else:
        from html import pre_generate_action as html_pregenerate_action
        return html_pregenerate_action(config)

def post_generate_action(config, *args):
    if config.options.pdf:
        for f, title in files.values():
            f.close()
    else:
        from html import post_generate_action as html_post_generate_action
        return html_post_generate_action(config, *args)

tree = None

def GenerateObjects(config, g_objects, *args):
    if config.options.pdf:
        files = args[0]
        f = files["doc"][0]
        f.write("Objects Definitions\n")
        f.write("===================\n\n")

        global tree
        tree = Tree(g_objects)
        set_tree(tree)

        for obj_name, obj in g_objects.iteritems():
            leef = Leef(obj, g_objects)
            tree.add_leef(leef)

        for id in tree.levels:
            obj_list = tree.levels[id]
            alpha_order = [leef_name for leef_name in obj_list]
            alpha_order.sort()

            for leef_name in alpha_order:
                GenerateObject(obj_list[leef_name].obj_decl, g_objects, files)
    else:
        from html import GenerateObjects as HTMLGenerateObjects
        HTMLGenerateObjects(config, g_objects, *args)

def GenerateServices(config, g_services, g_objects, *args):
    if config.options.pdf:
        files = args[0]
        for f, title in [files["doc"], files["annexe"]]:
            f.write("Services Definitions\n")
            f.write("====================\n\n")

        for ser_name, ser in g_services.iteritems():
            GenerateService(ser, g_objects, files)
    else:
        from html import GenerateServices as HTMLGenerateServices
        HTMLGenerateServices(config, g_services, g_objects, *args)

def GenerateErrors(config, g_errors, offsets, *args):
    if config.options.pdf:
        files = args[0]
        f = files["errors"][0]
        f.write("Errors Definitions\n")
        f.write("==================\n")
        f.write("\n")
        f.write("- *Category* stands for the category the errors belongs to\n")
        f.write("- *real_id* is the unique id used to identify an errors. It\
 dep    ends on the error ID and the category\n")

        error_ordering = {}
        for error_name, error in g_errors.iteritems():
            real_id = (int(error.id) + int(offsets[error.category]))
            error_ordering[real_id] = error
        error_order = error_ordering.keys()
        error_order.sort()
        for error_id in error_order:
            GenerateError(error_ordering[error_id], offsets, files)
    else:
        from html import GenerateErrors as HTMLGenerateErrors
        HTMLGenerateErrors(config, g_errors, offsets, *args)

from optparse import OptionGroup

class DocConfig(object):
    def __init__(self):
        self.html   = False
        self.pdf    = False
    def set_html(self):
        self.html   = True

    def set_pdf(self):
        self.pdf    = True

def options_check(option, opt_str, value, parser):
    value = []
    for arg in parser.rargs:
        if arg[:2] == "__" or \
            arg not in ["html", "pdf"]:
            break
        elif (arg == "html" and "pdf" in value) or \
            (arg == "pdf" and "html" in value):
            raise OptionValueError("Can't have pdf and html")
        getattr(parser.doc, "set_%s" % arg)()
        value.append(arg)

    del parser.rargs[:len(value)]
    parser.generation.append("doc")

def get_options(parser):
    group = OptionGroup(parser, "Doc generation for client part",
                    "No parameters to provide")
    group.add_option("--doc", action="callback", help="Generate documentation [html|pdf]\
 for client", callback=options_check)
    return (group, DocConfig())

