from tools import Tree, Leef, set_tree, base_types, genFakeJSONObject, FakeFile, genFakeJSONService
from optimus_prime.utils import Enum, DefaultType
from optimus_prime.utils import list_param, dict_param, gen_obj, result_set_param

import json


def get_type(param_type):
    if isinstance(param_type, list_param):
        temp_value = "<a href='List.html'>List</a><<nop>%s</nop>>" % get_type(param_type.sub_type)
    elif isinstance(param_type, dict_param):
        temp_value = "<a href='Dict.html'>Dict</a><<nop>%s</nop>>" % get_type(param_type.sub_type)
    elif isinstance(param_type, result_set_param):
        temp_value = "<a href='ResultSet.html'>ResultSet</a><<nop>%s</nop>>" % get_type(param_type.sub_type)
    elif isinstance(param_type, gen_obj):
        temp_value = get_type(param_type.name)
    else:
        temp_value = param_type
    return temp_value


def get_type_link(param_type, objects):
    if isinstance(param_type, list_param):
        temp_value = "<a href='List.html'>List</a><<nop>%s</nop>>" % get_type_link(param_type.sub_type, objects)
    elif isinstance(param_type, dict_param):
        temp_value = "<a href='Dict.html'>Dict</a><<nop>%s</nop>>" % get_type_link(param_type.sub_type, objects)
    elif isinstance(param_type, result_set_param):
        temp_value = "<a href='ResultSet.html'>ResultSet</a><<nop>%s</nop>>" % get_type_link(param_type.sub_type, objects)
    elif isinstance(param_type, gen_obj):
        try:
            obj = objects[param_type.name]
            temp_value = "<a href='%s.%s.html'>%s</a>" % (obj.namespaces['python'], obj.name, obj.name)
        except KeyError:
            temp_value = param_type.name

    else:
        try:
            obj = objects[param_type]
            temp_value = "<a href='%s.%s.html'>%s</a>" % (obj.namespaces['python'], obj.name, obj.name)
        except KeyError:
            temp_value = param_type
    return temp_value


def write_header(f, title):
    f.write("<html>\n")
    f.write("<head>\n")
    f.write("<title>%s</title>" % title)
    f.write("<!-- Le styles -->\n")
    #f.write('<link href="bootstrap/css/bootstrap.css" rel="stylesheet">\n')
    f.write('<style type="text/css">\n')
    f.write('body {\n')
    f.write('padding-top: 60px;\n')
    f.write('padding-bottom: 40px;\n')
    f.write('}\n')
    f.write('.sidebar-nav {\n')
    f.write('padding: 9px 0;\n')
    f.write('}')
    f.write('</style>\n')
    #f.write('<link href="bootstrap/css/bootstrap-responsive.css" rel="stylesheet">\n')
    f.write('<link href="scripts/doc.css" rel="stylesheet">\n')
    f.write('<script type="text/javascript" src="scripts/auth.js"></script>\n')
    f.write('<script type="text/javascript" src="scripts/syntax_highlight.js"></script>\n')
    #f.write('<script type="text/javascript" src="scripts/jquery.min.js"></script>\n')
    f.write('<script type="text/javascript" src="auth.json.js"></script>\n')
    #f.write('<script type="text/javascript" src="scripts/jquery.easing.1.3.js"></script>\n')
    f.write("</head>\n")


def GenerateObject(current_object, objects, *args):

    objects.config.index_html.write("<li><a href='javascript:void(0)' onclick='window.location = \"%s.%s.html\" + location.search;'>%s.%s</a></li>\n" %
        (current_object.namespaces['python'], current_object.name, current_object.namespaces['python'], current_object.name))

    with open("doc/html/%s.%s.html" % (current_object.namespaces['python'], current_object.name), "w") as f:
        write_header(f, current_object.name)
        f.write("<div id='floatMenu'>\n")
        f.write("<h3>%s</h3>\n" % current_object.name)

        if not isinstance(current_object, Enum):
            f.write("<ul>")

            f.write("<li><a href=#attributes class='menu_item'>Attributes</a></li>\n")
            if current_object.comments:
                f.write("<li><a href=#comments class='menu_item'>Comments</a></li>\n")
            f.write("<li><a href=#json_exemple_h2 class='menu_item'>Json Exemple</a></li>\n")

            f.write("</ul>")
        f.write("<a href=\"javascript:void(0)\" onclick=\"window.location='index.html' + location.search;\" class=\"button\">Home</a>")
        f.write("</div>")
        f.write("<div id='main'>")
        f.write("<body onload='nice_display()'>\n")
        f.write("<h1>%s</h1>\n" % current_object.name)

        if isinstance(current_object, Enum):
            f.write("<h3>Values</h3>")
            quotes = ''
            invert = {}
            for key, value in current_object.values.iteritems():
                invert[value] = key

            values = invert.keys()

            if current_object.sub_type == DefaultType.Unicode:
                quotes = '"'
            values.sort()
            f.write("<ul>\n")
            for value in values:
                key = invert[value]
                f.write("<li>%s= %s%s%s</li>\n" % (key, quotes, value, quotes))
            f.write("</ul>\n")
            f.write("</html>\n")
            return
        f.write("")
        f.write("<h2 id=\"attributes\">attributes</h2>\n")
        f.write("<table cellspacing='0'>\n")
        f.write("<thead>\n")
        f.write("<tr>")
        f.write("<th>Name</th><th>Type</th><th>Read</th><th>Write</th><th>Hidden</th><th>Optional</th>\n")
        f.write("</tr>")
        f.write("</thead>\n")
        i = 0
        for param, attribute in current_object.get_attributes().iteritems():
            if 0 == (i % 2):
                f.write("<tr class='even'>\n")
            else:
                f.write("<tr class='odd'>\n")

            if attribute.type.name in base_types:
                attribute_type = get_type(attribute.type)
            else:
                if isinstance(attribute.type, list_param):
                    attribute_type = "<a href='List.html'>List</a><%s>" %\
                        (get_type_link(attribute.type.sub_type, objects))
                elif isinstance(attribute.type, dict_param):
                    attribute_type = "<a href='Dict.html'>Dict</a><%s>" %\
                        (get_type_link(attribute.type.sub_type, objects))
                elif isinstance(attribute.type, result_set_param):
                    attribute_type = "<a href='ResultSet.html'>ResultSet</a><%s>" %\
                        (get_type_link(attribute.type.sub_type, objects))
                else:
                    attribute_type = "%s" % (get_type_link(attribute.type, objects))

            f.write("<td>%s</td><td>%s</td><td>%s</td><td>%s</td><td>%s</td><td>%s</td>\n" % (param,
                attribute_type,
                (attribute.read() and True) or False,
                (attribute.write() and True) or False,
                (attribute.hidden and True) or False,
                hasattr(attribute.type, "is_optional") and attribute.type.is_optional))
            f.write("</tr>\n")
            i = i + 1

        f.write("</table>\n")

        if current_object.comments:
            f.write("<h2>Comments</h2>\n")
            f.write("<br>\n".join(current_object.comments))
            object_comments = True
        else:
            object_comments = False

        comment_header = False
        for param, attribute in current_object.attributes.iteritems():

            if attribute.comments:
                if not object_comments:
                    f.write("<h2>Comments</h2>\n")
                    object_comments = True
                if not comment_header:
                    f.write("<h3>Attributes Comments</h3>\n")
                    comment_header = True
                f.write("<h4>%s</h4>\n" % param)
                f.write("%s\n" % ("<br>".join(attribute.comments)))

        #f.write("</div>")
        f.write("<h2 id=\"json_exemple_h2\">Json Exemple</h2>")
        f.write("<code id='json_exemple'>\n")
        f.write("</code>\n")
        f.write("<script type='text/javascript'>\n")
        f.write("function nice_display(){\n")
        f.write("var str = '\\n\\\n")
        temp = FakeFile()
        genFakeJSONObject(temp, 1, current_object, objects, 2)
        jDict = json.loads(temp.datas)
        f.write(json.dumps(jDict, indent=2).replace('\n', '\\n\\\n'))
        f.write("\\n\\\n\\n\\\n';\n")
        f.write("document.getElementById('json_exemple').appendChild(document.createElement('pre')).innerHTML = syntaxHighlight(str);\n")
        f.write("}\n</script>")
        f.write("\n\n")
        f.write("</div>\n")
        f.write("</body>\n")
        f.write("</html>\n")

tree = None


def pre_generate_action(config):
    import shutil
    import os
    html_default_path = os.path.join(os.path.dirname(os.path.abspath(__file__)), "html_default")
    for file_name in os.listdir(html_default_path):
        shutil.copy(os.path.join(html_default_path, file_name), "doc/html")
    config.index_html = open("doc/html/index.html", "w")
    write_header(config.index_html, "index")
    config.index_html.write("<body onload='init()''>\n")

    return set()


def post_generate_action(config):
    config.index_html.write("</body>\n")
    config.index_html.write("</html>\n")
    config.index_html.close()


def GenerateObjects(config, g_objects, *args):
    config.index_html.write("<h2>Objects</h2>\n")
    config.index_html.write("Objects are used for transferring information<br>\n")
    config.index_html.write("<ul>\n")
    global tree
    tree = Tree(g_objects)
    set_tree(tree)

    for obj_name, obj in g_objects.iteritems():
        leef = Leef(obj, g_objects)
        tree.add_leef(leef)

    for id in tree.levels:
        obj_list = tree.levels[id]
        alpha_order = [leef_name for leef_name in obj_list]
        alpha_order.sort()

        for leef_name in alpha_order:
            GenerateObject(obj_list[leef_name].obj_decl, g_objects, *args)
    config.index_html.write("</ul>\n")


def GenerateService(current_object, objects, g_services, services_dict, *args):
    if current_object.interface:
        return
    full_service_name = "%s.%s" % (current_object.namespaces['python'], current_object.name)
    #objects.config.index_html.write("<li><a href=%s.html>%s</a></li>\n" %
    #    (full_service_name, full_service_name))
    services_dict.setdefault("global_auth_list", [])
    services_dict.setdefault("services", {})
    services_dict["services"][full_service_name] = {"auth": [], "functions": {}}

    functions = {}
    for func_name in current_object.functions:
        functions[func_name] = current_object.functions[func_name]

    if current_object.extends:
        for extend in current_object.extends:
            interface = g_services[extend]
            functions.update(interface.functions)

    with open("doc/html/%s.html" % (full_service_name), "w") as f:

        write_header(f, current_object.name)
        f.write("<body onload='nice_display()'>\n")
        #f.write("<div id='functions'>\n")
        f.write("<div id='floatMenu'>\n")
        f.write("<h3>%s</h3>\n" % current_object.name)
        f.write("<ul>")
        for func_name in functions:
            f.write("<li><a href=#%s class='menu_item'>%s</a></li>\n" % (func_name, func_name))
        f.write("</ul>")
        f.write("<a href=\"javascript:void(0)\" onclick=\"window.location='index.html' + location.search;\" class=\"button\">Home</a>")
        f.write("</div>")
        f.write("<div id='main'>")
        f.write("<h1>%s</h1>\n" % current_object.name)
        f.write("<div id='functions'>")
        for func_name, function in functions.iteritems():
            f.write("<div>\n")
            f_name = function.display_name
            f.write("<h2 id='%s'>%s</h2>\n" % (f_name, f_name))

            if function.async:
                f.write("<b>\nThis function is <a href=\"async.html\">asynchronous</a></b>")
                f.write("<br/><br/>")

            f.write("\n%s\n" % "<br/>".join(function.comments))
            f.write("<h3>Access</h3>\n")
            f.write("%s<br/>\n" % " ".join(function.authentication))
            for auth in function.authentication:
                if auth not in services_dict["services"][full_service_name]["auth"]:
                    services_dict["services"][full_service_name]["auth"].append(auth)

                if auth not in services_dict["global_auth_list"]:
                    services_dict["global_auth_list"].append(auth)

            services_dict["services"][full_service_name]["functions"][f_name] = function.authentication

            obj_version = current_object.version
            if obj_version.is_lower(objects.config.min_version):
                obj_version = objects.config.min_version

            url_prefix = u"api/external"
            if function.private:
                url_prefix = u"api/local"
            f.write("\n")
            url = "\"%s/%s.%s/%s.%s\"\n" % (
                url_prefix,
                obj_version.version,
                obj_version.sub_version,
                current_object.identification,
                function.func_name
            )
            f.write("<h3>url:</h3>")
            f.write(url)

            f.write("<h3>input</h3>\n")

            f.write("<ul>\n")
            if len(function.params[0]) == 0:
                f.write("<li>None</li>\n")
            else:

                for key, attribute in function.params[0]:
                    f.write("<li>\n")
                    if attribute.name in base_types:
                        attribute_type = get_type(attribute)
                    else:
                        if isinstance(attribute, list_param):
                            attribute_type = "<a href='List.html'>List</a><%s>" %\
                                (get_type_link(attribute.sub_type, objects))
                        elif isinstance(attribute, dict_param):
                            attribute_type = "<a href='Dict.html'>Dict</a><%s>" %\
                                (get_type_link(attribute.sub_type, objects))
                        elif isinstance(attribute, result_set_param):
                            attribute_type = "<a href='ResultSet.html'>ResultSet</a><%s>" %\
                                (get_type_link(attribute.sub_type, objects))
                        else:
                            attribute_type = "%s" % (get_type_link(attribute, objects))

                    f.write("%s %s" % (attribute_type, key))
                    f.write("</li>")
            f.write("</ul>")
            f.write("<h3>output</h3>\n")

            f.write("<ul>\n")
            if len(function.params[1]) == 0:
                f.write("<li>None</li>\n")
            else:

                for key, attribute in function.params[1]:
                    f.write("<li>\n")
                    if attribute.name in base_types:
                        attribute_type = get_type(attribute)
                    else:
                        if isinstance(attribute, list_param):
                            attribute_type = "<a href='List.html'>List</a><%s>" %\
                                (get_type_link(attribute.sub_type, objects))
                        elif isinstance(attribute, dict_param):
                            attribute_type = "<a href='Dict.html'>Dict</a><%s>" %\
                                (get_type_link(attribute.sub_type, objects))
                        elif isinstance(attribute, result_set_param):
                            attribute_type = "<a href='ResultSet.html'>ResultSet</a><%s>" %\
                                (get_type_link(attribute.sub_type, objects))
                        else:
                            attribute_type = "%s" % (get_type_link(attribute, objects))

                    f.write("%s %s" % (attribute_type, key))
                    f.write("</li>")
            f.write("</ul>")

            search_criteria     = function.params[3]
            order_by_criteria   = function.params[4]
            if search_criteria:
                f.write("<h3> <a href='ListInfo.html#search_criteria'>Search Criteria</a></h3>\n")
                f.write("<ul>\n")
                for p, t in search_criteria:
                    f.write("<li>%s: %s</li>" % (p, get_type_link(t, objects)))
                f.write("</ul>\n")

            if order_by_criteria:
                f.write("<h3> <a href='ListInfo.html#order_by_criteria'>Order by Criteria</a> </h3>\n")
                f.write("<ul>\n")
                for p, t in order_by_criteria:
                    f.write("<li>%s: %s</li>" % (p, get_type_link(t, objects)))
                f.write("</ul>\n")

            f.write("<h3>errors</h3>\n")

            if len(function.errors) == 0:
                f.write("None\n")
            else:
                f.write("<ul>\n")
                for error in function.errors:
                    f.write("<li>%s</li>\n" % error)
                f.write("</ul>\n")

            f.write("<h3>Json exemple</h3>\n")
            f.write("<h4>Input</h4>\n")
            f.write("<div id='json_exemple_%s_input'>\n" % func_name)
            f.write("</div>\n")
            f.write("<h4>ouput</h4>\n")
            f.write("<div id='json_exemple_%s_output'></div>\n" % func_name)

            f.write("</div>")

        #end functions <div>
        f.write("</div>")

        f.write("<script type='text/javascript'>\n")
        f.write("function nice_display(){\n")
        for func_name, function in current_object.functions.iteritems():

            f.write("var str_%s_input = '\\n\\\n" % func_name)
            temp = FakeFile()
            genFakeJSONService(temp, function, objects, True)
            jDict = json.loads(temp.datas)
            f.write(json.dumps(jDict, indent=2).replace('\n', '\\n\\\n'))
            f.write("\\n\\\n\\n\\\n';\n")
            f.write("document.getElementById('json_exemple_%s_input').appendChild(document.createElement('pre')).innerHTML = syntaxHighlight(str_%s_input);\n" % (func_name, func_name))

            f.write("var str_%s_output = '\\n\\\n" % func_name)
            temp = FakeFile()
            genFakeJSONService(temp, function, objects, False)
            jDict = json.loads(temp.datas)
            f.write(json.dumps(jDict, indent=2).replace('\n', '\\n\\\n'))
            f.write("\\n\\\n\\n\\\n';\n")
            f.write("document.getElementById('json_exemple_%s_output').appendChild(document.createElement('pre')).innerHTML = syntaxHighlight(str_%s_output);\n" % (func_name, func_name))

        f.write("remove_elements('%s');\n" % full_service_name)
        f.write("}\n</script>")
        f.write("\n\n")

        f.write("</div>\n")
        f.write("</body>\n")
        f.write("</html>\n")


def GenerateServices(config, g_services, g_objects, *args):
    config.index_html.write("<h2>Services</h2>\n")
    config.index_html.write("Services are using Rest Authentication and are used to make calls to the server.<br>\n")
    config.index_html.write("Objects will be sent from client to server / server to client using the services.<br>\n")
    config.index_html.write("<br>\n")
    config.index_html.write("<ul id='services'>\n</ul>")
    keys = {}

    for ser in g_services.values():
        keys["%s.%s" % (ser.namespaces['python'], ser.name)] = ser.name

    keys_keys = keys.keys()
    keys_keys.sort()
    services_dict = {}

    for ser_name in keys_keys:
        ser = g_services[keys[ser_name]]
        GenerateService(ser, g_objects, g_services, services_dict, *args)

    with open("doc/html/auth.json.js", "w") as auth_json_file:
        auth_json = "var auth_dict = "
        auth_json += json.dumps(services_dict["services"])
        auth_json += ";\n"
        auth_json_file.write(auth_json)
        auth_json_file.write("var global_auth_list = %s;" % services_dict["global_auth_list"])


def GenerateErrors(config, g_errors, offsets):
    with open("doc/html/errors.html", "w") as error_html:
        error_html.write("<h2>Errors</h2>\n")
        error_html.write("<ul>\n")
        for err_name, error in g_errors.iteritems():
            error_html.write("<li>%s: %s</li>\n" % (error.name, error.real_id))
        error_html.write("</ul>\n")
