from optimus_prime.utils import default_types, DefaultType, Enum, Object
from optimus_prime.utils import list_param, dict_param, gen_obj, result_set_param
TABS = "   "


def get_type(param_type):
    if isinstance(param_type, list_param):
        temp_value = "List<%s>" % get_type(param_type.sub_type)
    elif isinstance(param_type, dict_param):
        temp_value = "Dict<%s>" % get_type(param_type.sub_type)
    elif isinstance(param_type, result_set_param):
        temp_value = "ResultSet<%s>" % get_type(param_type.sub_type)
    elif isinstance(param_type, gen_obj):
        temp_value = get_type(param_type.name)
    else:
        temp_value = param_type
    return temp_value


class Tree(object):
    def __init__(self, objects):
        self.levels = {}
        self.leeves = {}
        self.objects = objects

    def add_leef(self, leef):
        if not (leef.obj_decl.name in self.leeves):
            self.leeves[leef.obj_decl.name] = leef
            self.levels.setdefault(leef.get_level(), {})
            self.levels[leef.get_level()][leef.obj_decl.name] = leef

    def get_leef(self, type_name):
        if not (type_name in self.leeves):
            obj_type = self.objects[type_name]
            leef = Leef(obj_type, self.objects)
            self.add_leef(leef)
        return self.leeves[type_name]


tree = None


def set_tree(new_tree):
    global tree
    tree = new_tree

base_types = [b_type for b_type in default_types]
base_types += [DefaultType.Integer, DefaultType.Unicode,
        DefaultType.Boolean]


class Leef(object):
    def __init__(self, obj_decl, objects):
        self.obj_decl = obj_decl
        self.objects = objects
        self.level = None

    def get_level(self):
        if self.level is None:
            if isinstance(self.obj_decl, Enum):
                return 1

            level_list = [0, ]

            for obj_type in self.obj_decl.imports:
                if not obj_type in base_types:
                    type_leef = tree.get_leef(obj_type)
                    level_list.append(type_leef.get_level())
            self.level = max(level_list) + 1

        return self.level


class FakeFile(object):
    def __init__(self):
        self.datas = ""

    def write(self, data):
        self.datas += data


def getToJSONType(objects, s_type):
    if s_type in ["Unicode", "Integer", "Boolean", "Long", "Date", "IP", "Double"]:
        return s_type
    else:
        return objects[s_type]


def genFakeJSONObject(f, nb_tabs, param_type, objects, write):
    if isinstance(param_type, list_param):
        f.write("\n")
        f.write("%s{\n" % (TABS * nb_tabs))
        f.write("%s\"__type__\": \"List\",\n" % (TABS * (nb_tabs + 1)))
        f.write("%s\"__sub_type__\": \"%s\",\n" % (TABS * (nb_tabs + 1), param_type.sub_type))
        f.write("%s\"__elements__\":[\n" % (TABS * (nb_tabs + 1)))
        genFakeJSONObject(f, nb_tabs + 2,
                getToJSONType(objects, param_type.sub_type), objects, write)
        f.write("%s,\n" % (TABS * (0)))
        genFakeJSONObject(f, nb_tabs + 2,
                getToJSONType(objects, param_type.sub_type), objects, write)
        f.write("\n%s]\n" % (TABS * (nb_tabs + 1)))
        f.write("%s}" % (TABS * nb_tabs))
    elif isinstance(param_type, dict_param):
        f.write("\n")
        f.write("%s{\n" % (TABS * nb_tabs))
        f.write("%s\"__type__\": \"Dict\",\n" % (TABS * (nb_tabs + 1)))
        f.write("%s\"__sub_type__\": \"%s\",\n" % (TABS * (nb_tabs + 1), param_type.sub_type))
        f.write("%s\"__elements__\":{\n" % (TABS * (nb_tabs + 1)))
        f.write("%s\"key1\":\n" % (TABS * (nb_tabs + 1)))
        genFakeJSONObject(f, nb_tabs + 2,
                getToJSONType(objects, param_type.sub_type), objects, write)

        f.write("%s,\n" % (TABS * (0)))
        f.write("%s\"key2\":\n" % (TABS * (nb_tabs + 1)))
        genFakeJSONObject(f, nb_tabs + 2,
                getToJSONType(objects, param_type.sub_type), objects, write)
        f.write("\n%s}\n" % (TABS * (nb_tabs + 1)))
        f.write("%s}" % (TABS * nb_tabs))
    elif isinstance(param_type, result_set_param):
        f.write("\n")
        f.write("%s{\n" % (TABS * nb_tabs))
        f.write("%s\"__type__\": \"ResultSet\",\n" % (TABS * (nb_tabs + 1)))
        f.write("%s\"__sub_type__\": \"%s\",\n" % (TABS * (nb_tabs + 1), param_type.sub_type))
        f.write("%s\"__total_items__\": 1,\n" % (TABS * (nb_tabs + 1)))
        f.write("%s\"__elements__\":[\n" % (TABS * (nb_tabs + 1)))
        genFakeJSONObject(f, nb_tabs + 2,
                getToJSONType(objects, param_type.sub_type), objects, write)
        f.write("%s,\n" % (TABS * (0)))
        genFakeJSONObject(f, nb_tabs + 2,
                getToJSONType(objects, param_type.sub_type), objects, write)
        f.write("\n%s]\n" % (TABS * (nb_tabs + 1)))
        f.write("%s}" % (TABS * nb_tabs))

    elif isinstance(param_type, gen_obj):
        if param_type.name == "Unicode":
            f.write("%s\"azerty\"" % (TABS * 0))
        elif param_type.name == "Integer":
            return f.write("%s1" % (TABS * 0))
        elif param_type.name == "Long":
            f.write("%s{\"__type__\": \"Long\",\n" % (TABS * 0))
            f.write("%s\"__value__\": \"-123456402131454313543215\"}" % (TABS * nb_tabs))
        elif param_type.name == "ULong":
            f.write("%s{\"__type__\": \"ULong\",\n" % (TABS * 0))
            f.write("%s\"__value__\": \"123456402131454313543215\"}" % (TABS * nb_tabs))
        elif param_type.name == "Boolean":
            return f.write("%strue" % (TABS * 0))
        elif param_type.name == "Blob":
            f.write("%s{\"__type__\": \"Blob\",\n" % (TABS * 0))
            f.write("%s\"__value__\": \"231321AEB==\"}" % (TABS * nb_tabs))
        elif param_type.name == "Date":
            f.write("%s{\"__type__\": \"Date\",\n" % (TABS * 0))
            f.write("%s\"__value__\": 123654879}" % (TABS * nb_tabs))
        elif param_type.name == "Time":
            f.write("%s{\"__type__\": \"Time\",\n" % (TABS * 0))
            f.write("%s\"__value__\": 123654879}" % (TABS * nb_tabs))
        elif param_type.name == "IP":
            f.write("%s{\"__type__\": \"IP\",\n" % (TABS * 0))
            f.write("%s\"__value__\": \"192.168.8.125\"}" % (TABS * nb_tabs))
        elif param_type.name == "Double":
            f.write("%s{\"__type__\": \"Double\",\n" % (TABS * 0))
            f.write("%s\"__value__\": \"123456789.1424\"}" % (TABS * nb_tabs))
        elif param_type.name == "ListInfo":
            f.write('%s{\n' % (TABS * 0))
            f.write('%s"__type__": "ListInfo",\n' % (TABS * nb_tabs))
            f.write('%s"__properties__": {\n' % (TABS * nb_tabs))
            f.write('%s"limit": -1,\n' % (TABS * nb_tabs))
            f.write('%s"offset": 0,\n' % (TABS * nb_tabs))
            f.write('%s"order": {\n' % (TABS * nb_tabs))
            f.write('%s"__type__": "Ordering",\n' % (TABS * nb_tabs))
            f.write('%s"__properties__": {\n' % (TABS * nb_tabs))
            f.write('%s"asc": true,\n' % (TABS * nb_tabs))
            f.write('%s"order_by": "something"\n' % (TABS * nb_tabs))
            f.write('%s}\n' % (TABS * nb_tabs))
            f.write('%s},\n' % (TABS * nb_tabs))
            f.write('%s"search_parameters":     {\n' % (TABS * nb_tabs))
            f.write('%s"__sub_type__": "Boolean",\n' % (TABS * nb_tabs))
            f.write('%s"__elements__": {\n' % (TABS * nb_tabs))
            f.write('%s"key2": "value1",\n' % (TABS * nb_tabs))
            f.write('%s"key1": "value2"\n' % (TABS * nb_tabs))
            f.write('%s},\n' % (TABS * nb_tabs))
            f.write('%s"__type__": "Dict"\n' % (TABS * nb_tabs))
            f.write('%s}\n' % (TABS * nb_tabs))
            f.write('%s}\n' % (TABS * nb_tabs))
            f.write('%s}\n' % (TABS * nb_tabs))
        else:
            obj = objects[param_type.name]
            genFakeJSONObject(f, nb_tabs, obj, objects, write)
    elif isinstance(param_type, Object):
        f.write("\n")
        f.write("%s{\n" % (TABS * (nb_tabs)))
        f.write("%s\"__type__\":\"%s\",\n" % (TABS * (nb_tabs + 1), param_type.name))
        f.write("%s\"__properties__\":{\n" % (TABS * (nb_tabs + 1)))

        loop_stuff = []
        for attr_name, attr_type in param_type.attributes.iteritems():
            if (2 == write) or \
                (attr_type.read() and 1 == write) or\
                (attr_type.write() and 0 == write):
                temp = FakeFile()
                temp.write("%s\"%s\":" % (TABS * (nb_tabs + 2), attr_name))
                genFakeJSONObject(temp, nb_tabs + 2, attr_type.type, objects, write)
                loop_stuff.append(temp.datas)

        f.write("%s" % ",\n".join(loop_stuff))
        f.write("%s}\n" % (TABS * (nb_tabs + 1)))
        f.write("%s}" % (TABS * (nb_tabs)))

    elif isinstance(param_type, Enum):
        if param_type.sub_type == "Unicode":
            f.write("\"%s\"" % param_type.values.values()[1])
        else:
            f.write("%s" % param_type.values.values()[1])

    else:
        if param_type == "Unicode":
            f.write("%s\"azertyuiop\"" % (TABS * nb_tabs))
        elif param_type == "Integer":
            return f.write("%s1" % (TABS * nb_tabs))
        elif param_type == "Long":
            f.write("%s{\"__type__\": \"Long\",\n" % (TABS * nb_tabs))
            f.write("%s\"__value__\": \"-123456402131454313543215\"}" % (TABS * nb_tabs))
        elif param_type == "ULong":
            f.write("%s{\"__type__\": \"ULong\",\n" % (TABS * nb_tabs))
            f.write("%s\"__value__\": \"123456402131454313543215\"}" % (TABS * nb_tabs))
        elif param_type == "Boolean":
            return f.write("%strue" % (TABS * nb_tabs))
        elif param_type == "Blob":
            f.write("%s{\"__type__\": \"Blob\",\n" % (TABS * nb_tabs))
            f.write("%s\"__value__\": \"231321AEB==\"}" % (TABS * nb_tabs))
        elif param_type == "Date":
            f.write("%s{\"__type__\": \"Date\",\n" % (TABS * 0))
            f.write("%s\"__value__\": 123654879}" % (TABS * nb_tabs))
        elif param_type == "Time":
            f.write("%s{\"__type__\": \"Time\",\n" % (TABS * 0))
            f.write("%s\"__value__\": 123654879}" % (TABS * nb_tabs))
        elif param_type == "IP":
            f.write("%s{\"__type__\": \"IP\",\n" % (TABS * 0))
            f.write("%s\"__value__\": \"192.168.8.125\"}" % (TABS * nb_tabs))
        elif param_type == "Double":
            f.write("%s{\"__type__\": \"Double\",\n" % (TABS * 0))
            f.write("%s\"__value__\": \"12345678.123\"}" % (TABS * nb_tabs))
        elif param_type.name == "ListInfo":
            f.write('%s{\n' % (TABS * nb_tabs))
            f.write('%s"__type__": "ListInfo",\n' % (TABS * nb_tabs))
            f.write('%s"__properties__": {\n' % (TABS * nb_tabs))
            f.write('%s"limit": -1,\n' % (TABS * nb_tabs))
            f.write('%s"offset": 0,\n' % (TABS * nb_tabs))
            f.write('%s"order": {\n' % (TABS * nb_tabs))
            f.write('%s"__type__": "Ordering",\n' % (TABS * nb_tabs))
            f.write('%s"__properties__": {\n' % (TABS * nb_tabs))
            f.write('%s"asc": true,\n' % (TABS * nb_tabs))
            f.write('%s"order_by": "something"\n' % (TABS * nb_tabs))
            f.write('%s}\n' % (TABS * nb_tabs))
            f.write('%s},\n' % (TABS * nb_tabs))
            f.write('%s"search_parameters":     {\n' % (TABS * nb_tabs))
            f.write('%s"__sub_type__": "Boolean",\n' % (TABS * nb_tabs))
            f.write('%s"__elements__": {\n' % (TABS * nb_tabs))
            f.write('%s"key2": "value1",\n' % (TABS * nb_tabs))
            f.write('%s"key1": "value2"\n' % (TABS * nb_tabs))
            f.write('%s},\n' % (TABS * nb_tabs))
            f.write('%s"__type__": "Dict"\n' % (TABS * nb_tabs))
            f.write('%s}\n' % (TABS * nb_tabs))
            f.write('%s}\n' % (TABS * nb_tabs))
            f.write('%s}\n' % (TABS * nb_tabs))
        else:
            print "--------------------------------------------------%s" % type(param_type)


def genFakeJSONService(f, function, objects, inputJson):
    import json
    if inputJson:
        id = 0
    else:
        id = 1

    json_result = FakeFile()
    json_result.write("    {\n")

    loop_stuff = []

    for key, v_type in function.params[id]:
        temp = FakeFile()
        temp.write("  \"%s\":" % key)
        genFakeJSONObject(temp, 2, v_type, objects, id)
        loop_stuff.append(temp.datas)
    json_result.write(",\n".join(loop_stuff))
    json_result.write("\n  }\n")

    jDict = json.loads(json_result.datas)
    f.write(json.dumps(jDict, indent=1))


def write_stuff(string, size):
    size_used = len(str(string))
    return str(string) + " " * (size - size_used)
