#!/usr/bin/python
# -*- coding: utf-8 -*-

#                         LACIE CONFIDENTIAL
#
# The information contained in this file is the property of LACIE.
# The user of this file shall keep information contained herein confidential
# and shall protect it in whole or in part from disclosure and dissemination
# to third parties.
# Without prior written consent of LACIE, you may not reproduce, represent,
# or download through any means, the information contained herein in any way
# or in any form for a purpose other that the use for internal evaluation,
# operation or maintenance within LACIE.
#
#          (c) LACIE, 2009 - 2010, All Rights Reserved.
import os
from tools import get_instantiator, check_and_create_gen_file, get_imports
from optimus_prime.utils import list_param, dict_param, result_set_param
from optimus_prime.utils import Enum


def generate_object(current_obj, g_objects):
    config = g_objects.config
    if isinstance(current_obj, Enum):
        #GenerateEnum(Object, objects)
        return
    full_name = "%s.%s" % (config.lib_path, current_obj.get_import_path(config.min_version, "default"))

    f = check_and_create_gen_file(current_obj, g_objects, g_objects.config)
    f.write("register_namespace('%s');\n" % full_name)
    f.write(get_imports(current_obj, g_objects))
    f.write("%s.%s = function(){\n" % (full_name, current_obj.name))
    for param, attribute in current_obj.attributes.iteritems():

        if attribute.read():
            f.write("\tthis.get%s = function(){\n" % param)
            f.write("\t\treturn this._%s;\n" % param)
            f.write("\t}\n")

        if attribute.write():
            f.write("\tthis.set%s = function(value){\n" % param)
            f.write("\t\tthis._%s = value;\n" % param)
            f.write("\t}\n")

    f.write("}\n")

    f.write("%s.%s_Instantiator = function(){\n" % (full_name, current_obj.name))
    f.write("}\n")

    f.write("\t%s.%s_Instantiator.serialize = function(object_to_serialize){\n" % (full_name, current_obj.name))

    config = g_objects.config
    version = config.min_version
    if (version.is_lower(current_obj.version)):
        version = current_obj.version

    f.write("\t\tjson_string = {};\n")
    f.write("\t\tjson_string['__version__'] = %s;\n" % version.version)
    f.write("\t\tjson_string['__sub_version__'] = 0;\n")
    f.write("\t\tjson_string['__type__'] = '%s';\n" % current_obj.name)
    f.write("\t\tjson_string['__properties__'] = {};\n")
    for param, attribute in current_obj.attributes.iteritems():

        if attribute.write():
            if attribute.type.is_optional:
                f.write("\t\t\tif (object_to_serialize._%s != undefined){\n" % param)

            if isinstance(attribute.type, list_param):
                instantiator = get_instantiator(attribute.type.sub_type, g_objects)
                if instantiator is None:
                    instantiator = "null"
                f.write("\t\t\tjson_string['__properties__']['%s'] = LacieLib.List_Instantiator.serialize(\"%s\", %s, object_to_serialize._%s);\n" % (param, attribute.type.name, instantiator, param))
            elif isinstance(attribute.type, dict_param):
                instantiator = get_instantiator(attribute.type.sub_type, g_objects)
                if instantiator is None:
                    instantiator = "null"
                f.write("\t\t\tjson_string['__properties__']['%s'] = LacieLib.Dict_Instantiator.serialize(\"%s\", %s, object_to_serialize._%s);\n" % (param, attribute.type.name, instantiator, param))
            elif isinstance(attribute.type, result_set_param):
                instantiator = get_instantiator(attribute.type.sub_type, g_objects)
                if instantiator is None:

                    instantiator = ""
                f.write("\t\t\tjson_string['__properties__']['%s'] = LacieLib.ResultSet_Instantiator.serialize(\"%s\", %s, object_to_serialize._%s);\n" % (param, attribute.type.name, instantiator, param))
            else:
                print attribute
                instantiator = get_instantiator(attribute.type, g_objects)
                if instantiator is not None:
                    instantiator += ".serialize"
                else:
                    instantiator = ""
                f.write("\t\t\tjson_string['__properties__']['%s'] = %s(object_to_serialize._%s);\n" % (param, instantiator, param))
            if attribute.type.is_optional:
                f.write("\t\t\t}\n")

    f.write("\t\treturn json_string;\n")
    f.write("\t}\n")

    f.write("%s.%s_Instantiator.deserialize = function(json_dict){\n" % (full_name, current_obj.name))

    f.write("\t\tvar obj = new %s.%s();\n" % (full_name, current_obj.name))
    for param, attribute in current_obj.attributes.iteritems():
        if attribute.read():
            if attribute.hidden:
                f.write("\t\t\tif (json_dict['__properties__']['%s']){\n" % param)
            if isinstance(attribute.type, list_param):
                instantiator = get_instantiator(attribute.type.sub_type, g_objects)
                if instantiator is None:
                    instantiator = "null"
                f.write("\t\t\tobj._%s = LacieLib.List_Instantiator.deserialize(%s, json_dict['__properties__']['%s']);\n" % (param, instantiator, param))
            elif isinstance(attribute.type, dict_param):
                instantiator = get_instantiator(attribute.type.sub_type, g_objects)
                if instantiator is None:
                    instantiator = "null"
                f.write("\t\t\tobj._%s = LacieLib.Dict_Instantiator.deserialize(%s, json_dict['__properties__']['%s']);\n" % (param, instantiator, param))
            elif isinstance(attribute.type, result_set_param):
                instantiator = get_instantiator(attribute.type.sub_type, g_objects)
                if instantiator is None:

                    instantiator = ""
                f.write("\t\t\tobj._%s = LacieLib.ResultSet_Instantiator.deserialize(%s, json_dict['__properties__']['%s']);\n" % (param, instantiator, param))
            else:
                print attribute
                instantiator = get_instantiator(attribute.type, g_objects)
                if instantiator is not None:
                    instantiator += ".deserialize"
                else:
                    instantiator = ""
                f.write("\t\t\tobj._%s = %s(json_dict['__properties__']['%s']);\n" % (param, instantiator, param))
            if attribute.hidden:
                f.write("\t\t\t}\n")

    f.write("\t\treturn obj;\n")
    f.write("\t}\n")


