#!/usr/bin/python
# -*- coding: utf-8 -*-

#                         LACIE CONFIDENTIAL
#
# The information contained in this file is the property of LACIE.
# The user of this file shall keep information contained herein confidential
# and shall protect it in whole or in part from disclosure and dissemination
# to third parties.
# Without prior written consent of LACIE, you may not reproduce, represent,
# or download through any means, the information contained herein in any way
# or in any form for a purpose other that the use for internal evaluation,
# operation or maintenance within LACIE.
#
#          (c) LACIE, 2009 - 2010, All Rights Reserved.

import os
from tools import get_instantiator, check_and_create_gen_file, get_imports
from optimus_prime.utils import list_param, dict_param, result_set_param


def generate_service(current_service, g_objects):
    config = g_objects.config

    full_name = "%s.%s" % (config.lib_path, current_service.get_import_path(config.min_version, "default"))

    f = check_and_create_gen_file(current_service, g_objects.unique_id, g_objects.config)
    f.write("register_namespace('%s');\n" % full_name)
    f.write(get_imports(current_service, g_objects))
    f.write("%s.%s = function(){}\n" % (full_name, current_service.name))

    for func_name, function in current_service.functions.iteritems():
        f_name = function.display_name
        list_string = []
        if function.async:
            list_string.append('async_client_ref')
        for key, v_type in function.params[0]:
            list_string.append('%s' % key)
        list_string.append('onSuccess')
        list_string.append('onError')
        f.write("%s.%s.%s = function(%s){\n" % (full_name, current_service.name, f_name, ",".join(list_string)))
        #We know the kind of object we are having serialize in here
        f.write("\trequest_json = {};\n")
        for key, attribute in function.params[0]:
            if isinstance(attribute, list_param):
                instantiator = get_instantiator(attribute.sub_type, g_objects)
                if instantiator is None:
                    instantiator = "null"
                f.write("\trequest_json['%s'] = LacieLib.List_Instantiator.serialize(\"%s\", %s, %s);\n" % (key, attribute.name, instantiator, key))
            elif isinstance(attribute, dict_param):
                if instantiator is None:

                    instantiator = "null"
                f.write("\trequest_json['%s'] = LacieLib.Dict_Instantiator.serialize(\"%s\", %s, %s);\n" % (key, attribute.name, instantiator, key))
            elif isinstance(attribute, result_set_param):
                if instantiator is None:

                    instantiator = ""
                f.write("\trequest_json['%s'] = LacieLib.ResultSet_Instantiator.serialize(\"%s\", %s, %s);\n" % (key, attribute.name, instantiator, key))
            else:
                instantiator = get_instantiator(attribute, g_objects)
                if instantiator is not None:
                    instantiator += ".serialize"
                else:
                    instantiator = ""
                f.write("\trequest_json['%s'] = %s(%s);\n" % (key, instantiator, key))

        #Now create a callback to convert to correct type
        f.write("\tfunction ok_callback(text){\n")
        if len(function.params[1]) == 0:
            f.write("\t\tonSuccess();\n")
        else:
            f.write("\t\tjson = JSON.parse(text);\n")

            for key, attribute in function.params[1]:
                f.write("\t\t%s = json['%s'];\n" % (key, key))
                if isinstance(attribute, list_param):
                    instantiator = get_instantiator(attribute.sub_type, g_objects)
                    if instantiator is None:
                        instantiator = "null"
                    f.write("\t\tvar response = LacieLib.List_Instantiator.deserialize(%s, %s);\n" % (instantiator, key))
                elif isinstance(attribute, dict_param):
                    instantiator = get_instantiator(attribute.sub_type, g_objects)
                    if instantiator is None:
                        instantiator = "null"
                    f.write("\t\tvar response = LacieLib.Dict_Instantiator.deserialize(%s, %s);\n" % (instantiator, key))
                elif isinstance(attribute, result_set_param):
                    instantiator = get_instantiator(attribute.sub_type, g_objects)
                    if instantiator is None:
                        instantiator = "null"
                    f.write("\t\tvar response = LacieLib.ResultSet_Instantiator.deserialize(%s, %s);\n" % (instantiator, key))
                else:
                    instantiator = get_instantiator(attribute, g_objects)
                    if instantiator is not None:
                        instantiator += ".deserialize"
                    else:
                        instantiator = ""
                    f.write("\t\tvar response = %s(%s);\n" % (instantiator, key))
                f.write("\t\tonSuccess(response);\n")

        f.write("\t}\n")
        obj_version = current_service.version
        if obj_version.is_lower(g_objects.config.min_version):
            obj_version = g_objects.config.min_version

        url_prefix = u"api/external"
        if function.private:
            url_prefix = u"api/local"
        f.write("\n")
        url = "\"/%s/%s.%s/%s.%s\"" % (
            url_prefix,
            obj_version.version,
            obj_version.sub_version,
            current_service.identification,
            function.func_name
        )

        f.write("\trequest = new NewRequest(%s, JSON.stringify(request_json), ok_callback, onError);\n" % url)
        f.write("\trequest.send();\n")

        f.write("\t}\n")

