#!/usr/bin/python
# -*- coding: utf-8 -*-

#                         LACIE CONFIDENTIAL
#
# The information contained in this file is the property of LACIE.
# The user of this file shall keep information contained herein confidential
# and shall protect it in whole or in part from disclosure and dissemination
# to third parties.
# Without prior written consent of LACIE, you may not reproduce, represent,
# or download through any means, the information contained herein in any way
# or in any form for a purpose other that the use for internal evaluation,
# operation or maintenance within LACIE.
#
#          (c) LACIE, 2009 - 2012, All Rights Reserved.

from optimus_prime.utils import Enum, default_types, DefaultType
from optimus_prime.utils import list_param, dict_param, optional_param, gen_obj, result_set_param

import os
import os.path

PHP = "php"

g_services = None
TAB = u"    "

errors_string = '''<?php
class Errors{

    public $error = array(
'''
errors_string2 = '''
    );
}
?>
'''


def GenerateErrors(config, g_errors, errors_offsets):
    with open("%s/errors.php" % config.output, "w") as f:
        f.write(errors_string)

        errors = []
        for error_name, error in g_errors.iteritems():
            error_code = int(error.id) + int(errors_offsets[error.category])
            errors.append("%s\"%s\" => %s" % (TAB * 2, error_code, error.name.upper()))
        f.write(", \n".join(errors))
        f.write(errors_string2)


def GenerateServices(config, services, g_objects):
    from services import GenerateService
    global g_services
    g_services = services

    for ser_name, ser in services.iteritems():
        GenerateService(ser, g_services, g_objects, config)


def GenerateObjects(config, g_objects):
    from objects import GenerateObject
    for obj_name, obj in g_objects.iteritems():
        GenerateObject(obj, g_objects, config)


def pre_generate_action(config):
    if not os.path.exists("gen/php"):
        os.makedirs("gen/php")
    return set()


def post_generate_action(config):
    return

from optparse import OptionGroup, OptionValueError


class PHPConfig(object):
    def __init__(self):
        self.client = True
        self.async = False

    def set_client(self):
        self.client = True

    def set_server(self):
        self.client = False

    def set_async(self):
        self.async = True


def options_check(option, opt_str, value, parser):
    value = []
    for arg in parser.rargs:
        if arg[:2] == "--" or arg not in ["client", "server"]:
            break
        elif (arg == "client" and "server" in value) or (arg == "server" and "client" in value):
            raise OptionValueError("Can't have server and client")
        getattr(parser.php, "set_%s" % arg)()
        value.append(arg)
    del parser.rargs[:len(value)]
    parser.generation.append(PHP)


def get_options(parser):
    group = OptionGroup(parser, "PHP Client Options",
                        "No parameters to provide")
    group.add_option("--php", action="callback", help="[client|server] Generate for PHP\
 client.", callback=options_check)
    return (group, PHPConfig())
