from tools import check_and_create_gen_file, TAB, get_imports, get_instantiator
from optimus_prime.utils import DefaultType, Enum
from optimus_prime.utils import list_param, dict_param, result_set_param


def GenerateEnum(current_object, objects):
    f = check_and_create_gen_file(current_object, objects.unique_id, objects.config)
    f.write("public $%s = array(\n" % current_object.name)
    quotes = ''
    if current_object.sub_type == DefaultType.Unicode:
        quotes = '"'
    for key, value in current_object.values.iteritems():
        f.write("%s\"%s\" => %s%s%s,\n" % (TAB, key, quotes, value, quotes))
    f.write(");\n?>")


def GenerateObject(current_object, objects, config):
    if isinstance(current_object, Enum):
        GenerateEnum(current_object, objects)
        return

    warnings = []
    f = check_and_create_gen_file(current_object, objects.unique_id, config)

    r_list = []
    w_list = []
    op_list = {}
    h_list = []
    f.write("namespace %s\%s;\n" % ("\\".join(config.lib_path.split(".")), "\\".join(current_object.get_import_path(config.min_version, "default").split("."))))
    f.write("set_include_path(get_include_path() . PATH_SEPARATOR . \"/usr/lib/php_transformer/\");\n\n")
    f.write(get_imports(current_object, objects))

    f.write("class %s_Instantiator {\n" % current_object.name)

    #f.write("%spublic $attributes = array(\n" % TAB)
    for param, attribute in current_object.attributes.iteritems():
        if attribute.read():
            r_list.append(param)
        if attribute.write():
            w_list.append(param)
        if attribute.hidden:
            h_list.append(param)
        if attribute.type.is_optional:
            if attribute.type.default_value == "void":
                default_value = u"NULL"
            else:
                default_value = attribute.type.default_value
            op_list[param] = default_value

    #Now check client or server and replace it
    if not config.options.client:
        temp = w_list
        w_list = r_list
        r_list = temp

    f.write("\tstatic function serialize($object){\n")
    f.write("\t\t$json_dict=array();\n")
    f.write("\t\t$json_dict['__type__'] = \"%s\";\n" % current_object.name)
    f.write("\t\t$json_dict['__properties__'] = array();\n")
    for param, attribute in current_object.attributes.iteritems():
        action = "serialize"

        if param in w_list:
            if param in op_list:
                f.write("\tif (null != $object->get%s()){\n" % param)
            if isinstance(attribute.type, list_param):
                instantiator = get_instantiator(attribute.type.sub_type, objects, config)
                if instantiator is None:
                    instantiator = "null"
                f.write("\t\t$json_dict['__properties__']['%s'] = \LacieLib\NiceList_Instantiator::%s(\"%s\", \'%s\', $object->get%s());\n" % (param, action, attribute.type.name, instantiator, param))
            elif isinstance(attribute, dict_param):
                instantiator = get_instantiator(attribute.type.sub_type, objects, config)
                if instantiator is None:
                    instantiator = "null"
                f.write("\t\t$json_dict['__properties__']['%s'] = \LacieLib\Dict_Instantiator::%s(\"%s\", \'%s\', $object->get%s());\n" % (param, action, attribute.type.name, instantiator, param))
            elif isinstance(attribute, result_set_param):
                instantiator = get_instantiator(attribute.type.sub_type, objects, config)
                if instantiator is None:

                    instantiator = ""
                f.write("\t\t$json_dict['__properties__']['%s'] = \LacieLib\ResultSet_Instantiator::%s(\"%s\", \'%s\', $object->get%s());\n" % (param, action, attribute.type.name, instantiator, param))
            else:
                instantiator = get_instantiator(attribute.type, objects, config)
                if instantiator is not None:
                    instantiator += "::%s" % action
                else:
                    instantiator = ""
                f.write("\t\t$json_dict['__properties__']['%s'] = %s($object->get%s());\n" % (param, instantiator, param))
            if param in op_list:
                f.write("\t}\n")
    f.write("\t\treturn $json_dict;\n")
    f.write("\t}\n")

    f.write("\tstatic function deserialize($object){\n")
    f.write("\t\t$result = new %s();\n" % current_object.name)
    for param, attribute in current_object.attributes.iteritems():
        action = "deserialize"
        if param in r_list:
            if param in op_list or param in h_list:
                f.write("\tif (true===isset($object->__properties__->%s)){\n" % param)

            if isinstance(attribute.type, list_param):
                instantiator = get_instantiator(attribute.type.sub_type, objects, config)
                if instantiator is None:
                    instantiator = "null"
                f.write("\t\t$result->set%s(\LacieLib\NiceList_Instantiator::%s(\"%s\", \'%s\', $object->__properties__->%s));\n" % (param, action, attribute.type.name, instantiator, param))
            elif isinstance(attribute, dict_param):
                instantiator = get_instantiator(attribute.type.sub_type, objects, config)
                if instantiator is None:
                    instantiator = "null"
                f.write("\t\t$result->set%s(\LacieLib\Dict_Instantiator::%s(\"%s\", \'%s\', $object->__properties__->%s));\n" % (param, action, attribute.type.name, instantiator, param))
            elif isinstance(attribute, result_set_param):
                instantiator = get_instantiator(attribute.type.sub_type, objects, config)
                if instantiator is None:
                    instantiator = ""
                f.write("\t\t$result->set%s(\LacieLib\ResultSet_Instantiator::%s(\"%s\", \'%s\', $object->__properties__->%s));\n" % (param, action, attribute.type.name, instantiator, param))
            else:
                instantiator = get_instantiator(attribute.type, objects, config)
                if instantiator is not None:
                    instantiator += "::%s" % action
                else:
                    instantiator = ""
                f.write("\t\t$result->set%s(%s($object->__properties__->%s));\n" % (param, instantiator, param))
            if param in op_list or param in h_list:
                f.write("\t}\n")
            if not config.options.client:
                #TODO here store the default value
                pass
    f.write("\t\treturn $result;\n")

    f.write("\t}\n")

    f.write("\n}\n")

    f.write("\nclass %s{\n" % current_object.name)
    for param, attribute in current_object.attributes.iteritems():
        f.write("\tprivate $%s = null;\n" % param)
    for param, attribute in current_object.attributes.iteritems():
        f.write("\tpublic function get%s(){\n" % param)
        f.write("\t\treturn $this->%s;\n" % param)
        f.write("\t}\n")
        f.write("\tpublic function set%s($value){\n" % param)
        f.write("\t\t$this->%s = $value;\n" % param)
        f.write("\t}\n")

    f.write("}\n")

    f.write("\n\n?>\n")
    f.close()

    return warnings
