from tools import check_and_create_gen_file, get_imports, TAB, get_instantiator
from optimus_prime.utils import list_param, dict_param, result_set_param


def GenerateServiceClient(current_object, services, objects, config):
    '''
    Generate the corresponding PHP File
    '''
    f = check_and_create_gen_file(current_object, objects.unique_id, config)
    #f.write("include_once(\"transformer/lib/Optional.php\");\n")
    f.write(get_imports(current_object, objects))
    if current_object.extends:
        for extend in current_object.extends:
            interface = services[extend]
            f.write(get_imports(interface, objects))

    if config.options.client:
        f.write("include_once(\"transformer/client/ClientTransport.php\");\n")
        f.write("include_once(\"transformer/client/ClientSerializer.php\");\n\n")

    f.write("class %s {\n" % current_object.name)

    functions = [t for t in current_object.functions.iteritems()]
    if current_object.extends:
        for extend in current_object.extends:
            interface = services[extend]
            functions.extend(interface.functions.iteritems())

    # Now write all the methods
    for func_name, function in functions:
        in_params = [p for p, t in function.params[0]]
        out_params = [p for p, t in function.params[1]]

        if 0 == len(function.params[0]):
            f.write("\n%sfunction %s() {\n" % (TAB, function.func_name))
        else:
            f.write("\n%sfunction %s($%s) {\n" % (TAB, function.func_name, ", $".join(in_params)))

        #Create the dictionnary to send the parameters
        f.write("%s$params = (object)array();\n" % (TAB * 2))

        if config.options.client:
            action = "serialize"

            for key, attribute in function.params[0]:

                if isinstance(attribute, list_param):
                    instantiator = get_instantiator(attribute.sub_type, objects, config)
                    if instantiator is None:
                        instantiator = "null"
                    f.write("\t\t$params->%s = \LacieLib\NiceList_Instantiator::%s(\"%s\", \'%s\', $%s);\n" % (key, action, attribute.name, instantiator, key))
                elif isinstance(attribute, dict_param):
                    instantiator = get_instantiator(attribute.sub_type, objects, config)
                    if instantiator is None:
                        instantiator = "null"
                    f.write("\t\t$params->%s = \LacieLib\Dict_Instantiator::%s(\"%s\", \'%s\', $%s);\n" % (key, action, attribute.name, instantiator, key))
                elif isinstance(attribute, result_set_param):
                    instantiator = get_instantiator(attribute.sub_type, objects, config)
                    if instantiator is None:

                        instantiator = ""
                    f.write("\t\t$params->%s = \LacieLib\ResultSet_Instantiator::%s(\"%s\", \'%s\', $%s);\n" % (key, action, attribute.name, instantiator, key))
                else:
                    instantiator = get_instantiator(attribute, objects, config)
                    if instantiator is not None:
                        instantiator += "::%s" % action
                    else:
                        instantiator = ""
                    f.write("\t\t$params->%s = %s($%s);\n" % (key, instantiator, key))

            obj_version = current_object.version
            if obj_version.is_lower(objects.config.min_version):
                obj_version = objects.config.min_version

            url_prefix = u"api/external"
            if function.private:
                url_prefix = u"api/local"
            f.write("\n")
            url = "\"/%s/%s.%s/%s.%s\"" % (
                url_prefix,
                obj_version.version,
                obj_version.sub_version,
                current_object.identification,
                function.func_name
            )
            f.write("\n%s$resp = ClientTransport::send(%s, " % (TAB * 2, url))
            f.write("ClientSerializer::to_string($params));\n")
            f.write("%s$res = ClientSerializer::from_string($resp[0], $resp[1]);\n" % (TAB * 2))
            #f.write("%s$res = $returned[1];\n" % (TAB * 2))


            if 0 != len(out_params):

                key, attribute = function.params[1][0]
                action = "deserialize"
                if isinstance(attribute, list_param):
                    instantiator = get_instantiator(attribute.sub_type, objects, config)
                    if instantiator is None:
                        instantiator = "null"
                    f.write("\t\treturn \LacieLib\NiceList_Instantiator::%s(\"%s\", \'%s\', $res->%s);\n" % (action, attribute.name, instantiator, key))
                elif isinstance(attribute, dict_param):
                    instantiator = get_instantiator(attribute.sub_type, objects, config)
                    if instantiator is None:
                        instantiator = "null"
                    f.write("\t\treturn \LacieLib\Dict_Instantiator::%s(\"%s\", \'%s\', $res->%s);\n" % (action, attribute.name, instantiator, key))
                elif isinstance(attribute, result_set_param):
                    instantiator = get_instantiator(attribute.sub_type, objects, config)
                    if instantiator is None:
                        instantiator = ""
                    f.write("\t\treturn \LacieLib\ResultSet_Instantiator::%s(\"%s\", \'%s\', $res->%s);\n" % (action, attribute.name, instantiator, key))
                else:
                    instantiator = get_instantiator(attribute, objects, config)
                    if instantiator is not None:
                        instantiator += "::%s" % action
                    else:
                        instantiator = ""
                    f.write("\t\treturn %s($res->%s);\n" % (instantiator, key))

            f.write('%s}' % TAB)

        #if 0 == len(function.params[0]):
        #    f.write("\n\n%sfunction %s_async($callback) {\n" % (TAB, function.func_name))
        #else:
        #    f.write("\n\n%sfunction %s_async($%s, $callback) {\n" % (TAB, function.func_name, ", $".join(in_params)))
        ##Create the dictionnary to send the parameters
        #f.write("%s$params = array();\n" % (TAB * 2))
        #f.write("\n".join(["%s$params['%s'] = $%s;" % (TAB * 2, param, param) for param in in_params]))
        #f.write("\n%sClientTransport::async(\"%s.%s\", ClientSerializer::to_string($params), $callback);\n" % (TAB * 2, current_object.identification, function.func_name))
        #f.write('%s}\n' % TAB)

    f.write("\n}\n?>")
    f.close()


def GenerateServiceServer(current_object, services, objects, config):
    f = check_and_create_gen_file(current_object, objects.unique_id, config)
    f.write("namespace %s\%s\%s;\n" % ("\\".join(config.gen_path.split(".")), "\\".join(current_object.get_import_path(config.min_version, "default").split(".")), current_object.name))
    f.write("set_include_path(get_include_path() . PATH_SEPARATOR . \"/usr/lib/php_transformer/\");\n\n")
    #f.write("include_once(\"transformer/lib/Optional.php\");\n")
    f.write(get_imports(current_object, objects))


    if current_object.extends:
        for extend in current_object.extends:
            interface = services[extend]
            f.write(get_imports(interface, objects))

    imp_path = current_object.implementation.split(".")

    f.write("include_once('%s.php');\n" % "/".join(imp_path))

    f.write("class %s {\n" % current_object.name)

    functions = [t for t in current_object.functions.iteritems()]
    if current_object.extends:
        for extend in current_object.extends:
            interface = services[extend]
            functions.extend(interface.functions.iteritems())

    # Now write all the methods
    for func_name, function in functions:
        in_params = [p for p, t in function.params[0]]
        out_params = [p for p, t in function.params[1]]

        f.write("\n%spublic static function %s($context, $input) {\n" % (TAB, function.func_name))

        for key, attribute in function.params[0]:
            action = "deserialize"
            if isinstance(attribute, list_param):
                instantiator = get_instantiator(attribute.sub_type, objects, config)
                if instantiator is None:
                    instantiator = "null"
                f.write("\t\t$obj_%s = \LacieLib\NiceList_Instantiator::%s(\"%s\", \'%s\', $input->%s);\n" % (key, action, attribute.name, instantiator, key))
            elif isinstance(attribute, dict_param):
                instantiator = get_instantiator(attribute.sub_type, objects, config)
                if instantiator is None:
                    instantiator = "null"
                f.write("\t\t$obj_%s = \LacieLib\Dict_Instantiator::%s(\"%s\", \'%s\', $input->%s);\n" % (key, action, attribute.name, instantiator, key))
            elif isinstance(attribute, result_set_param):
                instantiator = get_instantiator(attribute.sub_type, objects, config)
                if instantiator is None:
                    instantiator = ""
                f.write("\t\t$obj_%s = \LacieLib\ResultSet_Instantiator::%s(\"%s\", \'%s\', $input->%s);\n" % (key, action, attribute.name, instantiator, key))
            else:
                instantiator = get_instantiator(attribute, objects, config)
                if instantiator is not None:
                    instantiator += "::%s" % action
                else:
                    instantiator = ""
                f.write("\t\t$obj_%s = %s($input->%s);\n" % (key, instantiator, key))
        if len(in_params) == 0:
            f.write("\t\t$res = \%s::%s($context);\n" % (current_object.implementation.replace(".", "\\"), func_name))
        else:
            f.write("\t\t$res = \%s::%s($context, %s);\n" % (current_object.implementation.replace(".", "\\"), func_name, ", ".join(["$obj_%s" % p for p in in_params])))

        f.write("\t\t$object = (object)array();\n")

        if (0 != len(function.params[1])):
            action = "serialize"
            key, attribute = function.params[1][0]
            if isinstance(attribute, list_param):
                instantiator = get_instantiator(attribute.sub_type, objects, config)
                if instantiator is None:
                    instantiator = "null"
                f.write("\t\t$object->%s = \LacieLib\NiceList_Instantiator::%s(\"%s\", \'%s\', $res);\n" % (key, action, attribute.name, instantiator))
            elif isinstance(attribute, dict_param):
                instantiator = get_instantiator(attribute.sub_type, objects, config)
                if instantiator is None:
                    instantiator = "null"
                f.write("\t\t$object->%s = \LacieLib\Dict_Instantiator::%s(\"%s\", \'%s\', $res);\n" % (key, action, attribute.name, instantiator))
            elif isinstance(attribute, result_set_param):
                instantiator = get_instantiator(attribute.sub_type, objects, config)
                if instantiator is None:
                    instantiator = ""
                f.write("\t\t$object->%s = \LacieLib\ResultSet_Instantiator::%s(\"%s\", \'%s\', $res);\n" % (key, action, attribute.name, instantiator))
            else:
                instantiator = get_instantiator(attribute, objects, config)
                if instantiator is not None:
                    instantiator += "::%s" % action
                else:
                    instantiator = ""
                f.write("\t\t$object->%s = %s($res);\n" % (key, instantiator))

        f.write("\t\treturn $object;\n")

        #TODO return serialized object
        f.write("\t}\n")

    f.write("}\n")


def GenerateService(current_object, services, objects, config):
    if config.options.client:
        return GenerateServiceClient(current_object, services, objects, config)
    return GenerateServiceServer(current_object, services, objects, config)
