from optimus_prime.utils import Enum, default_types, DefaultType
from optimus_prime.utils import list_param, dict_param, gen_obj, result_set_param

import os

TAB = u"    "


base_types = [b_type for b_type in default_types]
base_types += [DefaultType.Integer, DefaultType.Unicode, DefaultType.Boolean]

base_type = {
    DefaultType.Unicode: "string",
    DefaultType.Integer: "int",
    DefaultType.Long: "long",
    DefaultType.Boolean: "bool",
    "void": "NULL",
}


def get_instantiator(serializer_type, g_objects, config):
    if hasattr(serializer_type, "name"):
        serializer_type = serializer_type.name

    if serializer_type in [DefaultType.Integer, DefaultType.Unicode, DefaultType.Boolean]:
        return None
    elif serializer_type in base_types:
        return "\LacieLib\%s_Instantiator" % serializer_type
    else:
        return "%s_Instantiator" % ('\%s\%s\%s' % ("\\".join(config.lib_path.split(".")), "\\".join(g_objects[serializer_type].get_import_path(config.min_version, 'default').split(".")), serializer_type))


def check_and_create_gen_file(obj, gen_date, config):

    start = ""

    if config.options.client:
        splited_path = [config.output, config.lib_path] + obj.get_import_path(config.min_version, "default").split('.')
    else:
        splited_path = [config.output, ] + config.gen_path.split('.') + obj.get_import_path(config.min_version, "default").split(".")

    for path in splited_path:
        start = os.path.join(start, path)
        if os.path.exists(start):
            pass
        else:
            os.mkdir(start)

    header = "<?php\n\
# -*- coding: utf-8 -*-\n\
#GEN_DATE %s\n\
\n\
#                         LACIE CONFIDENTIAL\n\
#\n\
# The information contained in this file is the property of LACIE.\n\
# The user of this file shall keep information contained herein confidential\n\
# and shall protect it in whole or in part from disclosure and dissemination\n\
# to third parties.\n\
# Without prior written consent of LACIE, you may not reproduce, represent,\n\
# or download through any means, the information contained herein in any way\n\
# or in any form for a purpose other that the use for internal evaluation,\n\
# operation or maintenance within LACIE.\n\
#\n\
#          (c) LACIE, 2009 - 2012, All Rights Reserved.\n\n\
" % gen_date

    # Create the file
    file_path = os.path.join(*splited_path)
    file_path = os.path.join(file_path, "%s.php" % obj.name)

    file = open(file_path, "w")
    file.write(header)

    return file


def get_imports(o_object, objects):
    string = u""
    base_list = []
    for obj_type in o_object.imports:
        if not(obj_type in base_type) and not(obj_type in default_types):
            ob = objects[obj_type]

            if isinstance(ob, Enum):
                string += "include_once(\"%s/%s.php\");\n" % (
                    objects.config.lib_path,
                    ob.get_import_path(objects.config.min_version, "default").replace(".", "/")
                )
            else:
                string += "include_once(\"%s/%s/%s.php\");\n" % (
                    objects.config.lib_path,
                    ob.get_import_path(objects.config.min_version, "default").replace(".", "/"),
                    obj_type
                )
        elif obj_type in default_types:
            base_list.append(obj_type)
    string += "include_once(\"transformer/lib/lib.php\");\n"
    if base_list:
        for el_base in base_list:
            if el_base in (DefaultType.List, DefaultType.ResultSet, DefaultType.Dict):
                if el_base == "List":
                    el_base = "NiceList"
                string += "include_once(\"transformer/lib/%s.php\");\n" % el_base

    return string


def get_php_type(param_type):

    if isinstance(param_type, list_param):
        temp_value = "NiceList(%s)" % get_php_type(param_type.sub_type)
    elif isinstance(param_type, dict_param):
        temp_value = "Dict(%s)" % get_php_type(param_type.sub_type)
    elif isinstance(param_type, result_set_param):
        temp_value = "ResultSet(%s)" % get_php_type(param_type.sub_type)
    elif isinstance(param_type, gen_obj):
        temp_value = get_php_type(param_type.name)
    elif param_type in base_type:
        temp_value = base_type[param_type]
    else:
        temp_value = param_type

    if hasattr(param_type, "is_optional") and param_type.is_optional:
        return "Optional(%s)" % temp_value
    return temp_value
