#!/usr/bin/python
# -*- coding: utf-8 -*-

#                         LACIE CONFIDENTIAL
#
# The information contained in this file is the property of LACIE.
# The user of this file shall keep information contained herein confidential
# and shall protect it in whole or in part from disclosure and dissemination
# to third parties.
# Without prior written consent of LACIE, you may not reproduce, represent,
# or download through any means, the information contained herein in any way
# or in any form for a purpose other that the use for internal evaluation,
# operation or maintenance within LACIE.
#
#          (c) LACIE, 2009 - 2012, All Rights Reserved.

from optimus_prime.utils import Enum, default_types, DefaultType
from optimus_prime.utils import list_param, dict_param, gen_obj, result_set_param

import os
import os.path

PYTHON = "python"

g_services = None


base_serializers = {
    DefaultType.Unicode: unicode,
    DefaultType.Integer: int,
    DefaultType.Boolean: bool,
}


def sub_type_is_raw_type(stype):
    rstype = get_python_type(stype)
    return rstype in [DefaultType.Date, DefaultType.IP, DefaultType.Long,
            DefaultType.ULong, DefaultType.Time, DefaultType.Double]


TAB = u"    "
_1_TAB = TAB
_2_TAB = TAB * 2
_3_TAB = TAB * 3
_4_TAB = TAB * 4
_5_TAB = TAB * 5


class File(object):
    value = ""

    def write(self, string):
        self.value += string

    def close(self):
        pass

    def __str__(self):
        return self.value


def _open(file, *args, **kwargs):
    return File()

base_type = {
    DefaultType.Unicode: "unicode",
    DefaultType.Integer: "int",
    DefaultType.Boolean: "bool",
    "void": "None",
}


def get_python_type(param_type):
    if isinstance(param_type, list_param):
        temp_value = "List(%s)" % get_python_type(param_type.sub_type)
    elif isinstance(param_type, dict_param):
        temp_value = "Dict(%s)" % get_python_type(param_type.sub_type)
    elif isinstance(param_type, result_set_param):
        temp_value = "ResultSet(%s)" % get_python_type(param_type.sub_type)
    elif isinstance(param_type, gen_obj):
        temp_value = get_python_type(param_type.name)
    elif param_type in base_type:
        temp_value = base_type[param_type]
    else:
        temp_value = param_type
    if hasattr(param_type, "is_optional") and param_type.is_optional:
        return "Optional(%s)" % temp_value
    return temp_value


def check_and_create_gen_file(obj, gen_date, config, for_enum=False):

    if config.options.client:
        splited_path = [config.output, config.lib_path] + obj.get_import_path(config.min_version, PYTHON).split('.')
    else:
        splited_path = [config.output, ] + config.gen_path.split('.') + obj.get_import_path(config.min_version, PYTHON).split(".")

    start = ""

    for path in splited_path:
        start = os.path.join(start, path)
        if os.path.exists(start):
            pass
        else:
            os.mkdir(start)
        init = os.path.join(start, "__init__.py")
        if not os.path.exists(init):
            with open(init, "w") as f:
                pass

    # if not isinstance(obj, Enum):
    #     gen_date = ""

    header = "\
#!/usr/bin/python\n\
# -*- coding: utf-8 -*-\n\
\n\
#                         LACIE CONFIDENTIAL\n\
#\n\
# The information contained in this file is the property of LACIE.\n\
# The user of this file shall keep information contained herein confidential\n\
# and shall protect it in whole or in part from disclosure and dissemination\n\
# to third parties.\n\
# Without prior written consent of LACIE, you may not reproduce, represent,\n\
# or download through any means, the information contained herein in any way\n\
# or in any form for a purpose other that the use for internal evaluation,\n\
# operation or maintenance within LACIE.\n\
#\n\
#          (c) LACIE, 2009 - 2012, All Rights Reserved.\n\
\n\
#GEN_DATE %s\n\
#\n\
# 888b. .d88b.    8b  8 .d88b. 88888    8888 888b. 888 88888\n\
# 8   8 8P  Y8    8Ybm8 8P  Y8   8      8www 8   8  8    8\n\
# 8   8 8b  d8    8  '8 8b  d8   8      8    8   8  8    8\n\
# 888P' `Y88P'    8   8 `Y88P'   8      8888 888P' 888   8\n\
#\n\
#\n\n\
" % gen_date

    new = True
    if not isinstance(obj, Enum):
        # Create the file
        file_path = "%s/%s.py" % (start, obj.name)
    else:
        file_path = "%s/%s.py" % (start, "__init__")
        # First READ for the #GEN_DATE parameter
        new = not os.path.exists(file_path)
        if not new:
            with open(file_path) as f:
                lines = f.readlines()
                if len(lines) < 3:
                    new = True
                else:
                    if not lines[16].startswith("#GEN_DATE"):
                        new = True
                    else:
                        f_gen_date = lines[16].replace("#GEN_DATE", "").strip()
                        if str(f_gen_date) != str(gen_date):
                            new = True
                        else:
                            new = False

    if new:
        file = open(file_path, "w")
        file.write(header)
        if for_enum:
            file.write("from transformer.lib import Enum\n")
    else:
        file = open(file_path, "a")

    return file


def get_imports(o_object, objects, tabs=0):
    string = u""
    base_list = []
    #merge parent an child imports
    imports_list = o_object.get_imports()

    for obj_type in imports_list:
        if not(obj_type in base_type) and not(obj_type in default_types):
            ob = objects[obj_type]
            if objects.config.options.client:
                if isinstance(ob, Enum):
                    string += "%sfrom %s.%s import %s\n" % (
                        tabs * "\t",
                        objects.config.lib_path,
                        ob.get_import_path(objects.config.min_version, PYTHON),
                        obj_type
                    )
                else:
                    string += "%sfrom %s.%s.%s import %s\n" % (
                        tabs * "\t",
                        objects.config.lib_path,
                        ob.get_import_path(objects.config.min_version, PYTHON),
                        obj_type, obj_type
                    )
            else:
                if isinstance(ob, Enum):
                    string += "%sfrom %s.%s import %s\n" % (
                        tabs * "\t",
                        objects.config.gen_path,
                        ob.get_import_path(objects.config.min_version, PYTHON),
                        obj_type
                    )
                else:
                    string += "%sfrom %s.%s.%s import %s\n" % (
                        tabs * "\t",
                        objects.config.gen_path,
                        ob.get_import_path(objects.config.min_version, PYTHON),
                        obj_type, obj_type
                    )
        elif obj_type in default_types:
            base_list.append(obj_type)

    if base_list:
        string += "\n"
        string += "%sfrom transformer.lib import %s" % (tabs * "\t", (", ".join(base_list) + "\n"))
    return string


def GenerateEnum(Object, objects):
    f = check_and_create_gen_file(Object, objects.unique_id, objects.config,
            for_enum=True)

    f.write("class %s(Enum):\n" % Object.name)
    pre_quotes = post_quotes = ''
    invert = {}
    for key, value in Object.values.iteritems():
        invert[value] = key

    values = invert.keys()

    if Object.sub_type == DefaultType.Unicode:
        pre_quotes = 'u"'
        post_quotes = '"'
    values.sort()

    for value in values:
        key = invert[value]
        f.write(_1_TAB + "%s= %s%s%s\n" % (key, pre_quotes, value, post_quotes))

    if objects.config.options.client:
        f.write(_1_TAB + "@classmethod\n")
        f.write(_1_TAB + "def from_cli(cls, value):\n")
        f.write(_2_TAB + "return cls(getattr(cls, value))\n")
    f.close()


def GenerateObject(Object, objects):
    '''
    Generate the corresponding Python File
    '''
    options = objects.config.options
    if isinstance(Object, Enum):
        GenerateEnum(Object, objects)
        return

    warnings = []
    f = check_and_create_gen_file(Object, objects.unique_id, objects.config)
    f.write("from transformer.lib import Optional, Instantiator, register_instantiator\n")

    f.write("from transformer.lib.proxy_class import Instantiable\n")

    inherit = u"object"
    if not options.client:
        imp_path = Object.get_backend_path().split(".")
        name = imp_path[-1]
        inherit = "Backend%s" % name
        f.write("from %s import %s as %s\n" % (".".join(imp_path[:-1]), name, inherit))

    else:
        f.write("from transformer.lib.cli import ClientObject\n")

    f.write("%s\n\n" % get_imports(Object, objects))
    f.write("class %sInstantiator(Instantiator):\n" % Object.name)
    obj_version = Object.version
    if obj_version.is_lower(objects.config.min_version):
        obj_version = objects.config.min_version

    if not objects.config.legacy:
        f.write(_1_TAB + ("version = %s\n" % (obj_version.version)))
        f.write(_1_TAB + ("sub_version = %s\n" % obj_version.sub_version))

    f.write(_1_TAB + "attributes = {\n")
    r_list = []
    w_list = []
    op_list = {}
    h_list = []

    attributes = Object.get_attributes()

    keys = attributes.keys()
    keys.sort()
    for param in keys:
        attribute = attributes[param]
        f.write(_2_TAB + "'%s': %s,\n" % (param, get_python_type(attribute.type)))
        if attribute.read():
            r_list.append(param)

        if attribute.write():
            w_list.append(param)

        if attribute.hidden:
            h_list.append(param)

        if attribute.type.is_optional:
            if attribute.type.default_value is None:
                default_value = u"None"
            else:
                if attribute.type.default_value.startswith('"'):
                    default_value = "u%s" % attribute.type.default_value
                else:
                    default_value = attribute.type.default_value
            op_list[param] = default_value

    if not options.client:
        temp = w_list
        w_list = r_list
        r_list = temp
    f.write(_1_TAB + "}\n")

    f.write(_1_TAB + "w_attributes = [%s]\n" % (", ".join([('"%s"' % e) for e in w_list])))
    f.write(_1_TAB + "r_attributes = [%s]\n" % (", ".join([('"%s"' % e) for e in r_list])))
    f.write(_1_TAB + "h_attributes = [%s]\n" % (", ".join([('"%s"' % e) for e in h_list])))
    f.write(_1_TAB + "op_values = {%s}\n\n" % (", ".join(["'%s': %s" % (k, v) for k, v in op_list.iteritems()])))

    f.write(_1_TAB + "@classmethod\n")
    f.write(_1_TAB + "def name(cls):\n")
    f.write(_2_TAB + "return u'%s'\n\n" % Object.name)

    f.write(_1_TAB + "@classmethod\n")
    f.write(_1_TAB + "def new_object(cls):\n")
    f.write(_2_TAB + "return %s()\n\n" % Object.name)

    f.write("\n")
    if options.client:
        f.write("class %s(ClientObject):\n" % (Object.name))
        for p in w_list:
            f.write("\n")
            f.write(_1_TAB + "def set%s(self, value):\n" % (p))
            f.write(_2_TAB + "self.%s = value\n" % (p))

        for p in r_list:
            f.write("\n")
            f.write(_1_TAB + "def get%s(self):\n" % (p))
            f.write(_2_TAB + "return self.%s\n" % (p))

    else:
        f.write("class %s(Instantiable):\n" % (Object.name))
        f.write(_1_TAB + "__backend__ = %s\n" % inherit)

    f.write(_1_TAB + "__instantiator__ = %sInstantiator\n" % (Object.name))
    if options.client:
        f.write("register_instantiator('%s', %sInstantiator, True)\n" % (objects.config.lib_path,
                Object.name))

    else:
        f.write("register_instantiator('%s', %sInstantiator, False)\n" % (objects.config.lib_path,
                Object.name))

    f.close()

    return warnings


def GenerateService(Object, objects):
    '''
    Generate the corresponding Python File
    '''
    options = objects.config.options
    f = check_and_create_gen_file(Object, objects.unique_id, objects.config)
    f.write("from transformer.lib import Optional\n")
    f.write("%s\n" % get_imports(Object, objects))

    if Object.extends:
        for extend in Object.extends:
            interface = g_services[extend]
            f.write("%s\n" % get_imports(interface, objects))

    if not options.client:
        temp = Object.implementation.split(".")
        f.write("from %s import %s as backend%s\n" % (".".join(temp[:-1]),
            temp[-1], temp[-1]))

        f.write("from transformer.lib.errors import NotEnoughRights\n")
        f.write("from transformer.lib import private\n")
        f.write("from transformer.lib.tools import (list_from_string, search_criteria, order_by_criteria, bool_from_string, int_from_string, unicode_from_string)\n")

    if options.client:
        f.write("from transformer.client_transport import ClientTransport\n")
        f.write("from transformer.client_serializer import ClientSerializer\n")

    f.write("\n\n")
    f.write("class %s(object):\n" % Object.name)

    functions = {}
    for func_name in Object.functions:
        functions[func_name] = Object.functions[func_name]
    if Object.extends:
        for extend in Object.extends:
            interface = g_services[extend]
            functions.update(interface.functions)

    keys_list = functions.keys()
    keys_list.sort()

    if not options.client:
        f.write(_1_TAB + "instance = None\n\n")
        f.write(_1_TAB + "def __init__(self, user):\n")
        f.write(_2_TAB + "self.user = user\n\n")
        f.write(_1_TAB + "def getInstance(self, user):\n")
        f.write(_2_TAB + "if self.instance is None:\n")
        f.write(_3_TAB + "self.instance = backend%s()\n" % (temp[len(temp) - 1]))
        f.write(_2_TAB + "return self.instance\n\n")
        f.write(_1_TAB + "@classmethod\n")
        f.write(_1_TAB + "def __load_in_dispatcher__(cls):\n")
        f.write(_2_TAB + "if hasattr(backend%s, '__load_in_dispatcher__'):\n" % (temp[len(temp) - 1]))
        f.write(_3_TAB + "return backend%s.__load_in_dispatcher__(cls)\n" % (temp[len(temp) - 1]))
        f.write(_2_TAB + "return True\n")
        # Now generate the checkers for the types
        f.write(_1_TAB + "validators = {\n")

        for func_name in keys_list:
            function = functions[func_name]
            f.write(_2_TAB + "'%s': {\n" % function.func_name)

            # INPUT
            f.write(_3_TAB + "'input': {\n")
            list_string = []
            if function.async:
                list_string.append(_4_TAB + "'async_client_ref': unicode")
            for key, v_type in function.params[0]:
                list_string.append(_4_TAB + "'%s': %s" % (key, get_python_type(v_type)))
            if list_string:
                f.write(",\n".join(list_string))
                f.write("\n")
            f.write(_3_TAB + "},\n")
            if function.async:
                f.write(_3_TAB + "'output': {\n")
                f.write(_4_TAB + "'async_job_id':int\n")
                f.write(_3_TAB + "}\n")
                f.write(_2_TAB + "},\n")
                f.write(_2_TAB + "'%s_result': {\n" % function.func_name)
                f.write(_3_TAB + "'input':{\n")
                f.write(_4_TAB + "'async_job_id': int\n")
                f.write(_3_TAB + "},\n")

            # OUTPUT
            f.write(_3_TAB + "'output': {\n")
            list_string = []
            for key, v_type in function.params[1]:
                list_string.append(_4_TAB + "'%s': %s" % (key, get_python_type(v_type)))
            if list_string:
                f.write(",\n".join(list_string))
                f.write("\n")
            f.write(_3_TAB + "}\n")
            f.write(_2_TAB + "},\n")
        f.write(_1_TAB + "}\n")

    # Now write all the methods
    for func_name in keys_list:
        function = functions[func_name]
        in_params = [p for p, t in function.params[0]]
        out_params = [p for p, t in function.params[1]]
        if function.async:
            in_params.insert(0, "async_client_ref")

        search_criteria = function.params[3]
        order_by_criteria = function.params[4]

        return_params = ", ".join(out_params)
        if not options.client:
            f.write("\n")
            if function.private:
                f.write(_1_TAB + "@private\n")

            if search_criteria:
                join = []
                for p, t in search_criteria:

                    if isinstance(t, gen_obj):
                        if not (t.name in base_type):
                            obj_type = objects[t.name]
                            value = get_python_type(obj_type.sub_type)
                        else:
                            t = t.name
                            value = get_python_type(t)
                    else:
                        value = u"list"
                    join.append("%s=%s_from_string" % (p, value))

                f.write(_1_TAB + "@search_criteria(\"list_info\",%s)\n" %
                        ",".join(join))

            if order_by_criteria:
                join = []
                for p, t in order_by_criteria:
                    if not (t.name in base_type):
                        obj_type = objects[t.name]
                        t = obj_type.sub_type_is_raw_type
                    else:
                        t = get_python_type(t.name)
                    join.append("%s=%s_from_string" % (p, get_python_type(t)))
                f.write(_1_TAB + "@order_by_criteria(\"list_info\",%s)\n" %
                        ",".join(join))

            if 0 == len(in_params):
                f.write(_1_TAB + "def %s(self, context):\n" % function.func_name)
            else:

                f.write(_1_TAB + "def %s(self, context, %s):\n" %
                        (function.func_name, ", ".join(in_params)))

            if not ("with_parameters" in in_params):
                f.write(_2_TAB + "with_parameters = None\n")
            f.write(_2_TAB + "if not context.auth_struct.check_authentication(%s):\n" %
                function.authentication)
            f.write(_3_TAB + "raise NotEnoughRights(function_name=\"%s\")\n" %
                    function.func_name)
            parameters = "context.auth_struct"
            if function.params[0]:

                backend_params = []
                for p, t in function.params[0]:
                    if t.name in base_type or sub_type_is_raw_type(t):
                        backend_params.append(p)
                    else:
                        backend_params.append("%s.get_backend_obj()" % p)
                parameters += ", %s" % ", ".join(backend_params)

            if function.async:
                f.write(_2_TAB + "class async_wrapper(object):\n")
                f.write(_3_TAB + "def __init__(self, func, validators, *parameters):\n")
                f.write(_4_TAB + "self.func = func\n")
                f.write(_4_TAB + "self.args = parameters\n")
                f.write(_4_TAB + "self.kwargs = {}\n")
                f.write(_4_TAB + "self.validators = validators\n")
                f.write(_3_TAB + "def __call__(self):\n")
                f.write(_4_TAB + "res = self.func(*self.args, **self.kwargs)\n")
                f.write(_4_TAB + "d = dict()\n")
                for key, tvalue in function.params[1]:
                    return_value = "'%s'" % key
                    #if sub_type_is_raw_type(tvalue):
                    #    return_value = "%s(%s)" % (get_python_type(tvalue), key)

                    f.write(_4_TAB + "if res is None:\n")
                    f.write(_5_TAB + "d['%s'] = None\n" % key)
                    f.write(_4_TAB + "else:\n")
                    f.write(_5_TAB + "d['%s'] = self.validators['output'][%s](res)\n" % (key, return_value))
                f.write(_4_TAB + "return d\n")

                f.write(_2_TAB + "job_id = context.__async_handler__.add_job(async_client_ref, async_wrapper(self.getInstance(context.auth_struct).%s, self.__class__.validators['%s_result'], %s))\n" % (func_name, func_name, parameters))
                f.write(_2_TAB + "d = dict()\n")
                f.write(_2_TAB + "d['async_job_id'] = job_id\n")
                f.write(_2_TAB + "return d, with_parameters\n\n")

                func_name = "%s_result" % func_name
                f.write(_1_TAB + "def %s(self, context, async_job_id):\n" %
                        func_name)
                f.write(_2_TAB + "with_parameters = None\n")

                f.write(_2_TAB + "d = context.__async_handler__.get_result(context.auth_struct,\
                            async_job_id)\n")

                f.write(_2_TAB + "if isinstance(d, Exception):\n")
                f.write(_3_TAB + "raise d\n")

                f.write(_2_TAB + "return d, with_parameters\n")
            else:
                if return_params:
                    f.write(_2_TAB + "%s = self.getInstance(context.auth_struct).%s(%s)\n" %
                        (return_params, func_name, parameters))
                else:
                    f.write(_2_TAB + "self.getInstance(context.auth_struct).%s(%s)\n" %
                        (func_name, parameters))
                f.write(_2_TAB + "d = dict()\n")
                for key, tvalue in function.params[1]:
                    return_value = key
                    f.write(_2_TAB + "if %s is not None:\n" % key)
                    f.write(_3_TAB + "d['%s'] = self.__class__.validators['%s']['output']['%s'](%s)\n" % (key, func_name, return_value, return_value))
                    f.write(_2_TAB + "else:\n")
                    f.write(_3_TAB + "d['%s'] = None\n" % key)
                f.write(_2_TAB + "return d, with_parameters\n")
        else:
            if 0 == len(in_params):
                f.write("\n\n")
                f.write(_1_TAB + "def %s(self, __client__=None):\n" % function.func_name)
            else:
                f.write("\n\n")
                f.write(_1_TAB + "def %s(self, %s, __client__=None):\n" %
                    (function.func_name, ", ".join(in_params)))

            if function.comments:
                f.write(_2_TAB + '"""\n')
                for comment in function.comments:
                    f.write(_2_TAB + comment + "\n")
                f.write(_2_TAB + '"""\n')


            #Create the dictionnary to send the parameters
            f.write(_2_TAB + "params = dict()\n")
            f.write("\n".join(["%sparams['%s'] = %s" % (_2_TAB, param, param) for
                param in in_params]))

            f.write("\n" + _2_TAB + "if __client__:\n")
            f.write(_3_TAB + "ClientTransportSender = __client__\n")
            f.write(_2_TAB + "else:\n")
            f.write(_3_TAB + "ClientTransportSender = ClientTransport\n")

            obj_version = Object.version
            if obj_version.is_lower(objects.config.min_version):
                obj_version = objects.config.min_version

            url_prefix = u"api/external"
            if function.private:
                url_prefix = u"api/local"
            f.write("\n")
            f.write(_2_TAB + "url = \"%s/%s.%s/%s.%s\"\n" % (
                url_prefix,
                obj_version.version,
                obj_version.sub_version,
                Object.identification,
                function.func_name
            ))

            f.write(_2_TAB + "success, resp = ClientTransportSender.send(url,"
                "ClientSerializer.to_string('%s', params))\n" % objects.config.lib_path)
            f.write(_2_TAB + "res = ClientSerializer.from_string('%s', success, resp)\n" %
                objects.config.lib_path)

            if function.async:
                f.write(_2_TAB + "return res['async_job_id']\n")
                f.write("\n\n")
                f.write(_1_TAB + "def %s_result(self, async_job_id,__client__=None):\n" % function.func_name)
                f.write(_2_TAB + "params = dict()\n")
                f.write(_2_TAB + "params['async_job_id'] = async_job_id\n")
                f.write("\n" + _2_TAB + "if __client__:\n")
                f.write(_3_TAB + "ClientTransportSender = __client__\n")
                f.write(_2_TAB + "else:\n")
                f.write(_3_TAB + "ClientTransportSender = ClientTransport\n")

                obj_version = Object.version
                if obj_version.is_lower(objects.config.min_version):
                    obj_version = objects.config.min_version

                url_prefix = u"api/external"
                if function.private:
                    url_prefix = u"api/local"
                f.write("\n")
                f.write(_2_TAB + "url = \"%s/%s.%s/%s.%s_result\"\n" % (
                    url_prefix,
                    obj_version.version,
                    obj_version.sub_version,
                    Object.identification,
                    function.func_name
                ))

                f.write(_2_TAB + "success, resp = ClientTransportSender.send(url,"
                    "ClientSerializer.to_string('%s', params))\n" % objects.config.lib_path)
                f.write(_2_TAB + "res = ClientSerializer.from_string('%s', success, resp)\n" %
                    objects.config.lib_path)


            f.write(_2_TAB + "return %s\n\n" % (", ".join([u"res['%s']" % (param)
                for param in out_params])))

            if not function.async:
                if 0 == len(function.params[0]):
                    f.write(_1_TAB + "def %s_async(self, callback, __client__=None):\n" %
                        function.func_name)
                else:
                    f.write(_1_TAB + "def %s_async(self, %s, callback, __client__=None):\n" %
                            (function.func_name, ", ".join(in_params)))

                f.write("\n")
                f.write(_2_TAB + "if __client__:\n")
                f.write(_3_TAB + "ClientTransportSender = __client__\n")
                f.write(_2_TAB + "else:\n")
                f.write(_3_TAB + "ClientTransportSender = ClientTransport\n")

                #Create the dictionnary to send the parameters
                f.write(_2_TAB + "params = dict()\n")
                f.write("\n".join(["%sparams['%s'] = %s" % (_2_TAB, param, param) for
                    param in in_params]))
                f.write("\n")
                f.write(_2_TAB + "url = \"%s/%s.%s/%s.%s\"\n" % (
                    url_prefix,
                    obj_version.version,
                    obj_version.sub_version,
                    Object.identification,
                    function.func_name
                ))


                f.write(_2_TAB + "ClientTransportSender.async(url,"
                    "ClientSerializer.to_string('%s', params), callback, '%s')\n" % (
                        objects.config.lib_path,
                        objects.config.lib_path
                    )
                )

    f.close()


def GenerateError(f, error, errors_offsets, options):
    f.write("\nclass %s(Error):\n" % error.name)
    f.write(_1_TAB + "real_id  = %s\n" % (error.real_id))
    if not options.client:
        f.write(_1_TAB + "Category = Offsets.%s\n" % error.category)
        f.write(_1_TAB + "id       = %s\n" % error.id)
        f.write(_1_TAB + "text     = _(u\"%s\")\n" % error.text)


def GenerateObjects(config, g_objects):
    obj_name_list = g_objects.keys()
    obj_name_list.sort()
    for obj_name in obj_name_list:
        obj = g_objects[obj_name]
        GenerateObject(obj, g_objects)


def GenerateServices(config, services, g_objects):
    global g_services
    g_services = services

    # Need to create the dispatcher
    path = [config.output, ] + config.gen_path.split('.')
    path.append("v%s" % config.current_version.version)
    path.append("sv%s" % config.current_version.sub_version)
    path.append("__init__.py")

    if not os.path.exists(os.path.dirname(os.path.join(*path))):
        os.makedirs(os.path.dirname(os.path.join(*path)))

    if not config.options.client:
        with open(os.path.join(*path), "w") as f:

            f.write('\nimport logging')
            f.write('\ndef load_dispatcher():')
            f.write('\n    declared_function_list = {}')
            f.write('\n    capabilities = []')
            f.write('\n    from dispatch import services')

            f.write('\n    for service_view, service_class in services.iteritems():')
            f.write('\n        splitted = service_class.split(".")')
            f.write('\n        module_name = ".".join(splitted[:-1])')
            f.write('\n        class_name = splitted[len(splitted) -1]')
            f.write('\n        try:\n')
            f.write('\n            __module__ = __import__(module_name, fromlist=[class_name,])')
            f.write('\n        except ImportError, exc:')
            f.write('\n            logging.debug("Import Error %s" % str(exc))')
            f.write('\n            logging.debug("Disabling feature %s" % service_view)')
            f.write('\n            continue')

            f.write('\n        cls = getattr(__module__, class_name)')
            f.write('\n        if not cls.__load_in_dispatcher__():\n')
            f.write('\n            logging.debug("Disabling feature %s. Requested by Config" % service_view)\n')
            f.write('\n            continue\n')
            f.write('\n        capabilities.append(service_view)')
            f.write('\n        for func in cls.validators:')
            f.write('\n            declared_function_list["%s.%s" % (service_view, func)] = (cls, func)')

            f.write('\n    return capabilities, declared_function_list')

        path = [config.output, ] + config.gen_path.split('.')
        path.append("v%s" % config.current_version.version)
        path.append("sv%s" % config.current_version.sub_version)
        path.append("dispatch.py")
        with open(os.path.join(*path), "w") as f:
            f.write("services = {\n")

            keys = [(ser.identification, ser_name) for ser_name, ser in
                    services.iteritems()]
            keys.sort()

            for drop, ser_name in keys:
                ser = services[ser_name]
                if ser.interface:
                    continue
                #if config.min_version.is_lower(ser.version) or\
                #ser.version.equals(config.min_version):
                GenerateService(ser, g_objects)
                f.write(_1_TAB + "\"%s\": \"%s.%s.%s.%s\",\n" % (
                    ser.identification,
                    config.gen_path,
                    ser.get_import_path(config.min_version, PYTHON),
                    ser_name,
                    ser_name)
                )
            f.write("}\n")
    else:
        with open(os.path.join(config.output, "%s/cli.py" % config.lib_path), "w") as f:

            from cli import generate_cli_options, cli_pre, cli_post

            cli_pre(f, config)
            for ser_name, ser in services.iteritems():
                generate_cli_options(f, config, ser, g_objects, g_services)
                GenerateService(ser, g_objects)
            cli_post(f, config)


def GenerateErrors(config, errors, errors_offsets):
    if config.options.client:
        path = [config.output, config.lib_path]
    else:
        path = [config.output, ] + config.gen_path.split('.')
        path.append("v%s" % config.current_version.version)
        path.append("sv%s" % config.current_version.sub_version)

    start = ""
    for p in path:
        start = os.path.join(start, p)
        if os.path.exists(start):
            pass
        else:
            os.mkdir(start)
        init = os.path.join(start, "__init__.py")
        if not os.path.exists(init):
            with open(init, "w") as f:
                pass

    with open("%s/errors.py" % os.path.join(*path), "w") as f:
        if config.options.client:
            f.write("from transformer.lib.errors import ClientError as Error\n")
            f.write("from transformer.lib.errors import (error_table, AuthenticationError, TimeIsSkewedError,\n")
            f.write("\t\tWrongCredentialsError, NotEnoughRights, UnknownToken, ElapsedToken, WrongValue, UnknownResource)\n")
        else:
            f.write("from transformer.lib.errors import ServerError as Error\n")

        f.write("try:\n")
        f.write(_1_TAB + "from unicorn.core.translation import _\n")
        f.write("except:\n")
        f.write(_1_TAB + "class _(object):\n")
        f.write(_2_TAB + "def __init__(self, text):\n")
        f.write(_3_TAB + "self.text = text\n")


        f.write("\n\n")
        f.write("class Offsets:\n")

        for name, value in errors_offsets.iteritems():
            f.write(_1_TAB + "%s = %s\n" % (name, value))
        keys = errors.keys()
        keys.sort()
        for error_id in keys:
            error = errors[error_id]
            if error.generate:
                GenerateError(f, error, errors_offsets, config.options)

        if config.options.client:
            for error_name, error in errors.iteritems():
                f.write("error_table[%s] = %s\n" % (error.real_id, error.name))


def pre_generate_action(config):
    if config.options.client:
        path = [config.output, config.lib_path]
    else:
        path = [config.output, ] + config.gen_path.split('.')

    if not os.path.exists(os.path.join(*path)):
        os.makedirs(os.path.join(*path))
    return set()


def post_generate_action(config):
    return


class PythonConfig(object):
    def __init__(self):
        self.client = True
        self.async = False

    def set_client(self):
        self.client = True

    def set_server(self):
        self.client = False

    def set_async(self):
        self.async = True


from optparse import OptionGroup, OptionValueError


def options_check(option, opt_str, value, parser):
    value = []
    for arg in parser.rargs:
        if arg[:2] == "--" or\
            arg not in ["client", "server"]:
            break
        elif (arg == "client" and "server" in value) or \
         (arg == "server" and "client" in value):
            raise OptionValueError("Can't have server and client")
        getattr(parser.python, "set_%s" % arg)()
        value.append(arg)
    del parser.rargs[:len(value)]
    parser.generation.append(PYTHON)


def get_options(parser):
    group = OptionGroup(parser, "Python Client Options Options",
                        "Those are the options specific for Python in client mode")
    group.add_option(
        "--python",
        action="callback",
        help="Generate for Python client. Available parameters [client|server]",
        callback=options_check
    )
    return (group, PythonConfig())
