#!/usr/bin/python
# -*- coding: utf-8 -*-

#                         LACIE CONFIDENTIAL
#
# The information contained in this file is the property of LACIE.
# The user of this file shall keep information contained herein confidential
# and shall protect it in whole or in part from disclosure and dissemination
# to third parties.
# Without prior written consent of LACIE, you may not reproduce, represent,
# or download through any means, the information contained herein in any way
# or in any form for a purpose other that the use for internal evaluation,
# operation or maintenance within LACIE.
#
#          (c) LACIE, 2009 - 2012, All Rights Reserved.


def cli_pre(f, config):
    f.write("from argparse import ArgumentParser, Action\n")
    f.write("import sys\n")
    f.write("import os\n")
    f.write("from transformer.lib import ListInfo, Dict, List\n")
    f.write("from transformer.lib.errors import ClientError\n")
    f.write("from transformer.lib.cli import (cli_list_info, list_cli, \
             dict_cli, cli_int, cli_long, cli_unicode, cli_date, cli_bool)\n")
    f.write("import %s.errors\n" % config.lib_path)
    f.write("try:\n")
    f.write("\tfrom transformer.plugins.lacie import LacieClientAuthentication\n")
    f.write("\tlacie_auth = LacieClientAuthentication()\n")
    f.write("except:\n")
    f.write("\tlacie_auth = None\n")
    f.write("try:\n")
    f.write("\tfrom transformer.plugins.application import ApplicationClientAuthentication\n")
    f.write("\tapp_auth = ApplicationClientAuthentication()\n")
    f.write("except:\n")
    f.write("\tapp_auth = None\n")

    f.write("function = None\n")
    f.write("func_args = []\n")
    f.write("interactive = None\n")
    f.write("methods = {}\n")

    #Now define some default types

    description = "This cli is generated and will allow sending some requests to a given NAS.\\n"\
                  "unicorn_api -i ipaddress -u user_name -p user_password --myrequest param1 param2\\n"\
                  "If the param expected is a list use the \",\" seperator\\n"\
                  "If the param expected is an object, separate the fieldname and the value by a \":\" \\n"\
                  "and separate the different fields with a \",\". \\n"\
                  "If you want to provide an empty field, just add \"\" on the command line\\n"\
                  "If the field expected is a boolean values are 0 or 1\\n"

    f.write("def main():\n")
    f.write("\tglobal function, func_args\n")
    f.write("\tparser = ArgumentParser(description='%s', add_help=False)\n" % description)
    f.write("\tparser.add_argument(\"--all\", action=\"store_true\", dest=\"all\", default=False, help=\"Return all the available requests\")\n")
    f.write("\tparser.add_argument(\"--interactive\", action=\"store_true\", dest=\"interactive\", default=False, help=\"Interactive call. You'll be prompted for requiered fields\")\n")
    f.write("\tparser.add_argument(\"-i\",\"--ip\", action=\"store\", dest=\"ip\", type=unicode, default=None)\n")
    f.write("\tparser.add_argument(\"-a\", \"--async\", action=\"store\", dest=\"async\", type=int, default=None, help=\"Provided job id it will return current job status\")\n")
    f.write("\tparser.add_argument(\"-s\", \"--search\", action=\"store\", nargs='+', metavar='search_param ...', dest=\"search\", default=None, help=\"Search for a strings in the available requests.\")\n")
    f.write("\tparser.add_argument(\"--port\", action=\"store\", dest=\"port\", default=None, type=int)\n")

    f.write("\tif lacie_auth is not None:\n")
    f.write("\t\tparser.add_argument(\"-u\",\"--user\", action=\"store\", dest=\"user\", default=None)\n")
    f.write("\t\tparser.add_argument(\"-p\",\"--password\", action=\"store\", dest=\"password\", type=unicode, default=None)\n")
    f.write("\tif app_auth is not None:\n")
    f.write("\t\tparser.add_argument(\"--app\", action=\"store\", dest=\"app\", default=None, nargs=2, metavar=('app_id', 'app_token'))\n")
    f.write("\t\tparser.add_argument(\"--app_token\", action=\"store\", dest=\"app_token\", default=None, type=unicode)\n")
    f.write("\t\tparser.add_argument(\"--app_session\", action=\"store\", dest=\"app_session\", default=None, type=unicode)\n")
    f.write("\t\tparser.add_argument(\"--app_user\", action=\"store\", dest=\"app_user\", default=None, type=unicode)\n")

    f.write("\t\tparser.add_argument(\"--register_app\", action=\"store\", dest=\"register_app\", default=None, nargs=3, metavar=('app_id', 'mandatory_perms', 'optionals_perms'), help='Call the register app method')\n")
    f.write("\t\tparser.add_argument(\"--unregister_app\", action=\"store\", dest=\"unregister_app\", default=None, nargs=2, metavar=('app_id', 'app_token'), help='Call the unregister app method')\n")
    f.write("\t\tparser.add_argument(\"--register_status\", action=\"store\", dest=\"register_status\", default=None, type=int, help='Get current registration status')\n")
    f.write("\t\tparser.add_argument(\"--open_session\", action=\"store_true\", dest=\"open_session\", help='Get application session token', default=None)\n")

    f.write("\tif len(sys.argv) == 2 and (\"-h\" in sys.argv or \"--help\" in sys.argv):\n")
    f.write("\t\tparser.print_help()\n")
    f.write("\t\treturn\n")
    f.write("\t(options, args) = parser.parse_known_args()\n")
    f.write("\tsearch = False\n")
    f.write("\tinteractive_mode = options.interactive\n")
    f.write("\tif options.search is not None:\n")
    f.write("\t\tsearch = True\n")

    f.write("\tsubparsers = parser.add_subparsers()\n")


def cli_post(f, config):
    f.write("\tif search is True:\n")
    f.write("\t\tfor key in methods:\n")
    f.write("\t\t\tmatch=True\n")
    f.write("\t\t\tfor arg in options.search:\n")
    f.write("\t\t\t\tif not arg.lower() in key.lower():\n")
    f.write("\t\t\t\t\tmatch = False\n")
    f.write("\t\t\t\t\tbreak\n")
    f.write("\t\t\tif match is True:\n")
    f.write("\t\t\t\tprint key\n")
    f.write("\t\t\t\tprint \"\t\t%s %s\" % methods[key]\n")
    f.write("\t\texit(0)\n")

    f.write("\tfrom transformer.client_transport import set_client\n")

    f.write("\tip = u\"127.0.0.1\"\n ")
    f.write("\tport = 80\n")
    f.write("\tif options.ip is not None:\n")
    f.write("\t\tip = options.ip\n")
    f.write("\tif options.port is not None:\n")
    f.write("\t\tport = options.port\n")

    f.write("\tfrom transformer.transport.client_http import HTTPClient\n")
    f.write("\tfrom transformer.lib import ListInfo, Dict\n")
    f.write("\tclient = HTTPClient(ip, port)\n")
    f.write("\tset_client(client)\n")
    f.write("\tauth_forced = False\n")

    f.write("\tif lacie_auth is not None:\n")
    f.write("\t\tuser = u\"admin\"\n ")
    f.write("\t\tpassword = u\"admin\"\n")
    f.write("\t\tset_lacie_auth = False\n")
    f.write("\t\tif options.user is not None:\n")
    f.write("\t\t\tlacie_auth.set_user(options.user)\n")
    f.write("\t\t\tset_lacie_auth = True\n")
    f.write("\t\tif options.password is not None:\n")
    f.write("\t\t\tlacie_auth.set_password(options.password)\n")
    f.write("\t\t\tset_lacie_auth = True\n")
    f.write("\t\tif set_lacie_auth:\n")
    f.write("\t\t\tclient.set_authentication(lacie_auth)\n")
    f.write("\t\t\tauth_forced = True\n")

    f.write("\tif app_auth is not None:\n")
    #Now check for all the stuff regarding apps.
    f.write("\t\tif options.app is not None:\n")
    f.write("\t\t\tapp_auth.set_app_id(options.app[0])\n")
    f.write("\t\t\tapp_auth.set_app_token(options.app[1])\n")
    f.write("\t\t\tclient.set_authentication(app_auth)\n")
    f.write("\t\t\tauth_forced=True\n")

    f.write("\t\tif options.app_session is not None:\n")
    f.write("\t\t\tapp_auth.set_session(options.app_session)\n")
    f.write("\t\t\tclient.set_authentication(app_auth)\n")
    f.write("\t\tif options.app_user is not None:\n")
    f.write("\t\t\tapp_auth.set_user_token(options.app_user)\n")
    f.write("\t\t\tclient.set_authentication(app_auth)\n")

    f.write("\tif not auth_forced:\n")
    f.write("\t\t#Get the information for the environment variables\n")
    f.write("\t\tapp_id = os.environ.get('APP_ID', None)\n")
    f.write("\t\tapp_token = os.environ.get('APP_TOKEN', None)\n")
    f.write("\t\tif app_id is not None and app_token is not None:\n")
    f.write("\t\t\tapp_auth.set_app_id(app_id)\n")
    f.write("\t\t\tapp_auth.set_app_token(app_token)\n")
    f.write("\t\t\tclient.set_authentication(app_auth)\n")

    f.write("\tdef cleanup_string(value):\n")
    f.write("\t\tif value.startswith('\"') or value.startswith('\\''):\n")
    f.write("\t\t\treturn value[1:-1]\n")
    f.write("\t\treturn value\n")
    f.write("\tif options.open_session is not None:\n")
    f.write("\t\tclient.set_authentication(app_auth)\n")
    f.write("\t\tfor k, v in app_auth.open_session().iteritems():\n")
    f.write("\t\t\tprint '%s: %s'% (k, v)\n")
    f.write("\t\treturn\n")
    f.write("\tif options.register_app is not None:\n")
    f.write("\t\tclient.set_authentication(app_auth)\n")
    f.write("\t\tapp_id = options.register_app[0]\n")
    f.write("\t\tmandatory_perms = cleanup_string(options.register_app[1]).split(',')\n")
    f.write("\t\toptionals_perms = []\n")
    f.write("\t\tif options.register_app[2]:\n")
    f.write("\t\t\toptionals_perms = cleanup_string(options.register_app[2]).split(',')\n")
    f.write("\t\tprint app_auth.request_auth_token(app_id, mandatory_perms, optionals_perms)\n")
    f.write("\t\treturn\n")
    f.write("\tif options.unregister_app is not None:\n")
    f.write("\t\tclient.set_authentication(app_auth)\n")
    f.write("\t\tapp_id = options.unregister_app[0]\n")
    f.write("\t\tapp_token = options.unregister_app[1]\n")
    f.write("\t\tapp_auth.set_app_id(app_id)\n")
    f.write("\t\tapp_auth.set_app_token(app_token)\n")
    f.write("\t\tapp_auth.remove_auth_token()\n")
    f.write("\t\treturn\n")
    f.write("\telif options.register_status:\n")
    f.write("\t\tclient.set_authentication(app_auth)\n")
    f.write("\t\tapp_auth.req_id = options.register_status\n")
    f.write("\t\tres = app_auth.get_current_auth_status()\n")
    f.write("\t\tprint 'status %s\\ntoken %s' % (res['status'], res['token'])\n")
    f.write("\t\treturn\n")

    f.write("\tif options.async is not None:\n")
    f.write("\t\tfunction = client.get_job_status\n")
    f.write("\t\tfunc_args.append(options.async)\n")

    f.write("\toptions = parser.parse_args()\n")

    f.write("\t#now make the call\n")
    f.write("\tif function is None:\n")
    f.write("\t\tparser.print_help()\n")
    f.write("\t\treturn\n")
    f.write("\tif interactive_mode:\n")
    f.write("\t\tprint('@@@@@@@@@@Starting interactive mode@@@@@@@@\\n')\n")
    f.write("\t\tfrom transformer.lib.cli import interactive_builder\n")
    f.write("\t\tfunc_args = interactive_builder(interactive)\n")
    f.write("\ttry:\n")
    f.write("\t\ttemp = function(*func_args)\n")
    f.write("\t\ttemp = temp.__str__()\n")
    f.write("\t\tprint temp.encode('utf8')\n")
    f.write("\texcept ClientError, exc:\n")
    f.write("\t\tprint '%s: %s' % (exc.real_id, exc.text)\n")
    f.write("\t\texit(-2)\n")
    f.write("if __name__ == \"__main__\":\n")
    f.write("\tmain()\n")


def build_cli_object(v_type, objects):
    from optimus_prime.utils import list_param, dict_param, Enum
    '''
    Build the object expected format
    '''
    if isinstance(v_type, str):
        if v_type in objects:
            v_type = objects[v_type]
        else:
            class Dummy(object):
                def __init__(self, name):
                    self.name = name
            v_type = Dummy(v_type)

    if isinstance(v_type, list_param):
        return '["%s",...]' % build_cli_object(v_type.name, objects)
    elif isinstance(v_type, dict_param):
        return '{"key":%s,...}' % build_cli_object(v_type.name, objects)
    else:
        name = v_type.name

        if name == "Integer":
            return "int"
        elif name == "Long":

            return "long"
        elif name == "Boolean":

            return "True|False"
        elif name == "Unicode":

            return "text"
        elif name == "ListInfo":
            return "offset:XXX,limit:YYYY,search_parameters:{key:value,key2:value2}"
        elif name == "Date":

            return "long"
        elif name == "IP":
            return "www.xxx.yyy.zzz"
        elif name == "Blob":
            return "somestuff"
        else:
            #Ok now loop on the elements
            obj = objects[v_type.name]
            if isinstance(obj, Enum):
                return "%s" % "|".join(["%s" % k for k, v in obj.values.iteritems() if (not k == "UNKNOWN")])

            string = []
            opt = []

            attributes = obj.get_attributes()

            for key, value in attributes.iteritems():
                if value.write():
                    if value.type.is_optional:
                        opt.append('#"%s":%s#' % (key, build_cli_object(value.type, objects)))
                    else:
                        string.append('"%s":%s' % (key, build_cli_object(value.type, objects)))

            string.extend(opt)
            return "{%s}" % ",".join(string)


def generate_cli_options(f, config, service, objects, g_services):
    from optimus_prime.plugins.python import get_imports
    from optimus_prime.utils import Enum
    from collections import OrderedDict
    #f.write("\tgroup = OptionGroup(parser, \"Options for service %s\",\n"Methods that can be called on the service\")\n" % service.name)

    if service.interface:
        return
    f.write("\t%s = subparsers.add_parser('%s', help=\"help %s\")\n" % (service.identification.replace(".", "_"), service.identification.replace(".", "_"), service.identification))


    functions = {}
    for func_name in service.functions:
        functions[func_name] = service.functions[func_name]
    if service.extends:
        for extend in service.extends:
            interface = g_services[extend]
            functions.update(interface.functions)

    for function in functions.values():
        #Generate a callback that will convert the input into correct values.
        #For now on if entry is too complicated skip it...
        #Hardcode ListInfo to 0 -1 no search parameters for now

        f.write("\tclass %sAction(Action):\n" % ("%s_%s" % ("_".join(service.identification.split(".")), function.func_name)))
        f.write("\t\tdef __call__(self, parser, namespace, values, option_string=None):\n")
        checkers = OrderedDict()
        help = []
        to_import = []

        from optimus_prime.utils import list_param, dict_param
        if function.async:
            checkers["async_job_name"] = "cli_unicode"
        for key, v_type in function.params[0]:

            if isinstance(v_type, list_param):
                if v_type.name == "Integer":
                    name = "int"
                elif v_type.name == "Long":
                    name = "long"
                elif v_type.name == "Boolean":
                    name = "cli_bool"
                elif v_type.name == "Unicode":
                    name = "unicode"
                else:
                    name = v_type.name
                checkers[key] = "list_cli(%s)" % name

            elif isinstance(v_type, dict_param):
                if v_type.name == "Integer":
                    name = "int"
                elif v_type.name == "Long":
                    name = "long"
                elif v_type.name == "Boolean":
                    name = "cli_bool"
                elif v_type.name == "Unicode":
                    name = "unicode"
                else:
                    name = v_type.name

                checkers[key] = "dict_cli(%s)" % name
            else:
                name = v_type.name

                if name == "Integer":
                    checkers[key] = "cli_int"

                elif name == "Long":
                    checkers[key] = "cli_long"

                elif name == "Boolean":
                    checkers[key] = "cli_bool"

                elif name == "Unicode":
                    checkers[key] = "cli_unicode"

                elif name == "ListInfo":
                    checkers[key] = "cli_list_info"

                elif name == "Date":
                    checkers[key] = "cli_date"

                else:
                    to_import.append(v_type)
                    checkers[key] = "%s" % name
            help.append("'%s'" % build_cli_object(v_type, objects))

        f.write(get_imports(service, objects, 3))
        f.write("\t\t\tchecker = [%s]\n" % ",".join(checkers.values()))
        f.write("\t\t\tglobal func_args\n")
        f.write("\t\t\tfunc_args = []\n")
        #f.write("\t\t\tprint 'checker %s' % checker\n")
        #f.write("\t\t\tprint 'values %s' % values\n")
        #f.write("\t\t\tprint 'namespace %s' % namespace\n")
        f.write("\t\t\tfor i in range(0, len(values)):\n")
        f.write("\t\t\t\tfunc_args.append(checker[i].from_cli(values[i]))\n")

        path = config.lib_path.split('.')
        path.append(service.get_import_path(objects.config.min_version, "python"),)
        path.append(service.name)
        f.write("\t\t\tfrom %s import %s\n" % (".".join(path), service.name))
        f.write("\t\t\tservice = %s()\n" % service.name)
        f.write("\t\t\tglobal function, interactive\n")
        f.write("\t\t\tfunction = service.%s\n" % function.func_name)
        f.write("\t\t\tinteractive = []\n")
        params = []
        i = 0
        for key, checker in checkers.iteritems():
            f.write("\t\t\tinteractive.append(('%s', %s))\n" % (key, checker))
            i += 1
            params.append(key)
        f.write("\tmethods['%s'] = ('%s', '--%s')\n" % ("%s.%s" % (service.identification, function.func_name), service.identification.replace(".", "_"), function.func_name))
        #Now we have to display the required field
        metavars = ",".join(["'%s'" % p for p in params])
        f.write("\tmetavars = () if interactive_mode else (%s)\n" % metavars)
        f.write("\thelp = '' if interactive_mode else '%s' %% (%s)\n" % ("%s " * len(help),  ", ".join(help)))
        f.write("\tnargs = 0 if interactive_mode else %s\n" % len(params))
        f.write("\t%s.add_argument(\"--%s\", action=%sAction, nargs=nargs, metavar=metavars, help=help)\n" % (service.identification.replace(".", "_"), function.func_name, ("%s_%s" % ("_".join(service.identification.split(".")), function.func_name))))
        if function.async:
            f.write("\tclass %sResultAction(Action):\n" % ("%s_%s" % ("_".join(service.identification.split(".")), function.func_name)))
            f.write("\t\tdef __call__(self, parser, namespace, values, option_string=None):\n")
            f.write("\t\t\tchecker = [cli_int]\n")
            f.write("\t\t\tglobal func_args\n")
            f.write("\t\t\tfunc_args = []\n")
            f.write("\t\t\tfor i in range(0, len(values)):\n")
            f.write("\t\t\t\tfunc_args.append(checker[i].from_cli(values[i]))\n")
            path = config.lib_path.split('.')
            path.append(service.get_import_path(objects.config.min_version, "python"),)
            path.append(service.name)
            f.write("\t\t\tfrom %s import %s\n" % (".".join(path), service.name))
            f.write("\t\t\tservice = %s()\n" % service.name)
            f.write("\t\t\tglobal function, interactive\n")
            f.write("\t\t\tfunction = service.%s_result\n" % function.func_name)
            f.write("\t\t\tinteractive = []\n")
            f.write("\t\t\tinteractive.append(('async_job_id', cli_int))\n")
            f.write("\tmethods['%s'] = ('%s', '--%s_result')\n" % ("%s.%s_result" % (service.identification, function.func_name), service.identification.replace(".", "_"), function.func_name))
            f.write("\tmetavars = () if interactive_mode else ('async_job_id')\n")
            f.write("\tnargs = 0 if interactive_mode else 1\n")
            f.write("\t%s.add_argument(\"--%s_result\", action=%sResultAction, nargs=nargs, metavar=metavars)\n" % (service.identification.replace(".", "_"), function.func_name, ("%s_%s" % ("_".join(service.identification.split(".")), function.func_name))))
