#!/usr/bin/python
import os

import diffcp
import shutil


info = False
verbose = False

stats = {
    "c_dir":0,
    "c_file":0,
    "d_dir":0,
    "d_file":0
    }

def walkme(arg, dirname, files):
    folder1_dir = os.path.join(arg["folder1"], dirname[len(arg["folder2"])+1:])
    if ".svn" in dirname:
        return
    for f in files:
        file_path = os.path.join(folder1_dir, f)
        dst_path = os.path.join(dirname, f)
        if verbose:
            print "COMPARING %s with %s" % (file_path, dst_path)
        if ".svn" in file_path:
            continue
        if os.path.exists(file_path):
            if verbose:
                print "Do nothing for %s" % file_path
        elif os.path.isdir(dst_path):
            if not info:
                shutil.rmtree(dst_path)
            if verbose:
                print "Removing DIR %s" % dst_path

            stats["d_dir"] += 1
        else:
            if verbose:
                print "Removing File %s" % dst_path
            if not info:
                os.remove(dst_path)
            stats["d_file"] += 1



def copyme(arg, dirname, files):
    folder2_dir = os.path.join(arg["folder2"], dirname[len(arg["folder1"])+1:])
    for f in files:
        f1 = os.path.join(dirname, f)
        f2 = os.path.join(folder2_dir, f)
        if os.path.isdir(f1):
            if not os.path.exists(f2):
                if not info:
                    os.mkdir(f2)
                if verbose:
                    print "creating folder %s"  % f2

                stats["c_dir"] += 1
            continue
        else:
            copy = diffcp.copy(f1,f2, info)
            if verbose and copy:
                print "Copying %s to %s" % (f1, f2)

            if copy:
                stats["c_file"] += 1

###########
#TEST PART#
###########


def make_folder(parent, name):
    if parent is None:
        parent = ""
    folder_path = os.path.join(parent, name)
    if verbose:
        print "MAKING folder %s" % folder_path
    os.mkdir(folder_path)
    with open(os.path.join(folder_path, "__init__.py"), "w") as f:
        for i in range(0, diffcp.GEN_DATE_OFFSET):
            f.write("LINE%s\n" % i)
        import time
        f.write("%sLINE\n" % time.time())

def gen_text(file_path, random_value):
    import random
    import hashlib
    import time
    string = ""
    if random_value:
        for l in range(0, random.randint(4, 20)):
            string += ("%s\n" % hashlib.sha1("%s" % time.time()).hexdigest())
    else:
        for i in range(0, 20):
            string += "LINE%s" % i
    return string

def make_file(parent, file_config):
    random = False
    if isinstance(file_config, tuple):
        name = file_config[0]
        random = file_config[1]
    else:
        name = file_config

    if parent is None:
        parent = ""
    file_path = os.path.join(parent, name)
    print "MAKING file %s" % file_path
    with open(file_path, "w") as f:
        f.write(gen_text(file_path, random))
def create_it(test_structure, parent):

    for k,v in test_structure.iteritems():
        if isinstance(v, dict):
            make_folder(parent, k)
            if parent is None:
                temp = k
            else:
                temp = os.path.join(parent, k)
            create_it(v, temp)
        elif isinstance(v, list):
            for f in v:
                make_file(parent, f)



def test():
    test_structure = \
    {
    "folder1":
        {
            "sub_folder1":
                {

                    "files": ["toto", ("titi", True), "tata"]
                },
            "sub_folder2":
                {
                    "files": ["toto", "titi", ("tata", True)]
                }
        },
    "folder2":
        {
            "sub_folder1":
                {

                    "files": ["toto", "tata", "tutu"]
                },
            "sub_folder3":
                {
                    "files": ["toto", "titi", "tata"]
                }
        }
    }

    create_it(test_structure, None)

if __name__ == "__main__":
    from optparse import OptionParser
    parser = OptionParser()

    parser.add_option("-t", "--test",
                      action="store_true", dest="test", default=False,
                      help="run tests")

    parser.add_option("-i", "--info",
                        action="store_true", dest="info", default=False,
                        help="Show commands but do not execute")
    parser.add_option("-v", "--verbose",
                        action="store_true", dest="verbose", default=False,
                        help="Display stuff we are doing")
    (options, args) = parser.parse_args()
    print args
    print options
    if options.test:
        test()
        exit(0)

    info = options.info
    if info:
        verbose = True
    else:
        verbose = options.verbose

    folder1 = args[0]
    folder2 = args[1]
    config = {
                "folder2": folder2,
                "folder1": folder1
    }
    if not os.path.exists(folder2):
        print "FAILED %s does not exists" % folder2
        exit(-1)
    os.path.walk(folder2, walkme, config)
    os.path.walk(folder1, copyme, config)


    if verbose:
        print "********************"
        print "Folder removed %s" % stats["d_dir"]
        print "File removed %s" % stats["d_file"]
        print "Folder created %s" % stats["c_dir"]
        print "File copied %s" % stats["c_file"]
