from transformer.lib.errors import MissingProperty, UnexpectedProperty
#import transformer.lib.requests
import base64
import logging

base_serializers = {"Unicode": unicode,
                    "Integer": int,
                    "Boolean": bool,
                    }
base_type_to_string = {}

for key, value in base_serializers.iteritems():
    base_type_to_string[value] = key

instantiator_list = {}


def to_string(tabs, elem):
    if elem is None:
        return "%sNone" % ("  "*tabs)
    if type(elem) in [str, int, bool, long, unicode, float]:
        return "%s%s\n" % ("  "*tabs, elem)
    if hasattr(elem, "to_string"):
        return elem.to_string(tabs)
    else:
        return str(elem)


def get_attributes(cls, exclude_methods=True):
    """
    Get class attributes as a list
    """
    base_attrs = dir(type('dummy', (cls.__mro__[1],), {}))
    this_cls_attrs = dir(cls)
    res = []
    for attr in this_cls_attrs:
        if (base_attrs.count(attr) or
            (callable(getattr(cls, attr)) and exclude_methods)
        ) or attr.startswith("__"):
            continue
        res += [attr]
    return res


def get_values(cls, exclude_methods=True):
    """
    Get class values as a list
    """
    return [getattr(cls, key) for key in get_attributes(cls, exclude_methods)]


def sub_type_is_raw_type(subtype):
    return subtype.__name__ in [Date.__name__, IP.__name__, Long.__name__, ULong.__name__, Double.__name__]


def register_instantiator(context, instantiator, client=False):
    if not (context in instantiator_list):
        instantiator_list[context] = {}
        register_instantiator(context, ListInstantiator)
        register_instantiator(context, DictInstantiator)
        register_instantiator(context, DateInstantiator)
        register_instantiator(context, TimeInstantiator)
        register_instantiator(context, ResultSetInstantiator)
        register_instantiator(context, BlobInstantiator)
        register_instantiator(context, IPInstantiator)
        register_instantiator(context, LongInstantiator)
        register_instantiator(context, ULongInstantiator)
        register_instantiator(context, DoubleInstantiator)
        register_instantiator(context, ListInfoInstantiator)
        register_instantiator(context, OrderingInstantiator)
        register_instantiator(context, AsyncJobInstantiator)
        register_instantiator(context, EnumInstantiator)
        register_instantiator(context, NotificationInstantiator)

    store_dict = instantiator_list[context]
    if client:
        store_dict[instantiator.name()] = instantiator

    if hasattr(instantiator, "version"):

        instantiator_list[context].setdefault(instantiator.version, {})
        instantiator_list[context][instantiator.version].setdefault(instantiator.sub_version, {})
        store_dict = instantiator_list[context][instantiator.version][instantiator.sub_version]

    store_dict[instantiator.name()] = instantiator


def get_instantiator(context, type_name, version, sub_version):
    instantiator_dict = instantiator_list[context]

    if version is not None:
        #Now we have to check loop on subversion than version
        #And finally look in default
        found = False
        v_keys = instantiator_dict.keys()
        v_keys.sort(reverse=True)
        for v in v_keys:
            if v > version:
                continue
            if found:
                break
            temp_dict = instantiator_dict[v]
            keys = temp_dict.keys()
            keys.sort(reverse=True)
            for sv in keys:
                if v == version and sv > sub_version:
                    continue
                sub_temp_dict = temp_dict[sv]
                if type_name in sub_temp_dict:
                    instantiator_dict = sub_temp_dict
                    found = True
                    break

    instantiator = instantiator_dict[type_name]
    return instantiator


class Instantiator(object):
    attributes = None

    @classmethod
    def name(self):
        raise NotImplemented("name")

    @classmethod
    def new_object(cls):
        raise NotImplemented("new_object")

    @classmethod
    def serialize(cls, context, obj, with_params, ser):
        d = dict()
        d['__type__'] = cls.name()
        d['__properties__'] = dict()

        if hasattr(cls, "version"):
            d['__version__'] = cls.version
            d['__sub_version__'] = cls.sub_version
        for attr in cls.attributes:
            v_type = cls.attributes[attr]
            if isinstance(v_type, Optional) and not hasattr(obj, attr):
                pass
            elif attr in cls.w_attributes:
                if with_params is not None and (attr in cls.h_attributes) and not (attr in with_params):
                    continue
                value = getattr(obj, attr)
                try:
                    d['__properties__'][attr] = ser.serialize(context, value, with_params, v_type)
                except:
                    logging.debug("Failed to serialize %s with value %s" % (attr, value))
                    raise
        return d

    @classmethod
    def deserialize(cls, context, value, instantiator,  ser, version=None, sub_version=None):

        obj = instantiator.new_object()
        if cls.attributes is None:
            return cls.deserialize(context, obj, version=version, sub_version=sub_version)
        else:
            for attr in cls.attributes:
                instantiator = cls.attributes[attr]
                if attr in cls.op_values and not value['__properties__'].has_key(attr):
                    setattr(obj, attr, cls.op_values[attr])
                elif attr in cls.r_attributes:
                    try:
                        if attr in cls.h_attributes and not value["__properties__"].has_key(attr):
                            continue
                        attr_value = value["__properties__"][attr]
                    except KeyError:
                        raise MissingProperty(property_name=attr)

                    setattr(obj, attr, ser.deserialize(context, attr_value, instantiator, ser, version=version, sub_version=sub_version))

           # value['__properties__'].has_key(attr) and not (attr in\
           #             cls.r_attributes):
           #         raise UnexpectedProperty(object_name=cls.name(), property_name=attr)

        return obj


class ListInstantiator(Instantiator):

    @classmethod
    def name(cls):
        return u"List"

    @classmethod
    def new_object(cls):
        return List()

    @classmethod
    def serialize(cls, context, obj, with_params, serializer):
        d = dict()
        d['__type__']       = obj.__class__.__name__
        ser = serializer
        if hasattr(obj.sub_type, "__instantiator__"):
            d['__sub_type__']   = obj.sub_type.__instantiator__.name()
            ser = obj.sub_type.__instantiator__
        elif obj.sub_type in base_type_to_string:
            d['__sub_type__']   = base_type_to_string[obj.sub_type]
        else:
            d['__sub_type__'] = obj.sub_type.__name__

        d['__elements__']   = []
        for element in obj:
            d['__elements__'].append(ser.serialize(context, element, with_params,
                serializer))
        return d

    @classmethod
    def deserialize(cls, context, value, instantiator, serializer, version=None, sub_version=None):
        # Retrieve the type of objects that will be added to the list
        sub_type = value['__sub_type__']
        ser = serializer
        obj = instantiator.new_object()
        if sub_type in base_serializers:
            obj.sub_type = base_serializers[sub_type]
        else:
            if "__sub_type_version__" in value:
                version = value['__sub_type_version__']
                sub_version = value['__sub_type_sub_version__']
            ser = get_instantiator(context, sub_type, version, sub_version)
            obj.sub_type = ser.new_object().__class__

        sub_instantiator = None
        if hasattr(obj.sub_type, "__instantiator__"):
            sub_instantiator = ser
        for element in value['__elements__']:
            obj.append(ser.deserialize(context, element, sub_instantiator, serializer, version=version, sub_version=sub_version))
        return obj


class List(list):
    __instantiator__ = ListInstantiator

    def __init__(self, sub_type=None, in_list=None):
        self.sub_type = sub_type
        if in_list is not None:
            [self.append(e) for e in in_list]

    def append(self, element):
        if not isinstance(element, self.sub_type) and not sub_type_is_raw_type(self.sub_type):
            raise Exception("Can't add %s expected %s" % (type(element), self.sub_type))
        super(List, self).append(element)

    def __call__(self, backend_list):
        new_list = List(self.sub_type)
        for e in backend_list:
            new_list.append(self.sub_type(e))
        return new_list

    def get_backend_obj(self):
        if sub_type_is_raw_type(self.sub_type) or self.sub_type in base_serializers.values():
            return self

        backend_list = List(self.sub_type.__backend__)

        for e in self:
            backend_list.append(e.get_backend_obj())

        return backend_list

    def from_cli(self, value):
        from cli import list_cli
        cli = list_cli(self.sub_type)
        return cli.from_cli(value)

    def to_string(self, tab):
        if sub_type_is_raw_type(self.sub_type) or self.sub_type in base_serializers.values():
            return "%s%s\n" % ("  " * tab, str([e for e in self]))
        string = "%s[" % ("  " * tab)
        if len(self) == 0:
            string += "]\n"
            return string
        string += '\n'
        for e in self[:-1]:
            string += to_string(tab+1, e)
            string += "----\n"
        string += to_string(tab+1, self[-1])
        string += "%s]\n" % ("  "*tab)
        return string

    def __str__(self):
        return self.to_string(0)


class ResultSetInstantiator(Instantiator):

    @classmethod
    def name(cls):
        return u"ResultSet"

    @classmethod
    def new_object(cls):
        return ResultSet()

    @classmethod
    def serialize(cls, context, obj, with_params, serializer):
        d = dict()
        d['__type__']           = obj.__class__.__name__
        d['__total_items__']    = obj.total_items
        ser = serializer
        if hasattr(obj.sub_type, "__instantiator__"):
            d['__sub_type__']   = obj.sub_type.__instantiator__.name()
            ser = obj.sub_type.__instantiator__

        elif obj.sub_type in base_type_to_string:
            d['__sub_type__']   = base_type_to_string[obj.sub_type]
        else:
            d['__sub_type__'] = obj.sub_type.__class__.__name__

        d['__elements__']   = []
        for element in obj:
            d['__elements__'].append(ser.serialize(context, element, with_params,
                serializer))
        return d

    @classmethod
    def deserialize(cls, context, value, instantiator,  serializer, version=None, sub_version=None):
        # Retrieve the type of objects that will be added to the list
        sub_type = value['__sub_type__']
        ser = serializer
        obj = instantiator.new_object()
        if sub_type in base_serializers:
            obj.sub_type = base_serializers[sub_type]
        else:

            if "__sub_type_version__" in value:
                version = value['__sub_type_version__']
                sub_version = value['__sub_type_sub_version__']
            ser = get_instantiator(context, sub_type, version, sub_version)

            obj.sub_type = ser.new_object().__class__

        obj_instantiator = None
        if hasattr(obj.sub_type, '__instantiator__'):
            obj_instantiator = obj.sub_type.__instantiator__

        for element in value['__elements__']:
            obj.append(ser.deserialize(context, element, obj_instantiator, serializer, version=version, sub_version=sub_version))
        obj.total_items = value['__total_items__']
        return obj


class ResultSet(list):
    __instantiator__ = ResultSetInstantiator

    def __init__(self, sub_type=None):
        self.sub_type = sub_type
        self.total_items = 0

    def append(self, element):
        if not isinstance(element, self.sub_type) and not \
                sub_type_is_raw_type(self.sub_type):
            raise Exception("Can't add %s expected %s" % (type(element), self.sub_type))
        super(ResultSet, self).append(element)

    def __call__(self, backend_result_set):
        new_result_set = ResultSet(self.sub_type)
        new_result_set.total_items = backend_result_set.total_items
        for e in backend_result_set:
            new_result_set.append(self.sub_type(e))
        return new_result_set

    def get_backend_obj(self):
        if sub_type_is_raw_type(self.sub_type) or self.sub_type in base_serializers.values():
            return self

        backend_result_set = ResultSet(self.sub_type.__backend__)

        for e in self:
            backend_result_set.append(e.get_backend_obj())
        backend_result_set.total_items = self.total_items

        return backend_result_set

    def to_string(self, tab):
        string = "%stotal_items:%s\n" % ("  " * tab, self.total_items)
        if sub_type_is_raw_type(self.sub_type) or self.sub_type in base_serializers.values():
            string += "%s%s\n" % ("  " * tab, str([e for e in self]))
            return
        string += "%s[" % ("  "*tab)
        if len(self) > 0:
            string += "\n"

            for e in self[:-1]:
                string += to_string(tab+1, e)
                string += "----\n"
            string += to_string(tab+1, self[-1])
        string += "%s]\n" % ("  "*tab)
        return string

    def __str__(self):
        return self.to_string(0)


class DictInstantiator(Instantiator):

    @classmethod
    def name(cls):
        return u"Dict"

    @classmethod
    def new_object(cls):
        return Dict()

    @classmethod
    def serialize(cls, context, obj, with_params, serializer):
        d = dict()
        d['__type__']       = obj.__class__.__name__
        ser = serializer
        if hasattr(obj.sub_type, "__instantiator__"):
            d['__sub_type__']   = obj.sub_type.__instantiator__.name()
            ser = obj.sub_type.__instantiator__
        elif obj.sub_type in base_type_to_string:
            d['__sub_type__']   = base_type_to_string[obj.sub_type]
        else:
            d['__sub_type__'] = obj.sub_type.__name__

        d['__elements__']   = {}
        for key, value in obj.iteritems():
            d['__elements__'][key] = ser.serialize(context, value, with_params,
                    serializer)
        return d

    @classmethod
    def deserialize(self, context, value, instantiator, serializer,version=None, sub_version=None):
        # Retrieve the type of objects that will be added to the list
        sub_type = value['__sub_type__']
        ser = serializer
        obj = instantiator.new_object()
        if sub_type in base_serializers:
            obj.sub_type = base_serializers[sub_type]
        else:

            if "__sub_type_version__" in value:
                version = value['__sub_type_version__']
                sub_version = value['__sub_type_sub_version__']
            ser = get_instantiator(context, sub_type, version, sub_version)

            obj.sub_type = ser.new_object().__class__

        obj_instantiator = None
        if hasattr(obj.sub_type, '__instantiator__'):
            obj_instantiator = obj.sub_type.__instantiator__

        for key, element in value['__elements__'].iteritems():
            obj.put(key, ser.deserialize(context, element, obj_instantiator, serializer, version=version, sub_version=sub_version))
        return obj


class Dict(dict):
    __instantiator__ = DictInstantiator

    def __init__(self, sub_type=None):
        self.sub_type = sub_type

    def put(self, key, value):
        self[key] = value

    def __call__(self, backend_dict):
        new_dict = Dict(self.sub_type)
        for k, v in backend_dict.iteritems():
            new_dict[k] = self.sub_type(v)
        return new_dict

    def get_backend_obj(self):

        if sub_type_is_raw_type(self.sub_type) or self.sub_type in base_serializers.values():
            return self

        backend_dict = Dict(self.sub_type)

        for k, v in self.iteritems():
            backend_dict[k] = v

        return backend_dict

    def to_string(self, tab):
        if sub_type_is_raw_type(self.sub_type) or self.sub_type in base_serializers.values():
            return super(Dict, self).__str__()
        dict_elems = {}
        for k, v in self.iteritems():
            dict_elems[k] = to_string(tab+1, v)
        return str(dict_elems)

    def __str__(self):
        return self.to_string(0)


class Optional(object):
    def __init__(self, sub_type=None):
        if hasattr(sub_type, "__instantiator__"):
            self.__instantiator__   = sub_type.__instantiator__
        self.sub_type           = sub_type

    def __call__(self, value):
        if value is None:
            return None
        else:
            return self.sub_type(value)

    def get_backend_obj(self):
        return self


class TimeInstantiator(object):
    @classmethod
    def name(cls):
        return u"Time"

    @classmethod
    def new_object(cls):
        return Time()

    @classmethod
    def serialize(cls, context, obj, with_params, serializer):
        if isinstance(obj, Time):
            obj = obj.value
        d = dict()
        d['__type__']    = cls.name()
        d['__value__']   = obj
        return d

    @classmethod
    def deserialize(self, context, value, instantiator, serializer, version=None, sub_version=None):
        obj = value['__value__']
        return obj

class Time(object):
    def __init__(self, value=None):
        self.value = value

    def get_backend_obj(self):
        return self

    __instantiator__ = TimeInstantiator

    def to_string(self, tabs):
        return "%s%s" % ("  "*tabs, self.value)

class DateInstantiator(object):
    @classmethod
    def name(cls):
        return u"Date"

    @classmethod
    def new_object(cls):
        return Date()

    @classmethod
    def serialize(cls, context, obj, with_params, serializer):
        if isinstance(obj, Date):
            obj = obj.value
        d = dict()
        d['__type__']    = cls.name()
        d['__value__']   = obj
        return d

    @classmethod
    def deserialize(self, context, value, instantiator, serializer, version=None, sub_version=None):
        obj = value['__value__']
        return obj

class Date(object):
    def __init__(self, value=None):
        self.value = value

    def get_backend_obj(self):
        return self

    __instantiator__ = DateInstantiator

    def to_string(self, tabs):
        return "%s%s" % ("  "*tabs, self.value)


class IPInstantiator(object):
    @classmethod
    def name(cls):
        return u"IP"

    @classmethod
    def new_object(cls):
        return IP()

    @classmethod
    def serialize(cls, context, obj, with_params, serializer):
        if isinstance(obj, IP):
            obj = obj.value
        d = dict()
        d['__type__'] = cls.name()
        d['__value__'] = obj
        return d

    @classmethod
    def deserialize(self, context, value, instantiator, serializer, version=None, sub_version=None):
        obj = value['__value__']
        return obj

class IP(object):
    def __init__(self, value=None):
        self.value = value

    def get_backend_obj(self):
        return self

    __instantiator__ = IPInstantiator

    def to_string(self, tabs):
        return "%s%s" % ("  "*tabs, self.value)


class BlobInstantiator(object):
    @classmethod
    def name(cls):
        return u"Blob"

    @classmethod
    def new_object(cls):
        return Blob()

    @classmethod
    def serialize(cls, context, obj, with_params, serializer):
        d = dict()
        d['__type__']    = cls.name()
        d['__value__']   = base64.b64encode(obj.datas)
        return d

    @classmethod
    def deserialize(self, context, value, instantiator, serializer, version=None, sub_version=None):
        obj = instantiator.new_object()
        obj.datas = base64.b64decode(value['__value__'])
        return obj


class Blob(object):
    __instantiator__ = BlobInstantiator

    def __init__(self, datas=None):
        self.datas = datas

    def __unicode__(self):
        return self.datas

    def get_backend_obj(self):
        return self


    def to_string(self, tabs):
        return "%s%s" % ("  "*tabs, self.datas)



class LongInstantiator(object):
    @classmethod
    def name(cls):
        return u"Long"

    @classmethod
    def new_object(cls):
        return Long()

    @classmethod
    def serialize(cls, context, obj, with_params, serializer):
        if isinstance(obj, Long):
            obj = obj.value
        d = dict()
        d['__type__']    = cls.name()
        d['__value__']   = u"%s" % obj
        return d

    @classmethod
    def deserialize(self, context, value, instantiator, serializer, version=None, sub_version=None):
        obj = long(value['__value__'])
        return obj


class Long(object):

    def __init__(self, value=None):
        self.value = value

    def get_backend_obj(self):
        return self

    def to_string(self, tabs):
        return "%s%s" % ("  "*tabs, self.value)


    __instantiator__ = LongInstantiator

class ULongInstantiator(object):
    @classmethod
    def name(cls):
        return u"ULong"

    @classmethod
    def new_object(cls):
        return ULong()

    @classmethod
    def serialize(cls, context, obj, with_params, serializer):
        if isinstance(obj, ULong):
            obj = obj.value
        d = dict()
        d['__type__']    = cls.name()
        d['__value__']   = u"%s" % obj
        return d

    @classmethod
    def deserialize(self, context, value, instantiator, serializer):
        obj = long(value['__value__'])
        return obj


class ULong(object):
    def __init__(self, value=None):
        self.value = value
    def get_backend_obj(self):
        return self

    def to_string(self, tabs):
        return "%s%s" % ("  "*tabs, self.value)


    __instantiator__ = ULongInstantiator

class DoubleInstantiator(object):
    @classmethod
    def name(cls):
        return u"Double"

    @classmethod
    def new_object(cls):
        return Double()

    @classmethod
    def serialize(cls, context, obj, with_params, serializer):
        if isinstance(obj, Double):
            obj = obj.value
        d = dict()
        d['__type__'] = cls.name()
        d['__value__'] = u"%s" % obj
        return d

    @classmethod
    def deserialize(self, context, value, instantiator, serializer, version=None, sub_version=None):
        obj = float(value['__value__'])
        return obj

class Double(object):
    def __init__(self, value=None):
        self.value = value
    def get_backend_obj(self):
        return self
    def to_string(self, tabs):
        return "%s%s" % ("  "*tabs, self.value)

    __instantiator__ = DoubleInstantiator

class OrderingInstantiator(Instantiator):
    attributes = {
        'asc': bool,
        'order_by': unicode,
    }
    w_attributes = ["asc", "order_by"]
    r_attributes = ["asc", "order_by"]
    h_attributes = []
    op_values = {}

    @classmethod
    def name(cls):
        return u'Ordering'

    @classmethod
    def new_object(cls):
        return Ordering()


class Ordering(object):
    __instantiator__ = OrderingInstantiator

    def get_backend_obj(self):
        return self

    def to_string(self, tabs):
        string = ""
        string += "%s{asc: %s, order_by: %s}" % ("  "*tabs, self.asc,
                self.order_by)
        return string



class ListInfoInstantiator(Instantiator):
    attributes = {
        'limit': int,
        'offset': int,
        'order': Optional(Ordering),
        'search_parameters': Optional(Dict(unicode)),
    }
    w_attributes = ["limit", "offset", "order", "search_parameters"]
    r_attributes = ["limit", "offset", "order", "search_parameters"]
    h_attributes = []
    op_values = {'order': None, 'search_parameters': None}

    @classmethod
    def name(cls):
        return u'ListInfo'

    @classmethod
    def new_object(cls):
        return ListInfo()


class ListInfo(object):
    __instantiator__ = ListInfoInstantiator

    def get_backend_obj(self):
        return self


class AsyncJobInstantiator(Instantiator):
    attributes = {
        'id': int,
        'progress': int,
        'status': int,
        'client_ref': unicode,
    }
    w_attributes = ["id", "progress", "status", "client_ref"]
    r_attributes = ["id", "progress", "status", "client_ref"]
    h_attributes = []
    op_values = {}

    @classmethod
    def name(cls):
        return u"AsyncJob"

    @classmethod
    def new_object(cls):
        return AsyncJob()


class AsyncJob(object):
    def get_backend_obj(self):
        return self

    def __str__(self):
        return self.to_string(0)

    def to_string(self, tab):
        string = ""
        for key, value in self.__dict__.iteritems():
            if type(value) in [str, int, bool, long, unicode]:
                string += "%s%s:%s\n" % ("  "*tab, key, value)
            else:
                string += "%s%s:%s\n" % ("  "*tab, key, to_string(tab + 1, value))
        return string
    __instantiator__ = AsyncJobInstantiator


class NotificationInstantiator(Instantiator):
    attributes = {
        "id": int,
        "event_id": unicode,
        "parameters": Dict(unicode)
    }

    w_attributes = ["id", "event_id", "parameters"]
    r_attributes = ["id", "event_id", "parameters"]
    h_attributes = []

    @classmethod
    def name(cls):
        return u"Notification"

    @classmethod
    def new_object(cls):
        return Notification()


class Notification(object):
    def __init__(self, event):
        self.event_id = event.event_id
        self.parameters = Dict(unicode)
        for key, value in event.parameters.iteritems():
            self.parameters[key] = value
        self.id = event.id

    __instantiator__ = NotificationInstantiator


class EnumInstantiator(object):

    @classmethod
    def name(cls):
        return "Enum"

    @classmethod
    def serialize(cls, context, obj, with_params, serializer):
        if isinstance(obj, int) or isinstance(obj, unicode):
            return obj
        return obj.value

    @classmethod
    def deserialize(self, context, value, instantiator, serializer,version=None, sub_version=None):
        return value


class Enum(object):
    def __init__(self, value=None):
        if value in get_attributes(self.__class__):
            self.value = getattr(self.__class__, value)
        elif value in get_values(self.__class__):
            self.value = value
        else:
            raise ValueError("Unknown value %s" % value)

    def get_backend_obj(self):
        return self.value

    def to_string(self, tabs):
        return "%s%s" % ("  "*tabs, self.value)

    @classmethod
    def interactive_mode(cls, key):
        values = get_attributes(cls)
        text = "|".join(v for v in values if (not v == "UNKNOWN"))
        temp = ''
        while temp == '':
            temp = raw_input("Enter value for %s:\nExpecting %s\n" % (key, text))
        return temp

    __instantiator__ = EnumInstantiator


def private(function):
    function.private = True
    return function

instantiator_list["default"] = {}
register_instantiator("default", ListInstantiator)
register_instantiator("default", DictInstantiator)
register_instantiator("default", DateInstantiator)
register_instantiator("default", TimeInstantiator)
register_instantiator("default", ResultSetInstantiator)
register_instantiator("default", BlobInstantiator)
register_instantiator("default", IPInstantiator)
register_instantiator("default", LongInstantiator)
register_instantiator("default", ULongInstantiator)
register_instantiator("default", DoubleInstantiator)
register_instantiator("default", ListInfoInstantiator)
register_instantiator("default", OrderingInstantiator)
register_instantiator("default", AsyncJobInstantiator)
register_instantiator("default", EnumInstantiator)
register_instantiator("default", NotificationInstantiator)


