from transformer.lib import ListInfo, Dict, List
from transformer.lib.tools import bool_from_string

import json

class cli_int(object):

    @classmethod
    def from_cli(cls, value):
        return int(value)


class cli_long(object):

    @classmethod
    def from_cli(cls, value):
        return long(value)


class cli_bool(object):
    @classmethod
    def from_cli(cls, value):
        if value in ("true", "True", "1", 1):
            return True
        if value in ("false", "False", "0", 0):
            return False
        raise ValueError("Invalid value for bool %s" % value)


class cli_unicode(object):

    @classmethod
    def from_cli(cls, value):
        return value.decode("utf8")


class cli_date(object):

    @classmethod
    def from_date(cls, value):
        return long(value)


class int_to_bool(object):

    @classmethod
    def from_cli(cls, value):
        return bool(int(value))


class cli_list_info(object):
    @classmethod
    def from_cli(cls, input_value):
        list_info = ListInfo()

        values = input_value.split(",")
        values_dict = {}

        list_info.offset = 0
        list_info.limit = -1
        if input_value != "":
            for value in values:
                k, v = value.split(":")
                values_dict[k.strip()] = v.strip()

            print values_dict
            #Check the offset and limit parameters
            if "offset" in values_dict:
                list_info.offset = int(values_dict["offset"])
                del values_dict["offset"]

            if "limit" in values_dict:
                list_info.limit = int(values_dict["limit"])
                del values_dict["limit"]

        list_info.search_parameters = Dict(unicode)
        for k, v in values_dict.iteritems():
            list_info.search_parameters[k] = v
        return list_info


class list_cli(object):
    def __init__(self, sub_type):
        self.sub_type = sub_type

    def from_cli(self, value):
        if isinstance(value, basestring):
            elements = json.loads(value)
        else:
            elements = value

        if hasattr(self.sub_type, "from_cli"):
            return List(self.sub_type, [self.sub_type.from_cli(element) for element in elements])
        else:
            return List(self.sub_type, [self.sub_type(element) for element in elements])


class dict_cli(object):
    def __init__(self, sub_type):
        self.sub_type = sub_type

    def from_cli(self, value):

        print value

        if isinstance(value, basestring):
            value = json.loads(value)


        if self.sub_type == bool:
            convert = bool_from_string
        elif self.sub_type == int:
            convert = int
        elif self.sub_type == unicode:
            convert = unicode
        else:
            convert = self.sub_type.from_cli

        res = Dict(self.sub_type)

        for k, v in value.iteritems():
            res.put(k, convert(v))

        return res



class ClientObject(object):

    def to_string(self, tab):
        string = ""
        for key, value in self.__dict__.iteritems():
            if value is None:
                string += "%s%s: None\n" % ("  "*tab, key)
                continue
            #Check if value should be an enum or not.

            if type(value) in [str, int, bool, long, unicode, float]:
                from transformer.lib import get_attributes, Enum, Optional
                enum_class = self.__instantiator__.attributes[key]
                if isinstance(enum_class, Optional):
                    enum_class = enum_class.sub_type
                if issubclass(enum_class, Enum):
                    text = None

                    for attribute in get_attributes(enum_class):
                        if value == getattr(enum_class, attribute):
                            text = attribute
                            break
                    string += "%s%s:%s\n" % ("  " * tab, key, text)
                else:
                    string += "%s%s:%s\n" % ("  " * tab, key, value)
            else:
                string += "%s%s:\n%s" % ("  " * tab, key, value.to_string(tab + 1))
        return string

    @classmethod
    def from_cli(cls, params):
        from transformer.lib import Optional
        if isinstance(params, basestring):
            params = json.loads(params)
        res = cls()
        for key, value in params.iteritems():
            obj = cls.__instantiator__.attributes[key]
            if (not type(obj) == type) and isinstance(obj, Optional):
                obj = obj.sub_type
            if obj == bool:
                value = bool_from_string(value)
            elif obj == int:
                value = int(value)
            elif obj == unicode:
                value = unicode(value)
            else:
                value = obj.from_cli(value)
            func = getattr(res, "set%s" % key)
            func(value)
        return res

    @classmethod
    def cli_fields(cls):
        '''
        This will return the mandatory fields for an object as well as the optionals
        '''
        string = ""
        opt = ""
        for key in cls.__instantiator__.w_attributes:
            if not key in cls.__instantiator__.op_values:
                obj = cls.__instantiator__.attributes[key]
                if isinstance(obj, ClientObject):
                    string += obj.cli_fields()
                else:
                    string += "%s:XXX" % key
            else:
                obj = cls.__instantiator__.attributes[key]
                if isinstance(obj, ClientObject):
                    opt += obj.cli_fields()
                else:
                    opt += "[%s:XXX]" % key
        return string + opt

    @classmethod
    def interactive_mode(cls, key, tab=1):
        from transformer.lib import get_attributes, Enum
        result = {}
        for key in cls.__instantiator__.w_attributes:
            obj = cls.__instantiator__.attributes[key]
            if isinstance(obj, ClientObject):

                if key in cls.__instantiator__.op_values:
                    temp = ''
                    while not temp in ("Y", "N", "n", "y", "yes", "no"):
                        temp = raw_input("%sWould to like to build optional object %s: [Y/N]\n")
                    if temp in ("N", "n", "no"):
                        continue

                result[key] = obj.interactive_mode(key, tab + 1)
            else:
                text = ""
                if key in cls.__instantiator__.op_values:
                    obj = obj.sub_type

                if obj == bool:
                    text = "True|False"
                elif (type(obj) == type) and issubclass(obj, Enum):
                    values = get_attributes(obj)
                    text = "|".join(v for v in values if (not v == "UNKNOWN"))
                elif obj == int:
                    text = "int"
                elif obj == long:
                    text = "long"
                elif obj == unicode:
                    text = "text"

                temp = ''
                while temp == '':
                    extra = ""
                    if key in cls.__instantiator__.op_values:
                        extra = "(empty is skipped)"
                    temp = raw_input("%sEnter value for %s %s:\nExpecting %s\n%s" % ("  " * tab, key, extra, text, "  " * tab))
                    if key in cls.__instantiator__.op_values and temp == '':
                        break
                    try:
                        if hasattr(obj, "from_cli"):
                            #Try to parse the input value
                            obj.from_cli(temp)
                        else:
                            obj(temp)
                    except Exception:
                        print("Wrong value entered")
                        temp = ""
                if not temp == "":
                    result[key] = temp
        return "(%s)" % ",".join(["%s:%s" % (k, v) for k, v in result.iteritems()])

    def __str__(self):
        return self.to_string(0)


def interactive_builder(interactive_list):
    '''
    Make the interactive things in here.
    '''
    result = {}
    for key, value in interactive_list:
        if hasattr(value, "interactive_mode"):
            result[key] = value.interactive_mode(key)
        else:
            result[key] = raw_input("Enter value for %s\n" % key)

    args = []
    for key, value in interactive_list:
        args.append(value.from_cli(result[key]))
    return args
