class Offsets:
    AUTHENTICATION  = 0
    SERIALIZATION   = 200
    GENERIC         = 210


class Error(Exception):
    Category = -1

    def __str__(self):
        return u"%d: %s" % (self.real_id, self.text)

# This will be filled by the client errors
error_table = {}


class ClientError(Error):
    real_id = -1
    text    = u""

    def __init__(self, text):
        self.text = text
        super(ClientError, self).__init__()


class real_id_me(type):
    def __new__(meta, name, bases, atts):
        #for att, value in atts.iteritems():
        #    setattr(me, att, value)
        #me.real_id = me.class_id * 10
        if not bases[0].__name__ == "ServerError":
            category = bases[0].Category
        else:
            category = atts["Category"]
        atts["real_id"] = category + atts["id"]

        me = super(real_id_me, meta).__new__(meta, name, bases, atts)
        error_table[me.real_id] = me
        return me


class ServerError(Error):
    __metaclass__ = real_id_me

    Category    = -1
    id          = -1
    text        = ""

    def __init__(self, *args, **kwargs):
        import traceback
        self.line   = traceback.format_stack()[-2].split("\n")[0]
        self.kwargs = kwargs
        self.args   = args

        super(ServerError, self).__init__()
        #string = self.text
        #if "_" == type(self.text).__name__:
        #    string = string.text
        #for key, value in self.kwargs.iteritems():
        #    string = string.replace("%s" % key, value)
        #if "_" == type(self.text).__name__:
        #    self.text.text = string
        #else:
        #    self.text = string

    def __str__(self):
        return self.text


class BadPropertyType(ServerError):
    Category = Offsets.SERIALIZATION
    id       = 2
    text     = u"Bad Property {property_name} in {object_name}"


class MalformedBody(ServerError):
    Category = Offsets.SERIALIZATION
    id       = 3
    text     = u"Malformed Body: {exc}"


class MissingProperty(ServerError):
    Category = Offsets.SERIALIZATION
    id       = 1
    text     = u"Missing Property {property_name}"


class UnexpectedProperty(ServerError):
    Category = Offsets.SERIALIZATION
    id       = 5
    text     = u"{object_name} has an unexpected property {property_name}"


class ServerWrongFunctionPrototype(ServerError):
    Category = Offsets.SERIALIZATION
    id       = 4
    text     = u"{function_name} has a different function prototype"


class WrongValue(ServerError):
    Category    = Offsets.GENERIC
    id          = 1
    text        = u"Wrong value {value} for parameter {parameter}"


class UnknownResource(ServerError):
    Category    = Offsets.GENERIC
    id          = 2
    text        = u"Unknown resource {name} {id}"


class AuthenticationError(ServerError):
    Category = Offsets.AUTHENTICATION
    id       = 1
    text     = u"Authentication failed"


class NotAllowed(ServerError):
    Category    = Offsets.GENERIC
    id          = 3
    text        = u"Action not Allowed"


class AuthenticationFailedError(ServerError):
    id = 0
    Category = Offsets.AUTHENTICATION
    text = "{reason}"


class TimeIsSkewedError(AuthenticationFailedError):
    id = 1
    text = 'Time is skewed by {days} days and {seconds} seconds'


class WrongCredentialsError(AuthenticationFailedError):
    id = 2


class NotEnoughRights(AuthenticationFailedError):
    '''
    The user does not have enough rights to access the resource or the resource's attribute
    '''
    id = 3
    text = "Not enough rights for {function_name}"


class UnknownToken(AuthenticationFailedError):
    '''
    The user send an unknown token
    '''
    id = 4
    text = u"Unknown token {token}"


class ElapsedToken(AuthenticationFailedError):
    '''
    Token is too old
    '''
    id = 5
    text = u"User token elapsed"


class UnknownApplicationToken(AuthenticationFailedError):
    id = 6
    text = u"Unknown application token {token}"

class ElapsedApplicationToken(AuthenticationFailedError):
    '''
    Application token is too old
    '''
    id = 7
    text = u"Application token elapsed"

