#!/usr/bin/python
# -*- coding: utf-8 -*-

#                         LACIE CONFIDENTIAL
#
# The information contained in this file is the property of LACIE.
# The user of this file shall keep information contained herein confidential
# and shall protect it in whole or in part from disclosure and dissemination
# to third parties.
# Without prior written consent of LACIE, you may not reproduce, represent,
# or download through any means, the information contained herein in any way
# or in any form for a purpose other that the use for internal evaluation,
# operation or maintenance within LACIE.
#
#          (c) LACIE, 2009 - 2010, All Rights Reserved.

import logging


class Instantiable(object):

    __instantiator__ = None
    __backend__      = None
    _backend_obj     = None
    _from_web        = False

    def __init__(self, backend_obj=None):
        if backend_obj is None:
            backend_obj = self.__backend__()
            object.__setattr__(self, "_from_web", True)
            #Values should be affected to
            object.__setattr__(self, "_backend_obj", backend_obj)
        else:
            object.__setattr__(self, "_from_web", False)
            #Now we should affect current object values to ourself
            for attr in self.__instantiator__.w_attributes:
                try:
                    try:
                        value = getattr(backend_obj, attr)
                        if value is None:
                            object.__setattr__(self, attr, None)
                        else:
                            object.__setattr__(self, attr,
                                    self.__instantiator__.attributes[attr](value))
                    except AttributeError:
                        #Check the attribute error and not the hasattr which can fail if evaluting the element fails
                        continue
                except Exception:
                    logging.debug("error serializing %s with value %s" % (attr,
                        getattr(backend_obj, attr)))
                    raise

    def __getattr__(self, key):
        if (key in ["__instantiator__", "_backend_obj", "__backend__",
        "_from_web"]):
            return object.__getattribute__(self, key)
        if not self._from_web:

            if (key in [self.__instantiator__.attributes]):
                value = object.__getattribute__(self, key)

                return value
        return getattr(self._backend_obj, key)

    def __setattr__(self, key, value):
        return setattr(self._backend_obj, key, value)

    def get_backend_obj(self):
        return self._backend_obj

    def to_string(self, tab):
        string = ""
        for key in self.__instantiator__.attributes.keys():
            if hasattr(self, key):
                value = getattr(self, key)
                if value is None:
                    return "%sNone" % tab
                if type(value) in [str, int, bool, long, unicode]:
                    string += "%s%s:%s\n" % ("  " * tab, key, value)
                else:
                    string += "%s%s:%s\n" % ("  " * tab, key, value.to_string(tab + 1))
        return string


class getter(object):

    def __init__(self, key, value):
        self.key = key
        self.value = value

    def __call__(self, obj):
        if obj.is_proxy:
            current_obj = obj.proxy_object
            split = self.value.split('.')
            for s in split[:-1]:
                current_obj = getattr(current_obj, s)
            return getattr(current_obj, split[-1])
        return getattr(obj, "_%s" % self.key)


class setter(object):

    def __init__(self, key, value):
        self.key = key
        self.value = value

    def __call__(self, obj, value):
        setattr(obj, "_%s" % self.key, value)


def proxy_class(cls):

    for key, value in cls.__proxying_config__.iteritems():
        g = getter(key, value)
        s = setter(key, value)
        setattr(cls, "get_%s" % key, g)
        setattr(cls, "set_%s" % key, s)
        setattr(cls, key, property(g, s))

    return cls


class ProxyClass(object):
    def __init__(self, proxy_class="TEXT_KILLER_CANT_BE_SET_BY_ONE"):
        self.proxy_object    = None
        self.is_proxy        = False
        if proxy_class is None:
            raise "Can't set None object as object for proxy"
        if proxy_class == "TEXT_KILLER_CANT_BE_SET_BY_ONE":
            return
        self.proxy_object   = proxy_class
        self.is_proxy       = True

    @classmethod
    def create_object(cls, obj):
        if obj is None:
            return None
        return cls(obj)


if __name__ == "__main__":
    @proxy_class
    class A(ProxyClass):
        __proxying_config__ = {
                    "key": "value",
                    "key1": "value2"
                }

    class B(object):

        value = "the value"
        value2 = "the value2"

    a = A()
    a.key = "toto"

    print a.key

    c = A(B())
    print c.key
    print c.key1
