#!/usr/bin/python
# -*- coding: utf-8 -*-
#                         LACIE CONFIDENTIAL
#
# The information contained in this file is the property of LACIE.
# The user of this file shall keep information contained herein confidential
# and shall protect it in whole or in part from disclosure and dissemination
# to third parties.
# Without prior written consent of LACIE, you may not reproduce, represent,
# or download through any means, the information contained herein in any way
# or in any form for a purpose other that the use for internal evaluation,
# operation or maintenance within LACIE.
#
#          (c) LACIE, 2009 - 2010, All Rights Reserved.

import json
import hashlib

APPLICATION_AUTH_DATE = "APP_AUTH"

APP_TOKEN = "APP_TOKEN"
APP_USER = "APP_USER"

LOGIN_URL           = "/api/external/authenticate/app_login"
LOGOUT_URL          = "/api/external/authenticate/app_logout"
APP_REGISTER_URL    = "/api/external/authenticate/application"
APP_STATUS_URL      = "/api/external/application/status"


class ApplicationClientAuthentication(object):

    def __init__(self):
        self.app_id = None
        self.app_token = None
        self.app_session = None
        self.user_token = None

    def set_app_id(self, app_id):
        self.app_id = app_id

    def set_session(self, app_session):
        self.session = app_session

    def set_app_token(self, app_token):
        self.app_token = app_token

    def set_user_token(self, user_token):
        self.user_token = user_token

    def forge_request(self, request, url, data, delta):
        if self.session is not None:
            request.headers["Authentication"] = "%s %s" % (APPLICATION_AUTH_DATE, self.session)

            if self.user_token is not None:
                request.headers[APP_USER] = self.user_token

        return request

    def open_session(self):
        req_body = {}
        resp = self.transport_layer.send_raw_request("GET", LOGIN_URL)
        if resp.code != 200:
            raise Exception("Failed to get challenge")
        resp_body = json.loads(resp.body)
        req_body["app_id"] = self.app_id
        req_body["challenge_id"] = resp_body["challenge_id"]
        req_body["secret"] = hashlib.sha1(str(resp_body["challenge"]) + self.app_token).hexdigest()

        response = self.transport_layer.send_raw_request("POST", LOGIN_URL, json.dumps(req_body))
        if response.code != 200:
            raise Exception("Failed to get token")
        json_resp = json.loads(response.body)
        self.session = json_resp["session_token"]
        return json_resp

    def request_auth_token(self, id, mandatory_perms, optional_perms):
        req_body = {}
        req_body["app_id"]          = id
        req_body["mandatory_perms"] = mandatory_perms
        req_body["optional_perms"]  = optional_perms

        response = self.transport_layer.send_raw_request("POST", APP_REGISTER_URL, json.dumps(req_body))

        if response.code == 200:
            json_dump = json.loads(response.body)
            self.req_id = json_dump["req_id"]
            return self.req_id
        raise Exception("Failed with error code %s: body %s", response.code, response.body)

    def remove_auth_token(self):
        req_body = {}
        req_body["token"] = self.app_token
        response = self.transport_layer.send_raw_request("POST", "%s/unregister/%s" % (APP_REGISTER_URL, self.app_id) , json.dumps(req_body))
        if response.code == 200:
            return True
        raise Exception("Failed with error code %s: body %s", response.code, response.body)


    def get_current_auth_status(self):
        response = self.transport_layer.send_raw_request("GET", "%s/%s" % (APP_REGISTER_URL, self.req_id))
        return json.loads(response.body)

    def parse_error(self, error_id):
        try:
            from transformer.lib.errors import ElapsedApplicationToken, UnknownApplicationToken
            from transformer.plugins import ErrorReturn

            if error_id == ElapsedApplicationToken.real_id or error_id == UnknownApplicationToken.real_id:
                #Reopen a session !!
                self.open_session()
                return ErrorReturn.RESEND
        except:
            pass
        return ErrorReturn.PASS
