import json

import transformer.serialization
from transformer.lib.errors import MalformedBody
from transformer.lib import Enum, List

try:
    #This should be replaced by some translation plugins system
    from unicorn.core.translation import translate as UTranslate, _ as translateClass
except:
    UTranslate = None
    translateClass = None


def translate(text, skip, items):
    if isinstance(text, translateClass):
        return UTranslate(text, skip, items)
    else:
        for key, value in items.iteritems():
            text = text.replace("{%s}" % key, str(value))
        return text

class Serializer(transformer.serialization.DefaultSerializer):
    TYPE = u"json"

    @classmethod
    def check(cls, context, json_value, validators, version=None, sub_version=None):
        res = cls.from_string(context, json_value, version=version, sub_version=sub_version)
        assert(len(res) == len(validators['input'])), "Wrong number of parameters: %s" % json_value
        for name, obj in validators['input'].iteritems():
            if type == type(obj):
                if issubclass(obj, Enum):
                    res[name] = obj(res[name])
            if isinstance(res[name], int) and long == obj:
                continue
            if isinstance(obj, List):
                if isinstance(res[name], list):
                    continue
                elif isinstance(res[name], List):
                    continue
            assert(isinstance(res[name], obj)), "found %s for %s expecting %s" % (type(res[name]), name, str(obj))
        return res

    @classmethod
    def to_string(cls, context,  ser, with_params):
        json_value = cls.serialize(context, ser, with_params)
        temp = json.dumps(json_value)
        return temp

    @classmethod
    def from_string(cls, context, json_value, version=None, sub_version=None):
        try:
            res_json = json.loads(json_value)
        except Exception, exc:
            raise MalformedBody(exc=str(exc))
        res = cls.deserialize(context, res_json, version=version, sub_version=sub_version)
        return res

    @classmethod
    def serialize_error(cls, context, error, language="en"):
        d_json = dict()
        d_json['real_id'] = error.real_id
        text = error.text
        translated_text = translate(text, language, error.kwargs)
        d_json['text'] = translated_text
        d_json['file_line'] = error.line
        return cls.to_string(context, d_json, None)

    @classmethod
    def deserialize_error(cls, context, error):
        try:
            res_json = json.loads(error)
        except Exception, exc:
            raise MalformedBody(exc=str(exc))

        error_code = int(res_json['real_id'])
        text = res_json['text']
        file_line = res_json['file_line']

        return error_code, text, file_line

    @classmethod
    def extract_bundle(cls, body):
        json_value = json.loads(body)

        requests_list = []
        for r in json_value:
            request         = transformer.serialization.BundleledRequest()
            request.data    = r['data']
            request.url     = r['url']
            request.uid     = r['uri']
            request.code    = r['code']
            requests_list.append(request)
        return requests_list

    @classmethod
    def serialize_bundle(cls, bundle):
        l = []
        for r in bundle:
            d           = {}
            d['data']   = r.data
            d['url']    = r.url
            d['uid']    = r.uid
            d['code']   = r.code

            l.append(d)

        return json.dumps(l)
