from tornado.httpclient import HTTPClient as tHTTPClient, HTTPRequest, HTTPError, AsyncHTTPClient
#from http_authentication import forge_signature, TOKEN_DATE
from tornado.ioloop import IOLoop
from transformer.serialization import BundleledRequest
from transformer.lib.errors import TimeIsSkewedError
#AsyncHTTPClient.configure(CurlAsyncHTTPClient)

import threading


EXTERNAL_PREFIX = u"api/external"


class HTTPClient(object):
    client                  = None
    async_client            = None
    async_loop              = None
    lock                    = threading.Lock()
    in_requests_bundle      = False
    requests_list           = []
    debug                   = False
    delta                   = 0
    authentication_plugin   = None

    def __init__(self, host, port):
        self.host       = host
        self.port       = port
        if HTTPClient.client is None:
            self.client = tHTTPClient()

    def start_async(self):
        if HTTPClient.async_client is None:
            self.async_loop = IOLoop()
            HTTPClient.async_client = AsyncHTTPClient(io_loop=self.async_loop,
                    force_instance=True)

            class AsyncThread(threading.Thread):
                def __init__(self, ioloop):
                    self.ioloop = ioloop
                    super(AsyncThread, self).__init__()

                def run(self):
                    self.ioloop.start()
            self.async_thread = AsyncThread(self.async_loop)
            self.async_thread.start()

    def stop(self):
        self.async_loop.stop()
        self.async_thread.join()

    def set_authentication(self, authentication_plugin):
        if authentication_plugin is not None:
            authentication_plugin.transport_layer = self
        self.authentication_plugin = authentication_plugin

    def build_request(self, url, body, private):
        req = HTTPRequest("http://%s:%s/%s" %
                (self.host, self.port, url), "POST", body=body,
                request_timeout=60)
        #TODO call the forge_request on the authentication_plugin
        if self.authentication_plugin is not None:
            req = self.authentication_plugin.forge_request(req, url, body, self.delta)
        return req

    def send(self, url, body, private=False):
        '''
        returns status, +body
        '''
        req = self.build_request(url, body, private)
        try:
            #client = tHTTPClient()
            self.lock.acquire()
            try:
                resp = self.client.fetch(req)
            finally:
                self.lock.release()
            #client.close()
            #del client
        except HTTPError, exc:
            resp = exc
            resp.body = exc.response.body
        try:
            return self.parse_response(resp)
        except TimeIsSkewedError:
            return self.send(url, body, private)

    def send_raw_request(self, method, url, body=None, headers={}):
        if not "/" == url[0]:
            url = "/%s" % url
        req = HTTPRequest("http://%s:%s%s" %
                (self.host, self.port, url), method, body=body)

        if self.debug:
            print ("Sending message to %s : %s" % (url, body))

        try:
            #client = tHTTPClient()
            self.lock.acquire()
            try:
                resp = self.client.fetch(req)
            finally:
                self.lock.release()
            #client.close()
            #del client
        except HTTPError, exc:
            resp = exc
            resp.body = exc.response.body

        return resp

    def parse_response(self, response):
        res = True
        if 401 == response.code:
            #TODO Check for time is squid
            headers = {}
            for k,v in response.response.headers.iteritems():
                headers[k.lower()] = v
            if 'auth_error' in headers:
                error_id = int(headers['auth_error'])
                if error_id == TimeIsSkewedError.id:
                    #Time is skewed
                    import time
                    import datetime
                    client_time_struct = time.strptime(
                        response.response.headers['Date'],
                        '%a, %d %b %Y %H:%M:%S %Z'
                    )
                    date_obj = datetime.datetime(*client_time_struct[:6])
                    delta = datetime.datetime.utcnow() - date_obj
                    self.delta = delta.total_seconds()
                    raise TimeIsSkewedError
                else:
                    if self.authentication_plugin is not None:
                        try:
                            from transformer.plugins import ErrorReturn
                            if ErrorReturn.RESEND == self.authentication_plugin.parse_error(error_id):
                                #For now raise a timeisskewed error and resend it to upper layers
                                raise TimeIsSkewedError
                        except AttributeError:
                            pass
                res = False
            else:
                raise Exception("Something went wrong %d: %s" %
                    (response.code, response.body))
        elif 500 == response.code:
            res = False

        elif 404 == response.code:
            raise Exception("Remote Service does not exists")

        if self.debug:
            import logging
            logging.debug("Received %s\n" % response.body)
        return res, response.body

    def async(self, url, body, callback, application_id):

        #import threading
        #class MyHandler(threading.Thread):
        #    def __init__(self, client, url, body, callback):
        #        self.client = client
        #        self.url = url
        #        self.body = body
        #        self.callback = callback
        #        super(MyHandler, self).__init__()

        #    def run(self):
        #        client = HTTPClient(self.client.host, self.client.port)
        #        client.user = self.client.user
        #        client.password = self.client.password
        #        res, body = client.send(self.url, self.body)
        #        cb = self.callback
        #        from transformer.client_serializer import ClientSerializer
        #        cb(ClientSerializer.from_string(res, body).values()[0])

        def cb(response):
            res, value = self.parse_response(response)
            response.rethrow()
            from transformer.client_serializer import ClientSerializer
            res = ClientSerializer.from_string(application_id, res, value).values()
            if res:
                callback(res[0])
            else:
                callback()

        if self.in_requests_bundle:
            self.requests_list.append((url, body, callback))
            return

        request = self.build_request(url, body, False)
        request.request_timeout = 0
        self.async_client.fetch(request, cb)

    def get_capabilities(self, version=None):

        prefix = EXTERNAL_PREFIX

        url = "%s/%s" % (prefix, u"capabilities")

        if version:
            url += "/%s" % version
        req = HTTPRequest("http://%s:%s/%s" %
                (self.host, self.port, url), "GET")

        try:
            #client = tHTTPClient()
            self.lock.acquire()
            try:
                resp = self.client.fetch(req)
            finally:
                self.lock.release()
            #client.close()
            #del client
        except HTTPError, exc:
            resp = exc
            resp.body = exc.response.body

        res, body = self.parse_response(resp)
        if res:
            import json
            capabilities = json.loads(body)
            return capabilities
        return None

    def get_job_status(self, job_id):
        from transformer.client_serializer import ClientSerializer
        dict_json = dict()
        dict_json["id"] = job_id
        data = ClientSerializer.to_string("default", dict_json)

        prefix = EXTERNAL_PREFIX

        url = "%s/%s" % (prefix, "async_status")
        req = HTTPRequest("http://%s:%s/%s" %
                (self.host, self.port, url), "POST", body=data)

        try:
            self.lock.acquire()
            try:
                resp = self.client.fetch(req)
            finally:
                self.lock.release()
        except HTTPError, exc:
            resp = exc
            resp.body = exc.response.body

        res, body = self.parse_response(resp)
        if res:
            return ClientSerializer.from_string("default", res,
            body)["async_job"]

    def get_jobs_list(self, list_info):
        from transformer.client_serializer import ClientSerializer
        dict_json = dict()
        dict_json["list_info"] = list_info
        data = ClientSerializer.to_string("default", dict_json)

        prefix = EXTERNAL_PREFIX

        url = "%s/%s" % (prefix, "async_list")
        req = HTTPRequest("http://%s:%s/%s" %
                (self.host, self.port, url), "POST", body=data)

        try:
            self.lock.acquire()
            try:
                resp = self.client.fetch(req)
            finally:
                self.lock.release()
        except HTTPError, exc:
            resp = exc
            resp.body = exc.response.body

        res, body = self.parse_response(resp)
        if res:
            return ClientSerializer.from_string("default", res,
            body)["async_jobs"]

    def get_version(self):
        url = "api/%s" % (u"get_version")
        req = HTTPRequest("http://%s:%s/%s" %
                (self.host, self.port, url), "GET")

        try:
            #client = tHTTPClient()
            self.lock.acquire()
            try:
                resp = self.client.fetch(req)
            finally:
                self.lock.release()
            #client.close()
            #del client
        except HTTPError, exc:
            resp = exc
            resp.body = exc.response.body

        res, body = self.parse_response(resp)
        if res:
            return body
        return None

    def get_supported_versions(self):
        url = "api/%s" % (u"get_supported_versions")
        req = HTTPRequest("http://%s:%s/%s" %
                (self.host, self.port, url), "GET")

        try:
            #client = tHTTPClient()
            self.lock.acquire()
            try:
                resp = self.client.fetch(req)
            finally:
                self.lock.release()
            #client.close()
            #del client
        except HTTPError, exc:
            resp = exc
            resp.body = exc.response.body

        res, body = self.parse_response(resp)
        if res:
            return body
        return None

    def start_requests_bundle(self):

        assert(not self.in_requests_bundle)
        self.in_requests_bundle = True

    def stop_requests_bundle(self):
        from transformer.client_serializer import ClientSerializer
        i = 0
        l = []
        for request in self.requests_list:
            re = BundleledRequest()

            re.uid  = i
            re.url  = request[0]
            re.data = request[1]
            i += 1
            l.append(request)

        def cb(response):
            res, value = self.parse_response(response)
            response.rethrow()
            responses = ClientSerializer.extract_bundles(value)
            for resp in responses:
                resp_tuple = self.requests_list[resp['uid']]
                resp_tuple[2](ClientSerializer.from_string(resp['code'],
                        resp['data']).values()[0])
            self.requests_list = []
            self.in_requests_bundle = False

        self.send("/api/external/bundle_request",
                ClientSerializer.serialize_bundle(l))
