#!/usr/bin/python
# -*- coding: utf-8 -*-

#                         LACIE CONFIDENTIAL
#
# The information contained in this file is the property of LACIE.
# The user of this file shall keep information contained herein confidential
# and shall protect it in whole or in part from disclosure and dissemination
# to third parties.
# Without prior written consent of LACIE, you may not reproduce, represent,
# or download through any means, the information contained herein in any way
# or in any form for a purpose other that the use for internal evaluation,
# operation or maintenance within LACIE.
#
#          (c) LACIE, 2009 - 2010, All Rights Reserved.

import logging
from transformer.transport.authentication import PluginType


class Authentication(object):
    plugins         = []
    header_plugins  = {}
    url_get_plugins = []

    @classmethod
    def get_authenticator(cls, request):
        """
        Check in the header of the request for the Authentication flag.

        The authentication flag should be set.

        Authentication: AUTH_ID AUTH_VALUE

        if not header Authentication is done then we look at the GET parameters
        """
        #First check in headers

        headers = {}
        for key, value in request.headers.iteritems():
            headers[key.lower()] = value

        if "authentication" in headers:
            auth_id = headers['authentication'].split(" ", 1)
            auth_id = auth_id[0].strip()
            if auth_id in cls.header_plugins:
                return cls.header_plugins[auth_id]
            return None

        #So either not header Authentication
        for authenticator in cls.url_get_plugins:
            if authenticator.can_handle_authentication(request):
                #Yes the actual parameters describe an authentication for this user
                return authenticator

        return None

    @classmethod
    def register_auth_plugin(cls, plugin):
        logging.debug("Registering %s ", plugin)
        cls.plugins.append(plugin)
        if plugin.auth_type & PluginType.HEADER:
            for auth_id in plugin.auth_ids:
                cls.header_plugins[auth_id] = plugin
        if plugin.auth_type & PluginType.GET_PARAMETERS:
            cls.url_get_plugins.append(plugin)

    def __init__(self, auth_struct):
        self.auth_struct = auth_struct
        self.authenticated = auth_struct.authenticated
        self.user = auth_struct.user
        try:
            self.application = auth_struct.application
        except:
            self.application = None

    def specific_authentication_key(self, key):
        return False

    def check_authentication(self, auth_list):

        for key in auth_list:
            # Now loop on all the main keywords
            if key not in ["public", "admin", "user"]:
                # Unknown authentication key call specific handler
                if self.specific_authentication_key(key):
                    return True
        if not self.auth_struct.authenticated or self.auth_struct.user is None:
            if "public" in auth_list:
                return True
            return False
        else:
            if "public" in auth_list:
                return True

        if "user" in auth_list:
            return True
        elif self.auth_struct.user.is_admin:
            return True
        return False
