import tornado.httpserver
class Transaction(object):

    def create_request(self):
        '''
        To forge a correct request, we have to compute the signature with the current data.
        '''
        # Imports
        import web.net
        import hmac
        from hashlib import sha1
        from base64 import b64encode
        from datetime import datetime


        def b64encode_unicode(data):
            '''
            Encode into base 64 the data
            '''
            return b64encode(data.encode('utf8'))

        def hmac_digest_unicode(password_sha1, message):
            '''
            compute using hmac the message
            '''
            return hmac.new(password_sha1, message, sha1).digest()

        def auth_signature(password_sha1, message):
            '''
            Encode in base 64 the already encoded message
            '''
            return b64encode(hmac_digest_unicode(password_sha1, message))

        # Set the request standard parameters
        password = self.user.specific.password_sha1
        headers = {
            'DATE-AUTH': web.net.httpdate(datetime.utcnow()),
            'Content-Type': 'text/plain; charset=utf-8',
        }

        #Work with every thing in UTF-8
        # Forge the request signature
        payload_data = self.data

        payload = '%s %s\nDate: %s\n%s' % ("POST", self.url,
                                           headers.get('DATE-AUTH', ''),
                                           payload_data, )

        signature = auth_signature(password.encode('utf8'), payload)

        # Set the request signature
        headers['Authentication'] = 'HMAC-SHA1-DATE %s:%s' % (
            b64encode_unicode(self.user.login),
            signature
        )

        self.raw_request(headers)



    class Connection(object):
        '''
        This fakes a connection
        '''
        class Stream(object):
            '''
            Implement the close function that is called by the tornado backend
            '''
            def close(self):
                pass

            def set_close_callback(self, on_close_action=None):
                self.on_close_action = on_close_action


        def __init__(self):
            self._data = []
            self.xheaders = {}

            self.status = ''
            self.headers = {}
            self.data = ''
            self.stream = self.Stream()

        def write(self, data):
            self._data.append(data)

        def finish(self):
            data_it = iter(''.join(self._data).split('\r\n'))
            self._data = []
            self.status = data_it.next()[9:]
            for line in data_it:
                if line == '':
                    break
                key, value = line.split(': ')
                self.headers[key] = value

            self.data = data_it.next()

    def __init__(self, url, data, user):

        self.url =  url
        self.data = data
        self.user = user
        self.connection = None
        self.request = None
        self.create_request()


    def raw_request(self, headers):
        '''
        create a HTTPRequest which can later be sent to the tornado application
        '''
        self.connection = self.Connection()
        self.request = tornado.httpserver.HTTPRequest(
            method="POST", uri=self.url, version='HTTP/1.0',
            headers=headers, body=self.data, remote_ip='127.0.0.1',
            protocol='http', host='localhost',
            files=None, connection=self.connection,
        )
        self.request.fake_sync = True

