import tornado.httpserver
import tornado.ioloop
import tornado.web
import sys
import threading

from transformer.serialization.s_json import Serializer as JSONSerializer

from http_authentication import Authentication
from async import DummyAsyncHandler
from transformer.lib.errors import AuthenticationFailedError
import transformer.lib.errors

from google_auth import AuthenticateWithGoogle


import logging


#Ok now let's load all the plugins
import os
path = os.path.dirname(os.path.abspath(__file__))
plugins_path = os.path.join(path, "../plugins")
for plugin in os.listdir(plugins_path):
    if plugin.endswith(".py") and plugin != "__init__.py":
        plugin_name = plugin[:-3]
    else:
        continue
    name = "transformer.plugins.%s" % plugin_name
    __import__(name)
    module = sys.modules[name]
    try:
        auth_plugin = module.auth_plugin
        Authentication.register_auth_plugin(auth_plugin())
    except AttributeError:
        pass

CURRENT_VERSION = u"3.0"

serializers = {
        JSONSerializer.TYPE: JSONSerializer,
    }

declared_function_list = {}
capabilities = {}


def get_capabilities():
    return capabilities

registered_functions = {
        "upload": None,
        "bundle_request": None
    }

context = u"api"

req_id = 0
req_lock = threading.Lock()


def get_new_id():
    global req_id, req_lock
    req_lock.acquire()
    req_id += 1
    value = req_id
    req_lock.release()
    return value


def register_context(string):
    global context
    context = string


def get_instance(cls, user):
    return cls(user)


def validate_request_authentication(auth_function, request):
    auth_struct = auth_function(request)
    auth_struct.authenticate()
    return auth_struct


class DefaultHandler(object):
    __authentication__  = Authentication
    __async_handler__   = DummyAsyncHandler

    def __init__(self, auth_struct):
        self.auth_struct = self.__authentication__(auth_struct)

_specific_handler = DefaultHandler


def set_specific_handler(handler):
    global _specific_handler
    _specific_handler = handler


class ThreadRequestHandler(threading.Thread):
    def __init__(self, *args, **kwargs):
        super(ThreadRequestHandler, self).__init__()
        self.transformer_request_handler = TransformerRequestHandler(*args,
                **kwargs)

    def run(self):

        self.transformer_request_handler.run()


class TransformerRequestHandler(object):

    def __init__(self, request_handler, serializer, cls, function,
            exceptions=None):
        self.request_handler    = request_handler
        self.serializer         = serializer
        self.cls                = cls
        self.function           = function
        self.exceptions         = exceptions

    def add_finish_callback(self, handler, status, body, headers):
        class write_and_finish_callback(object):
            def __init__(self, handler, status, body, header_infos):
                self.status             = status
                self.body               = body
                self.handler            = handler
                self.header_infos       = header_infos

            def __call__(self):
                for key, value in self.header_infos.iteritems():
                    self.handler.set_header(key, value)
                self.handler.set_status(self.status)
                self.handler.write(self.body)
                self.handler.finish()
        tornado.ioloop.IOLoop.instance().add_callback(write_and_finish_callback(handler,
                                                    status, body, headers))

    def finish(self):
        try:
            self.request_handler.finish()
        except IOError:
            logging.debug("IOError catched")
            self.request_handler._log()
        except Exception:
            logging.debug("DO NOT CRASH")

    def run(self):

        import web.net
        from datetime import datetime

        headers = {"Date": web.net.httpdate(datetime.utcnow())}
        try:
            # From the request header extract the authentication information
            authenticator = Authentication.get_authenticator(self.request_handler.request)

            if authenticator is not None:
                auth_struct = authenticator()
                auth_struct.authenticate(self.request_handler)
            else:
                class DummyAuthStruct(object):
                    user = None
                    authenticated = False
                    application = None


                auth_struct = DummyAuthStruct()
                auth_struct.request_handler = self.request_handler

        except AuthenticationFailedError, exc:
            headers["auth_error"] = exc.id
            self.add_finish_callback(self.request_handler, 401,
                    self.serializer.serialize_error(context, exc), headers)
            return
        except Exception, exc:
            if self.exceptions is not None:
                self.exceptions(exc)
            import traceback
            text = traceback.format_exc()
            logging.debug(text)
            self.add_finish_callback(self.request_handler, 500, str(exc), headers)
            return

        self.request_handler.pre_dispatch()
        language = self.request_handler.request.headers.get("Language-Code", "en")
        try:
            try:
                try:
                    # Now validate that the request is OK
                    #logging.debug("Received %s" %
                    #        self.request_handler.request.body)

                    #TODO get those information from the url...
                    version = 5
                    sub_version = 0

                    kwargs = self.serializer.check(context, self.request_handler.request.body,
                            self.cls.validators[self.function], version=version, sub_version=sub_version)
                    #logging.debug(unicode(kwargs))
                    #res, with_params = getattr(get_instance(cls, auth_struct), function)(auth_struct, **kwargs)

                    function_context = _specific_handler(auth_struct)
                    req_id = get_new_id()

                    try:
                        logging.info("Dealing with %s [%s]" % (self.function,
                            req_id))

                        #Calling the actual function
                        function = getattr(get_instance(self.cls, function_context), self.function)

                        if hasattr(function, 'private') and (not self.request_handler.request.uri.startswith("/api/local") or
                             not self.request_handler.request.remote_ip == "127.0.0.1"):

                            raise
                        res, with_params = function(function_context, **kwargs)
                    except TypeError, exc:
                        from traceback import format_exc
                        logging.debug(format_exc())
                        logging.debug(exc)
                        raise transformer.lib.errors.ServerWrongFunctionPrototype(function_name=self.function)

                except transformer.lib.errors.Error, exc:
                    self.add_finish_callback(self.request_handler, 500,
                            self.serializer.serialize_error(context, exc,
                                language), headers)
                    return

                except Exception, exc:
                    if self.exceptions is not None:
                        self.exceptions(exc)
                    import traceback
                    text =traceback.format_exc()
                    logging.debug(text)
                    self.add_finish_callback(self.request_handler, 500, str(exc), headers)
                    return
                self.add_finish_callback(self.request_handler, 200,
                    self.serializer.to_string(context, res, with_params), headers)



            except transformer.lib.errors.Error, exc :
                self.add_finish_callback(self.request_handler, 500,
                        self.serializer.serialize_error(context, exc, language), headers)
                return
            except Exception, exc:
                if self.exceptions is not None:
                    self.exceptions(exc)
                import traceback
                text =traceback.format_exc()
                logging.debug(text)
                self.add_finish_callback(self.request_handler, 500, str(exc), headers)
                return
            finally:
                logging.info("Finished dealing with %s [%s]" % (self.function,
                    req_id))

        finally:
            self.request_handler.post_dispatch()

async_notifiers = []


class NoThreadRequestHandler(ThreadRequestHandler):
    def start(self):
        self.run()


class TransformerEvent(object):
    event_id = u""
    parameters = {}


last_notification = 0
thread_lock = threading.Lock()
notifications = []


def notify(value):
    '''
        Value should be of type TransformerEvent
    '''
    from transformer.lib import Notification, List
    global async_notifiers, last_notification, thread_lock, notifications
    thread_lock.acquire()
    #Ensure that no other thread is sending an event
    last_notification += 1
    value.id = last_notification
    notifications.append(value)
    #Keep 10 notifications not more
    if len(notifications) > 10:
        notifications.pop(0)
    thread_lock.release()

    while 0 != len(async_notifiers):
        notifier = async_notifiers.pop()
        res = List(Notification)
        res.append(Notification(value))
        try:
            notifier.request_handler.write(notifier.serializer.to_string(context, {"notifications": res}, None))
            notifier.request_handler.finish()
        except:
            pass


class NotifyHandler(tornado.web.RequestHandler):
    @tornado.web.asynchronous
    def post(self, *args, **kwargs):
        global async_notifiers
        from transformer.transport.transport_http import serializers
        from transformer.lib import List, Notification
        #TODO we should look at the current header for the serialization.
        #This is not authenticated.
        serializer = serializers["json"]
        validator = {
            "input": {
                    "last_id": int
            },
            "output": {
                    "notifications": List(Notification)
            }
        }
        kwargs = serializer.check(context, self.request.body,  validator)

        #we don't care about anything since, the request is empty
        last_id = kwargs["last_id"]
        if last_id == -1 or last_id == last_notification:
            class Notifier(object):
                def __init__(self, request_handler, serializer):
                    self.request_handler = request_handler
                    self.serializer = serializer

            #New connection
            async_notifiers.append(Notifier(self, serializer))
            pass
        else:
            res = List(Notification)
            thread_lock.acquire()
            for notification in notifications:
                if last_id  < notification.id:
                    res.append(Notification(notification))
            thread_lock.release()
            self.write(serializer.to_string(context, {"notifications": res}, None))
            self.finish()


class VersionDispatcher(tornado.web.RequestHandler):
    __import_path__         = "api"
    __exceptions_handler__  = None

    @classmethod
    def pre_dispatch(self):
        pass

    @classmethod
    def post_dispatch(self):
        pass

    @classmethod
    def load_dispatchers(cls):
        '''
        Load all the dispatcher for the various versions of the api.
        '''
        import os.path

        __import__(cls.__import_path__)

        module = sys.modules[cls.__import_path__]

        external_path    = os.path.dirname(module.__file__)
        import_path = cls.__import_path__
        global declared_function_list, capabilities

        for dir_name in os.listdir(external_path):
            if dir_name.startswith('v'):
                v_list = []
                version_path = os.path.join(external_path, dir_name)
                for version_dir in os.listdir(version_path):
                    if os.path.isdir(os.path.join(version_path, version_dir)):
                        if version_dir.startswith('sv'):
                            v_list.append(int(version_dir[2:]))
                v_list.sort()
                for version_dir in v_list:
                    version_dir_path = os.path.join(external_path, dir_name, u"sv%d" % version_dir)
                    for f in os.listdir(version_dir_path):
                        version_import = u"%s.sv%s" % (dir_name, version_dir)
                        version = u"%s.%s" % (dir_name, version_dir)
                        declared_function_list.setdefault(version, {})
                        capabilities.setdefault(version, {})
                        if f == "__init__.py":
                            #Skip __init__.py files
                            import_file_name = f.replace(".py", "")
                            full_import_name = "%s.%s.%s" % (import_path, version_import, import_file_name)
                            try:
                                module = __import__(full_import_name, globals(), locals(), (str(import_file_name),), -1)
                            except Exception, exc:
                                logging.error(exc)
                            # Copy function from previous version

                            (capabilities[version], declared_function_list[version]) = module.load_dispatcher()

    @tornado.web.asynchronous
    def post(self, *args):
        version = u"v%s" % CURRENT_VERSION
        if len(args) == 2:
            version = u"v%s" % args[0]
            function = args[1]
        else:
            function = args[0]

        #Now check the serialization protocol.
        self.handle_function(version, function)

    def handle_function(self, version, function):
        try:
            (cls, function) = declared_function_list[version][function]
        except KeyError:
            self.set_status(404)
            self.write("Unknown function %s" % function)
            self.finish()
            return

        serializer = serializers[JSONSerializer.TYPE]

        new_thread = ThreadRequestHandler(self, serializer, cls,
                function, self.__exceptions_handler__)
        new_thread.start()


def set_dispatcher_path(path):
    VersionDispatcher.__import_path__ = path


class SupportedSerialization(tornado.web.RequestHandler):
    def get(self):
        return JSONSerializer.TYPE


class SupportedVersions(tornado.web.RequestHandler):
    def get(self):
        self.write([c for c in capabilities].join(';'))


class Version(tornado.web.RequestHandler):
    def get(self):
        return self.write(CURRENT_VERSION)


class Ready(tornado.web.RequestHandler):
    def get(self):
        return self.write(str(True))

    def post(self, *args, **kwargs):
        return self.write(str(True))


class UploadDispatcher(tornado.web.RequestHandler):
    def post(self):
        from webapp2.backend.v2.sv0.filebrowsing import Browse
        b = Browse()
        b.upload(self)
        return self.write("file writen")


class BundleRequest(tornado.web.RequestHandler):
    @tornado.web.asynchronous
    def post(self):

        auth_struct = validate_request_authentication(_specific_handler.__authentication__,
                self.request)
        serializer = serializers[JSONSerializer.TYPE]

        requests = serializer.extract_bundles(self.request.body)

        for r in requests:
            import re
            data = r['data']
            url = r['url']

            regexp = "/api/external/(\d.\d)/(.*)"
            m = re.match(regexp, url)
            if m:
                version = u"v%s" % CURRENT_VERSION
                if len(args) == 2:
                    version = u"v%s" % args[0]
                    function = args[1]
                else:
                    function = args[0]

                #Now check the serialization protocol.
                try:
                    (cls, function) = declared_function_list[version][function]
                except KeyError:
                    self.set_status(404)
                    self.write("Unknown function %s" % function)
                    self.finish()
                return


                try:
                    try:
                        # Now validate that the request is OK
                        kwargs = serializer.check(data,\
                                cls.validators[function])
                        #res, with_params = getattr(get_instance(cls, auth_struct), function)(auth_struct, **kwargs)
                        try:
                            logging.info("Dealing with %s" % function)

                            #Calling the actual function
                            res, with_params = getattr(get_instance(cls,\
                                auth_struct), function)(auth_struct, **kwargs)
                        except TypeError, exc:
                            import traceback
                            logging.debug(exc)
                            raise transformer.lib.errors.ServerWrongFunctionPrototype(function_name=function)
                    except transformer.lib.errors.Error, exc:
                        r.code = 500
                        r.data = serializer.serialize_error(context, exc, "en")
                        continue

                    except Exception, exc:
                        import traceback
                        text =traceback.format_exc()
                        logging.debug(text)
                        r.code = 500
                        r.data = str(exc)
                        continue

                except Exception, exc:
                    import traceback
                    text =traceback.format_exc()
                    logging.debug(text)
                    r.code = 500
                    r.data = str(exc)
                    continue

                except transformer.lib.errors.Error, exc :
                    r.code = 500
                    r.data = serializer.serialize_error(context, exc, "en")
                finally:
                    logging.info("Finished dealing with %s" % self.function)

                r.data = serializer.to_string(context, res, with_params)
                r.code = 200

        self.write(serializer.serialize_bundle(requests))
        self.finish()


class Form(tornado.web.RequestHandler):
    def get(self):
        string = '<form method="POST" enctype="multipart/form-data" action="/api/external/2.0/upload_file">'
        string += '<input type=file name=upload>'
        string += '<input type=submit name=press value="OK">'
        string += '</form>'
        return self.write(string)


class Capabilities(tornado.web.RequestHandler):
    def get(self, *args):
        serializer = serializers[JSONSerializer.TYPE]
        if args:
            version = u"v%s" % (args[0])
            return self.write(serializer.to_string(context,
                capabilities[version], None))
        else:
            return self.write(serializer.to_string(context, capabilities, None))


class TransformerServer(object):
    def set_version(self, version):
        global CURRENT_VERSION
        CURRENT_VERSION = version

    def start(self, config={'host': 'localhost', 'port': 9999, 'path': "/home/blegrand/workspace_dev/Json/war", 'version': '3.0'}, callback=None, **kwargs):
        for key in 'host', 'port', 'path', 'version':
            assert 'host' in config, "Missing %s key" % key

        self.callback = callback

        self.PORT = config['port']
        self.HOST = config['host']
        self.PATH = config['path']
        self.set_version(config['version'])
        self.get_version_dispatcher().load_dispatchers()
        urls = [(r"/form", Form)]

        for auth_plugin in _specific_handler.__authentication__.plugins:
            for url in auth_plugin.get_specific_urls():
                urls.append(url)

        for url in self.get_specific_urls():
            urls.append(url)

        for url in self.get_default_urls():
            urls.append(url)

        urls.append((r"/(.*)", tornado.web.StaticFileHandler, {'path':
            self.PATH}))
        application = tornado.web.Application(urls, transforms=self.get_transforms(), **kwargs)
        self.application_ready(application)

    def application_ready(self, application):
        http_server = tornado.httpserver.HTTPServer(application)
        http_server.listen(self.PORT)
        logging.debug("starting server on port %s" % self.PORT)

        ioloop = tornado.ioloop.IOLoop.instance()

        if self.callback:
            # Call the callback every 5 seconds
            scheduler = tornado.ioloop.PeriodicCallback(self.callback, 5000)
            scheduler.start()

        ioloop.add_callback(self.get_start_done_cb())
        ioloop.start()

    def get_start_done_cb(self):
        def do_nothing():
            pass
        return do_nothing

    def get_default_urls(self):

        DefaultDispatcher = self.get_version_dispatcher()
        return [
                (r"/api/get_supported_serialization_protocols", SupportedSerialization),
                (r"/api/external/version", Version),
                (r"/api/external/supported_versions", SupportedVersions),
                (r"/api/external/notify", self.get_notify_handler()),
                (r'/api/external/bundle_request', BundleRequest),
                (r'/api/external/2.0/upload_file', UploadDispatcher),
                (r"/api/external/capabilities", Capabilities),
                (r"/api/external/capabilities/(\d.\d)", Capabilities),
                (r'/api/external/ready', Ready),
                (r'/api/external/async_status', self.get_async_status()),
                (r'/api/external/async_list', self.get_async_list()),
                (r'/api/external/async_delete', self.get_async_delete()),
                (r'/api/local/(\d.\d)/(.*)', DefaultDispatcher),
                (r'/api/local/(.*)', DefaultDispatcher),
                (r'/api/external/(\d.\d)/(.*)', DefaultDispatcher),
                (r'/api/external/(.*)', DefaultDispatcher),
                (r'/api/internal/authenticate/google',
                    self.get_google_authenticator()),
        ]

    def get_specific_urls(self):
        return []

    def get_transforms(self):
        return []

    def get_notify_handler(self):
        return NotifyHandler

    def get_async_status(self):
        class AsyncStatus(tornado.web.RequestHandler):
            def post(self, *args, **kwargs):
                return "{}"
        return AsyncStatus

    def get_async_list(self):
        class AsyncList(tornado.web.RequestHandler):
            def post(self, *args, **kwargs):
                from transformer.lib import ResultSet
                from transformer.lib import AsyncJob
                res = ResultSet(AsyncJob)
                return serializers[JSONSerializer.TYPE].to_string(res)
        return AsyncList

    def get_async_delete(self):
        class AsyncDelete(tornado.web.RequestHandler):
            def post(self, *args, **kwargs):
                return ""
        return AsyncDelete

    def get_google_authenticator(self):
        return AuthenticateWithGoogle

    def get_version_dispatcher(self):
        return VersionDispatcher



if __name__ == "__main__":
    import getopt
    import sys
    try:
        opts, args = getopt.getopt(sys.argv[1:], "h", ["help", ])
    except getopt.error, msg:
        print msg
        print "for help use --help"
        sys.exit(2)

    HOST, PORT = "localhost", 9999
    if len(args) == 0:
        HOST = "localhost"
    elif 1 == len(args):
        HOST = args[0]
    elif 2 == len(args):
        HOST, PORT = args
    else:
        sys.exit(2)

    PORT = int(PORT)
    server = TransformerServer()
    server.start()
