#!/usr/bin/python
# -*- coding: utf-8 -*-

#                         LACIE CONFIDENTIAL
#
# The information contained in this file is the property of LACIE.
# The user of this file shall keep information contained herein confidential
# and shall protect it in whole or in part from disclosure and dissemination
# to third parties.
# Without prior written consent of LACIE, you may not reproduce, represent,
# or download through any means, the information contained herein in any way
# or in any form for a purpose other that the use for internal evaluation,
# operation or maintenance within LACIE.
#
#          (c) LACIE, 2009 - 2012, All Rights Reserved.

#GEN_DATE Jun 18 2013 14:14:51
#
# 888b. .d88b.    8b  8 .d88b. 88888    8888 888b. 888 88888
# 8   8 8P  Y8    8Ybm8 8P  Y8   8      8www 8   8  8    8
# 8   8 8b  d8    8  '8 8b  d8   8      8    8   8  8    8
# 888P' `Y88P'    8   8 `Y88P'   8      8888 888P' 888   8
#
#

from transformer.lib import Optional
from unicorn_api.v3.sv0.multi_nic.Interface import Interface
from unicorn_api.v3.sv0.multi_nic.Bonding import Bonding
from unicorn_api.v3.sv0.multi_nic.IPConfig import IPConfig

from transformer.lib import List

from transformer.client_transport import ClientTransport
from transformer.client_serializer import ClientSerializer


class InterfaceService(object):


    def create_bonding_interface(self, friendly_name, slaves, bonding_config, ip_config, __client__=None):
        """
        When creating a bonding interface, the client must provide a bonding_config and an
        ip_config, otherwise, creation will fail. The bonding interface is then automatically
        enabled, indeed, it makes no sense to create a bonding between all network
        interfaces and not enabling it (as we'll then have no network connectivity at all).
        """
        params = dict()
        params['friendly_name'] = friendly_name
        params['slaves'] = slaves
        params['bonding_config'] = bonding_config
        params['ip_config'] = ip_config
        if __client__:
            ClientTransportSender = __client__
        else:
            ClientTransportSender = ClientTransport

        url = "api/external/3.0/multi_nic.administration.interface_service.create_bonding_interface"
        success, resp = ClientTransportSender.send(url,ClientSerializer.to_string('unicorn_api', params))
        res = ClientSerializer.from_string('unicorn_api', success, resp)
        return res['if_id']

    def create_bonding_interface_async(self, friendly_name, slaves, bonding_config, ip_config, callback, __client__=None):

        if __client__:
            ClientTransportSender = __client__
        else:
            ClientTransportSender = ClientTransport
        params = dict()
        params['friendly_name'] = friendly_name
        params['slaves'] = slaves
        params['bonding_config'] = bonding_config
        params['ip_config'] = ip_config
        url = "api/external/3.0/multi_nic.administration.interface_service.create_bonding_interface"
        ClientTransportSender.async(url,ClientSerializer.to_string('unicorn_api', params), callback, 'unicorn_api')


    def delete_bonding_interface(self, if_id, __client__=None):
        params = dict()
        params['if_id'] = if_id
        if __client__:
            ClientTransportSender = __client__
        else:
            ClientTransportSender = ClientTransport

        url = "api/external/3.0/multi_nic.administration.interface_service.delete_bonding_interface"
        success, resp = ClientTransportSender.send(url,ClientSerializer.to_string('unicorn_api', params))
        res = ClientSerializer.from_string('unicorn_api', success, resp)
        return 

    def delete_bonding_interface_async(self, if_id, callback, __client__=None):

        if __client__:
            ClientTransportSender = __client__
        else:
            ClientTransportSender = ClientTransport
        params = dict()
        params['if_id'] = if_id
        url = "api/external/3.0/multi_nic.administration.interface_service.delete_bonding_interface"
        ClientTransportSender.async(url,ClientSerializer.to_string('unicorn_api', params), callback, 'unicorn_api')


    def get_all_interfaces(self, __client__=None):
        params = dict()

        if __client__:
            ClientTransportSender = __client__
        else:
            ClientTransportSender = ClientTransport

        url = "api/external/3.0/multi_nic.administration.interface_service.get_all_interfaces"
        success, resp = ClientTransportSender.send(url,ClientSerializer.to_string('unicorn_api', params))
        res = ClientSerializer.from_string('unicorn_api', success, resp)
        return res['interfaces']

    def get_all_interfaces_async(self, callback, __client__=None):

        if __client__:
            ClientTransportSender = __client__
        else:
            ClientTransportSender = ClientTransport
        params = dict()

        url = "api/external/3.0/multi_nic.administration.interface_service.get_all_interfaces"
        ClientTransportSender.async(url,ClientSerializer.to_string('unicorn_api', params), callback, 'unicorn_api')


    def get_interface(self, if_id, __client__=None):
        params = dict()
        params['if_id'] = if_id
        if __client__:
            ClientTransportSender = __client__
        else:
            ClientTransportSender = ClientTransport

        url = "api/external/3.0/multi_nic.administration.interface_service.get_interface"
        success, resp = ClientTransportSender.send(url,ClientSerializer.to_string('unicorn_api', params))
        res = ClientSerializer.from_string('unicorn_api', success, resp)
        return res['interface']

    def get_interface_async(self, if_id, callback, __client__=None):

        if __client__:
            ClientTransportSender = __client__
        else:
            ClientTransportSender = ClientTransport
        params = dict()
        params['if_id'] = if_id
        url = "api/external/3.0/multi_nic.administration.interface_service.get_interface"
        ClientTransportSender.async(url,ClientSerializer.to_string('unicorn_api', params), callback, 'unicorn_api')


    def set_dhcp(self, if_id, __client__=None):
        params = dict()
        params['if_id'] = if_id
        if __client__:
            ClientTransportSender = __client__
        else:
            ClientTransportSender = ClientTransport

        url = "api/external/3.0/multi_nic.administration.interface_service.set_dhcp"
        success, resp = ClientTransportSender.send(url,ClientSerializer.to_string('unicorn_api', params))
        res = ClientSerializer.from_string('unicorn_api', success, resp)
        return 

    def set_dhcp_async(self, if_id, callback, __client__=None):

        if __client__:
            ClientTransportSender = __client__
        else:
            ClientTransportSender = ClientTransport
        params = dict()
        params['if_id'] = if_id
        url = "api/external/3.0/multi_nic.administration.interface_service.set_dhcp"
        ClientTransportSender.async(url,ClientSerializer.to_string('unicorn_api', params), callback, 'unicorn_api')


    def set_interface(self, if_id, interface, __client__=None):
        params = dict()
        params['if_id'] = if_id
        params['interface'] = interface
        if __client__:
            ClientTransportSender = __client__
        else:
            ClientTransportSender = ClientTransport

        url = "api/external/3.0/multi_nic.administration.interface_service.set_interface"
        success, resp = ClientTransportSender.send(url,ClientSerializer.to_string('unicorn_api', params))
        res = ClientSerializer.from_string('unicorn_api', success, resp)
        return 

    def set_interface_async(self, if_id, interface, callback, __client__=None):

        if __client__:
            ClientTransportSender = __client__
        else:
            ClientTransportSender = ClientTransport
        params = dict()
        params['if_id'] = if_id
        params['interface'] = interface
        url = "api/external/3.0/multi_nic.administration.interface_service.set_interface"
        ClientTransportSender.async(url,ClientSerializer.to_string('unicorn_api', params), callback, 'unicorn_api')


    def set_ipconfig(self, if_id, ip_config, __client__=None):
        params = dict()
        params['if_id'] = if_id
        params['ip_config'] = ip_config
        if __client__:
            ClientTransportSender = __client__
        else:
            ClientTransportSender = ClientTransport

        url = "api/external/3.0/multi_nic.administration.interface_service.set_ipconfig"
        success, resp = ClientTransportSender.send(url,ClientSerializer.to_string('unicorn_api', params))
        res = ClientSerializer.from_string('unicorn_api', success, resp)
        return res['interface']

    def set_ipconfig_async(self, if_id, ip_config, callback, __client__=None):

        if __client__:
            ClientTransportSender = __client__
        else:
            ClientTransportSender = ClientTransport
        params = dict()
        params['if_id'] = if_id
        params['ip_config'] = ip_config
        url = "api/external/3.0/multi_nic.administration.interface_service.set_ipconfig"
        ClientTransportSender.async(url,ClientSerializer.to_string('unicorn_api', params), callback, 'unicorn_api')
