#!/usr/bin/python
# -*- coding: utf-8 -*-

#                         LACIE CONFIDENTIAL
#
# The information contained in this file is the property of LACIE.
# The user of this file shall keep information contained herein confidential
# and shall protect it in whole or in part from disclosure and dissemination
# to third parties.
# Without prior written consent of LACIE, you may not reproduce, represent,
# or download through any means, the information contained herein in any way
# or in any form for a purpose other that the use for internal evaluation,
# operation or maintenance within LACIE.
#
#          (c) LACIE, 2009 - 2012, All Rights Reserved.

#GEN_DATE May 14 2014 12:22:35
#
# 888b. .d88b.    8b  8 .d88b. 88888    8888 888b. 888 88888
# 8   8 8P  Y8    8Ybm8 8P  Y8   8      8www 8   8  8    8
# 8   8 8b  d8    8  '8 8b  d8   8      8    8   8  8    8
# 888P' `Y88P'    8   8 `Y88P'   8      8888 888P' 888   8
#
#

from transformer.lib import Optional
from unicorn_api.v5.sv0.iscsi.IscsiLimits import IscsiLimits
from unicorn_api.v5.sv0.iscsi.IscsiTarget import IscsiTarget
from unicorn_api.v5.sv0.iscsi.IscsiLun import IscsiLun
from unicorn_api.v5.sv0.iscsi import IscsiLunKind
from unicorn_api.v5.sv0.iscsi.IscsiTargetStatus import IscsiTargetStatus
from unicorn_api.v5.sv0.iscsi.SimpleIscsiTarget import SimpleIscsiTarget

from transformer.lib import ListInfo, ResultSet, Long

from transformer.client_transport import ClientTransport
from transformer.client_serializer import ClientSerializer


class IscsiAdministration(object):


    def add_iqn(self, target_id, iqn, __client__=None):
        params = dict()
        params['target_id'] = target_id
        params['iqn'] = iqn
        if __client__:
            ClientTransportSender = __client__
        else:
            ClientTransportSender = ClientTransport

        url = "api/external/5.0/iscsi.IscsiAdministration.add_iqn"
        success, resp = ClientTransportSender.send(url,ClientSerializer.to_string('unicorn_api', params))
        res = ClientSerializer.from_string('unicorn_api', success, resp)
        return res['isci']

    def add_iqn_async(self, target_id, iqn, callback, __client__=None):

        if __client__:
            ClientTransportSender = __client__
        else:
            ClientTransportSender = ClientTransport
        params = dict()
        params['target_id'] = target_id
        params['iqn'] = iqn
        url = "api/external/5.0/iscsi.IscsiAdministration.add_iqn"
        ClientTransportSender.async(url,ClientSerializer.to_string('unicorn_api', params), callback, 'unicorn_api')


    def attach_lun(self, id, lun, __client__=None):
        params = dict()
        params['id'] = id
        params['lun'] = lun
        if __client__:
            ClientTransportSender = __client__
        else:
            ClientTransportSender = ClientTransport

        url = "api/external/5.0/iscsi.IscsiAdministration.attach_lun"
        success, resp = ClientTransportSender.send(url,ClientSerializer.to_string('unicorn_api', params))
        res = ClientSerializer.from_string('unicorn_api', success, resp)
        return res['target']

    def attach_lun_async(self, id, lun, callback, __client__=None):

        if __client__:
            ClientTransportSender = __client__
        else:
            ClientTransportSender = ClientTransport
        params = dict()
        params['id'] = id
        params['lun'] = lun
        url = "api/external/5.0/iscsi.IscsiAdministration.attach_lun"
        ClientTransportSender.async(url,ClientSerializer.to_string('unicorn_api', params), callback, 'unicorn_api')


    def check_iqn(self, iqn, __client__=None):
        """
         To check validity of an iqn
        """
        params = dict()
        params['iqn'] = iqn
        if __client__:
            ClientTransportSender = __client__
        else:
            ClientTransportSender = ClientTransport

        url = "api/external/5.0/iscsi.IscsiAdministration.check_iqn"
        success, resp = ClientTransportSender.send(url,ClientSerializer.to_string('unicorn_api', params))
        res = ClientSerializer.from_string('unicorn_api', success, resp)
        return res['valid']

    def check_iqn_async(self, iqn, callback, __client__=None):

        if __client__:
            ClientTransportSender = __client__
        else:
            ClientTransportSender = ClientTransport
        params = dict()
        params['iqn'] = iqn
        url = "api/external/5.0/iscsi.IscsiAdministration.check_iqn"
        ClientTransportSender.async(url,ClientSerializer.to_string('unicorn_api', params), callback, 'unicorn_api')


    def create_lun(self, async_client_ref, volume_id, kind, size, __client__=None):
        """
         To create an iSCSI Lun
        """
        params = dict()
        params['async_client_ref'] = async_client_ref
        params['volume_id'] = volume_id
        params['kind'] = kind
        params['size'] = size
        if __client__:
            ClientTransportSender = __client__
        else:
            ClientTransportSender = ClientTransport

        url = "api/external/5.0/iscsi.IscsiAdministration.create_lun"
        success, resp = ClientTransportSender.send(url,ClientSerializer.to_string('unicorn_api', params))
        res = ClientSerializer.from_string('unicorn_api', success, resp)
        return res['async_job_id']


    def create_lun_result(self, async_job_id,__client__=None):
        params = dict()
        params['async_job_id'] = async_job_id

        if __client__:
            ClientTransportSender = __client__
        else:
            ClientTransportSender = ClientTransport

        url = "api/external/5.0/iscsi.IscsiAdministration.create_lun_result"
        success, resp = ClientTransportSender.send(url,ClientSerializer.to_string('unicorn_api', params))
        res = ClientSerializer.from_string('unicorn_api', success, resp)
        return res['lun']



    def create_target(self, iscsi, __client__=None):
        """
         To create an iSCSI Target
        """
        params = dict()
        params['iscsi'] = iscsi
        if __client__:
            ClientTransportSender = __client__
        else:
            ClientTransportSender = ClientTransport

        url = "api/external/5.0/iscsi.IscsiAdministration.create_target"
        success, resp = ClientTransportSender.send(url,ClientSerializer.to_string('unicorn_api', params))
        res = ClientSerializer.from_string('unicorn_api', success, resp)
        return res['iscsi']

    def create_target_async(self, iscsi, callback, __client__=None):

        if __client__:
            ClientTransportSender = __client__
        else:
            ClientTransportSender = ClientTransport
        params = dict()
        params['iscsi'] = iscsi
        url = "api/external/5.0/iscsi.IscsiAdministration.create_target"
        ClientTransportSender.async(url,ClientSerializer.to_string('unicorn_api', params), callback, 'unicorn_api')


    def delete_lun(self, async_client_ref, id, kind, __client__=None):
        """
         To delete an iSCSI Lun
        """
        params = dict()
        params['async_client_ref'] = async_client_ref
        params['id'] = id
        params['kind'] = kind
        if __client__:
            ClientTransportSender = __client__
        else:
            ClientTransportSender = ClientTransport

        url = "api/external/5.0/iscsi.IscsiAdministration.delete_lun"
        success, resp = ClientTransportSender.send(url,ClientSerializer.to_string('unicorn_api', params))
        res = ClientSerializer.from_string('unicorn_api', success, resp)
        return res['async_job_id']


    def delete_lun_result(self, async_job_id,__client__=None):
        params = dict()
        params['async_job_id'] = async_job_id

        if __client__:
            ClientTransportSender = __client__
        else:
            ClientTransportSender = ClientTransport

        url = "api/external/5.0/iscsi.IscsiAdministration.delete_lun_result"
        success, resp = ClientTransportSender.send(url,ClientSerializer.to_string('unicorn_api', params))
        res = ClientSerializer.from_string('unicorn_api', success, resp)
        return 



    def delete_target(self, id, __client__=None):
        """
         To delete an iSCSI Target
        """
        params = dict()
        params['id'] = id
        if __client__:
            ClientTransportSender = __client__
        else:
            ClientTransportSender = ClientTransport

        url = "api/external/5.0/iscsi.IscsiAdministration.delete_target"
        success, resp = ClientTransportSender.send(url,ClientSerializer.to_string('unicorn_api', params))
        res = ClientSerializer.from_string('unicorn_api', success, resp)
        return 

    def delete_target_async(self, id, callback, __client__=None):

        if __client__:
            ClientTransportSender = __client__
        else:
            ClientTransportSender = ClientTransport
        params = dict()
        params['id'] = id
        url = "api/external/5.0/iscsi.IscsiAdministration.delete_target"
        ClientTransportSender.async(url,ClientSerializer.to_string('unicorn_api', params), callback, 'unicorn_api')


    def detach_lun(self, id, lun, __client__=None):
        params = dict()
        params['id'] = id
        params['lun'] = lun
        if __client__:
            ClientTransportSender = __client__
        else:
            ClientTransportSender = ClientTransport

        url = "api/external/5.0/iscsi.IscsiAdministration.detach_lun"
        success, resp = ClientTransportSender.send(url,ClientSerializer.to_string('unicorn_api', params))
        res = ClientSerializer.from_string('unicorn_api', success, resp)
        return res['target']

    def detach_lun_async(self, id, lun, callback, __client__=None):

        if __client__:
            ClientTransportSender = __client__
        else:
            ClientTransportSender = ClientTransport
        params = dict()
        params['id'] = id
        params['lun'] = lun
        url = "api/external/5.0/iscsi.IscsiAdministration.detach_lun"
        ClientTransportSender.async(url,ClientSerializer.to_string('unicorn_api', params), callback, 'unicorn_api')


    def duplicate_lun(self, async_client_ref, volume_id, kind, id, __client__=None):
        """
         To duplicate an iSCSI Lun
         STATUS FUNCTIONS
        """
        params = dict()
        params['async_client_ref'] = async_client_ref
        params['volume_id'] = volume_id
        params['kind'] = kind
        params['id'] = id
        if __client__:
            ClientTransportSender = __client__
        else:
            ClientTransportSender = ClientTransport

        url = "api/external/5.0/iscsi.IscsiAdministration.duplicate_lun"
        success, resp = ClientTransportSender.send(url,ClientSerializer.to_string('unicorn_api', params))
        res = ClientSerializer.from_string('unicorn_api', success, resp)
        return res['async_job_id']


    def duplicate_lun_result(self, async_job_id,__client__=None):
        params = dict()
        params['async_job_id'] = async_job_id

        if __client__:
            ClientTransportSender = __client__
        else:
            ClientTransportSender = ClientTransport

        url = "api/external/5.0/iscsi.IscsiAdministration.duplicate_lun_result"
        success, resp = ClientTransportSender.send(url,ClientSerializer.to_string('unicorn_api', params))
        res = ClientSerializer.from_string('unicorn_api', success, resp)
        return res['lun']



    def export_lun(self, async_client_ref, id, kind, dest_lun_file, __client__=None):
        """
         Export an iSCSI Lun file
        """
        params = dict()
        params['async_client_ref'] = async_client_ref
        params['id'] = id
        params['kind'] = kind
        params['dest_lun_file'] = dest_lun_file
        if __client__:
            ClientTransportSender = __client__
        else:
            ClientTransportSender = ClientTransport

        url = "api/external/5.0/iscsi.IscsiAdministration.export_lun"
        success, resp = ClientTransportSender.send(url,ClientSerializer.to_string('unicorn_api', params))
        res = ClientSerializer.from_string('unicorn_api', success, resp)
        return res['async_job_id']


    def export_lun_result(self, async_job_id,__client__=None):
        params = dict()
        params['async_job_id'] = async_job_id

        if __client__:
            ClientTransportSender = __client__
        else:
            ClientTransportSender = ClientTransport

        url = "api/external/5.0/iscsi.IscsiAdministration.export_lun_result"
        success, resp = ClientTransportSender.send(url,ClientSerializer.to_string('unicorn_api', params))
        res = ClientSerializer.from_string('unicorn_api', success, resp)
        return 



    def get_isns_enabled(self, __client__=None):
        params = dict()

        if __client__:
            ClientTransportSender = __client__
        else:
            ClientTransportSender = ClientTransport

        url = "api/external/5.0/iscsi.IscsiAdministration.get_isns_enabled"
        success, resp = ClientTransportSender.send(url,ClientSerializer.to_string('unicorn_api', params))
        res = ClientSerializer.from_string('unicorn_api', success, resp)
        return res['enabled']

    def get_isns_enabled_async(self, callback, __client__=None):

        if __client__:
            ClientTransportSender = __client__
        else:
            ClientTransportSender = ClientTransport
        params = dict()

        url = "api/external/5.0/iscsi.IscsiAdministration.get_isns_enabled"
        ClientTransportSender.async(url,ClientSerializer.to_string('unicorn_api', params), callback, 'unicorn_api')


    def get_isns_server(self, __client__=None):
        params = dict()

        if __client__:
            ClientTransportSender = __client__
        else:
            ClientTransportSender = ClientTransport

        url = "api/external/5.0/iscsi.IscsiAdministration.get_isns_server"
        success, resp = ClientTransportSender.send(url,ClientSerializer.to_string('unicorn_api', params))
        res = ClientSerializer.from_string('unicorn_api', success, resp)
        return res['isns_server']

    def get_isns_server_async(self, callback, __client__=None):

        if __client__:
            ClientTransportSender = __client__
        else:
            ClientTransportSender = ClientTransport
        params = dict()

        url = "api/external/5.0/iscsi.IscsiAdministration.get_isns_server"
        ClientTransportSender.async(url,ClientSerializer.to_string('unicorn_api', params), callback, 'unicorn_api')


    def get_limits(self, __client__=None):
        params = dict()

        if __client__:
            ClientTransportSender = __client__
        else:
            ClientTransportSender = ClientTransport

        url = "api/external/5.0/iscsi.IscsiAdministration.get_limits"
        success, resp = ClientTransportSender.send(url,ClientSerializer.to_string('unicorn_api', params))
        res = ClientSerializer.from_string('unicorn_api', success, resp)
        return res['limits']

    def get_limits_async(self, callback, __client__=None):

        if __client__:
            ClientTransportSender = __client__
        else:
            ClientTransportSender = ClientTransport
        params = dict()

        url = "api/external/5.0/iscsi.IscsiAdministration.get_limits"
        ClientTransportSender.async(url,ClientSerializer.to_string('unicorn_api', params), callback, 'unicorn_api')


    def import_lun(self, async_client_ref, volume_id, kind, src_lun_file, __client__=None):
        """
         To create an iSCSI Lun
        """
        params = dict()
        params['async_client_ref'] = async_client_ref
        params['volume_id'] = volume_id
        params['kind'] = kind
        params['src_lun_file'] = src_lun_file
        if __client__:
            ClientTransportSender = __client__
        else:
            ClientTransportSender = ClientTransport

        url = "api/external/5.0/iscsi.IscsiAdministration.import_lun"
        success, resp = ClientTransportSender.send(url,ClientSerializer.to_string('unicorn_api', params))
        res = ClientSerializer.from_string('unicorn_api', success, resp)
        return res['async_job_id']


    def import_lun_result(self, async_job_id,__client__=None):
        params = dict()
        params['async_job_id'] = async_job_id

        if __client__:
            ClientTransportSender = __client__
        else:
            ClientTransportSender = ClientTransport

        url = "api/external/5.0/iscsi.IscsiAdministration.import_lun_result"
        success, resp = ClientTransportSender.send(url,ClientSerializer.to_string('unicorn_api', params))
        res = ClientSerializer.from_string('unicorn_api', success, resp)
        return res['lun']



    def list_initiator(self, list_info, __client__=None):
        """
         To list all iSCSI initiators connected to target
         IQN MANAGEMENT
        """
        params = dict()
        params['list_info'] = list_info
        if __client__:
            ClientTransportSender = __client__
        else:
            ClientTransportSender = ClientTransport

        url = "api/external/5.0/iscsi.IscsiAdministration.list_initiator"
        success, resp = ClientTransportSender.send(url,ClientSerializer.to_string('unicorn_api', params))
        res = ClientSerializer.from_string('unicorn_api', success, resp)
        return res['initiators']

    def list_initiator_async(self, list_info, callback, __client__=None):

        if __client__:
            ClientTransportSender = __client__
        else:
            ClientTransportSender = ClientTransport
        params = dict()
        params['list_info'] = list_info
        url = "api/external/5.0/iscsi.IscsiAdministration.list_initiator"
        ClientTransportSender.async(url,ClientSerializer.to_string('unicorn_api', params), callback, 'unicorn_api')


    def list_lun(self, listInfo, __client__=None):
        """
         To list all iSCSI Luns
        """
        params = dict()
        params['listInfo'] = listInfo
        if __client__:
            ClientTransportSender = __client__
        else:
            ClientTransportSender = ClientTransport

        url = "api/external/5.0/iscsi.IscsiAdministration.list_lun"
        success, resp = ClientTransportSender.send(url,ClientSerializer.to_string('unicorn_api', params))
        res = ClientSerializer.from_string('unicorn_api', success, resp)
        return res['luns']

    def list_lun_async(self, listInfo, callback, __client__=None):

        if __client__:
            ClientTransportSender = __client__
        else:
            ClientTransportSender = ClientTransport
        params = dict()
        params['listInfo'] = listInfo
        url = "api/external/5.0/iscsi.IscsiAdministration.list_lun"
        ClientTransportSender.async(url,ClientSerializer.to_string('unicorn_api', params), callback, 'unicorn_api')


    def list_target(self, listInfo, __client__=None):
        """
         To list all iSCSI Targets
        """
        params = dict()
        params['listInfo'] = listInfo
        if __client__:
            ClientTransportSender = __client__
        else:
            ClientTransportSender = ClientTransport

        url = "api/external/5.0/iscsi.IscsiAdministration.list_target"
        success, resp = ClientTransportSender.send(url,ClientSerializer.to_string('unicorn_api', params))
        res = ClientSerializer.from_string('unicorn_api', success, resp)
        return res['iscsis']

    def list_target_async(self, listInfo, callback, __client__=None):

        if __client__:
            ClientTransportSender = __client__
        else:
            ClientTransportSender = ClientTransport
        params = dict()
        params['listInfo'] = listInfo
        url = "api/external/5.0/iscsi.IscsiAdministration.list_target"
        ClientTransportSender.async(url,ClientSerializer.to_string('unicorn_api', params), callback, 'unicorn_api')


    def modify(self, id, isci, __client__=None):
        """
         To modify advanced parameters of an iSCSI
         (Header digest, Data digest, CHAP, Multual CHAP)
         LUN FUNCTIONS
        """
        params = dict()
        params['id'] = id
        params['isci'] = isci
        if __client__:
            ClientTransportSender = __client__
        else:
            ClientTransportSender = ClientTransport

        url = "api/external/5.0/iscsi.IscsiAdministration.modify"
        success, resp = ClientTransportSender.send(url,ClientSerializer.to_string('unicorn_api', params))
        res = ClientSerializer.from_string('unicorn_api', success, resp)
        return res['isci']

    def modify_async(self, id, isci, callback, __client__=None):

        if __client__:
            ClientTransportSender = __client__
        else:
            ClientTransportSender = ClientTransport
        params = dict()
        params['id'] = id
        params['isci'] = isci
        url = "api/external/5.0/iscsi.IscsiAdministration.modify"
        ClientTransportSender.async(url,ClientSerializer.to_string('unicorn_api', params), callback, 'unicorn_api')


    def remove_iqn(self, target_id, iqn, __client__=None):
        """
         ISNS FUNCTIONS
        """
        params = dict()
        params['target_id'] = target_id
        params['iqn'] = iqn
        if __client__:
            ClientTransportSender = __client__
        else:
            ClientTransportSender = ClientTransport

        url = "api/external/5.0/iscsi.IscsiAdministration.remove_iqn"
        success, resp = ClientTransportSender.send(url,ClientSerializer.to_string('unicorn_api', params))
        res = ClientSerializer.from_string('unicorn_api', success, resp)
        return res['isci']

    def remove_iqn_async(self, target_id, iqn, callback, __client__=None):

        if __client__:
            ClientTransportSender = __client__
        else:
            ClientTransportSender = ClientTransport
        params = dict()
        params['target_id'] = target_id
        params['iqn'] = iqn
        url = "api/external/5.0/iscsi.IscsiAdministration.remove_iqn"
        ClientTransportSender.async(url,ClientSerializer.to_string('unicorn_api', params), callback, 'unicorn_api')


    def set_isns_enabled(self, enabled, __client__=None):
        params = dict()
        params['enabled'] = enabled
        if __client__:
            ClientTransportSender = __client__
        else:
            ClientTransportSender = ClientTransport

        url = "api/external/5.0/iscsi.IscsiAdministration.set_isns_enabled"
        success, resp = ClientTransportSender.send(url,ClientSerializer.to_string('unicorn_api', params))
        res = ClientSerializer.from_string('unicorn_api', success, resp)
        return res['enabled']

    def set_isns_enabled_async(self, enabled, callback, __client__=None):

        if __client__:
            ClientTransportSender = __client__
        else:
            ClientTransportSender = ClientTransport
        params = dict()
        params['enabled'] = enabled
        url = "api/external/5.0/iscsi.IscsiAdministration.set_isns_enabled"
        ClientTransportSender.async(url,ClientSerializer.to_string('unicorn_api', params), callback, 'unicorn_api')


    def set_isns_server(self, isns_server, __client__=None):
        params = dict()
        params['isns_server'] = isns_server
        if __client__:
            ClientTransportSender = __client__
        else:
            ClientTransportSender = ClientTransport

        url = "api/external/5.0/iscsi.IscsiAdministration.set_isns_server"
        success, resp = ClientTransportSender.send(url,ClientSerializer.to_string('unicorn_api', params))
        res = ClientSerializer.from_string('unicorn_api', success, resp)
        return res['isns_server']

    def set_isns_server_async(self, isns_server, callback, __client__=None):

        if __client__:
            ClientTransportSender = __client__
        else:
            ClientTransportSender = ClientTransport
        params = dict()
        params['isns_server'] = isns_server
        url = "api/external/5.0/iscsi.IscsiAdministration.set_isns_server"
        ClientTransportSender.async(url,ClientSerializer.to_string('unicorn_api', params), callback, 'unicorn_api')
